/************************************************************
* Este es el fichero de carga de los procedimientos de la BD
* No incluye los procedimientos CRUD/ABM
*
* Fecha creacin: 23/03/2021
* 
* TFG: Creacin base de datos de procesos electorales.
*
* Autor: Francisco Jos Martnez del Amo
* Tutor: Jordi Ferrer Durn
************************************************************/

/************************************************************
* Procedimiento para dar de alta demarcaciones electorales en 
* funcin del mbito del Proceso Electoral a la hora de aplicar 
* unas reglas determinadas.
*   Ambito General: Cargar demarcaciones por provincias
*   mbito Autonmico: Demarcaciones las provincias de la CCAA
*   mbito Local: demarcacin unica, el municipio.
* Entrada: ID_Proceso, lugar
* RSP salida: OK si se realiza con xito
*             ERROR+TIPO DE ERROR en caso contrario
* Salida: Nmero de demarcaciones creadas.
* Pre-condicin: el proceso electoral debe existir.
* Post-condicin: se crean las demarcaciones de ese proceso
*                 con los correspondientes escaos.
************************************************************/
CREATE OR REPLACE PROCEDURE crearDemarcaciones(
	P_IDProceso IN ProcesoElectoral.ID_Proceso%TYPE,
    P_lugar IN Localidad.nombreLoc%TYPE,
    paramSalida OUT VARCHAR2,
	RSP OUT VARCHAR2) 
IS
    paramEntrada VARCHAR2(500);
	numERROR INTEGER;
	tipoERROR VARCHAR2(100);
    VACIO_ERROR EXCEPTION;
    procesoExiste ProcesoElectoral.ID_Proceso%TYPE;
    P_escanos INTEGER;
    P_Ambito ProcesoElectoral.ambito%TYPE;
    P_nombre DemarcacionElectoral.nombreDemarcacion%TYPE;
    
BEGIN
    paramEntrada:= P_IDProceso||' - lugar de realizacin: '||P_lugar;
	
    /* Comprobamos que el ID no sea nulo */
    IF (P_IDProceso IS NULL)
    THEN
        RAISE VACIO_ERROR;
    END IF;  
    
    /* Comprobamos que exista el Proceso Electoral  */
    BEGIN
        SELECT ID_Proceso, ambito
        INTO procesoExiste, P_ambito
        FROM ProcesoElectoral
        WHERE ID_Proceso = P_IDProceso
        AND ROWNUM = 1;
        EXCEPTION
            WHEN NO_DATA_FOUND THEN
            RAISE NO_DATA_FOUND;
    END;
        
    /* Si ha pasado las comprobaciones anteriores, creamos la demarcacin */
    IF (P_ambito = 'Local')
    THEN
        P_nombre:= 'Demarcacin electoral de '||P_lugar;
        
        BEGIN
            SELECT escanosLoc
            INTO P_escanos
            FROM Localidad 
            WHERE nombreLoc = P_lugar
            AND ROWNUM = 1;
            EXCEPTION
                WHEN NO_DATA_FOUND THEN
                RAISE NO_DATA_FOUND;
        END;
        
        DECLARE
        paramSalida VARCHAR2(500);
        RSP VARCHAR2(500);
        BEGIN
        altaDemarcacionElectoral(Sec_DemarcacionElectoral.NEXTVAL, P_nombre, P_escanos, P_IDProceso, RSP);
        END;
        
        paramSalida:= 1;
    END IF;
    
    RSP:= 'OK';
    COMMIT;
    INSERT INTO LogOperaciones VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, paramSalida, RSP);
    COMMIT;
    
EXCEPTION
    WHEN VACIO_ERROR THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL VALOR DEL PROCEDIMEINTO NO PUEDE SER NULO';
    ROLLBACK;
    INSERT INTO LogOperaciones VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, paramSalida, RSP);
    COMMIT;
    
    WHEN NO_DATA_FOUND THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL PROCEDIMIENTO NO SE ENCUENTRA - NO EXISTE';
    ROLLBACK;
    INSERT INTO LogOperaciones VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, paramSalida, RSP);
    COMMIT;
    
    WHEN OTHERS THEN
    numERROR:= SQLCODE;
    tipoERROR:= SUBSTR(SQLERRM, 1, 90);
    RSP := TO_CHAR(numERROR)||' - '||tipoERROR;
    ROLLBACK;
    INSERT INTO LogOperaciones VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, paramSalida, RSP);
    COMMIT;
END;
/


/************************************************************
* Procedimiento para dar de alta Centros Electorales en 
* funcin de la poblacin determinada.
* En este caso organiza un Centro Electoral cada 20.000 personas
*
* Entrada: ID_Demarcacion, ID_Provincia, ID_Localidad
* RSP salida: OK si se realiza con xito
*             ERROR+TIPO DE ERROR en caso contrario
* Salida: Nmero de centros creados, de mesas, total de personas 
*         y nombre de la localidad
* Pre-condicin: La demarcacin y la ciudad deben existir.
* Post-condicin: se crean las Centros Electorales en esa poblacin.
************************************************************/
CREATE OR REPLACE PROCEDURE crearCentrosCiudad(
	P_IDDemarcacion IN DemarcacionElectoral.ID_Demarcacion%TYPE,
    P_Provincia IN Localidad.ID_Provincia%TYPE,
	P_Localidad IN Localidad.ID_Localidad%TYPE,
    paramSalida OUT VARCHAR2,
	RSP OUT VARCHAR2) 
IS
    paramEntrada VARCHAR2(500);
    P_nombre CentroVotacion.nombreCentro%TYPE;
    P_direccion CentroVotacion.direccionCentro%TYPE;
    P_nombreMesa MesaElectoral.nombreMesa%TYPE;
	numERROR INTEGER;
	tipoERROR VARCHAR2(100);
    VACIO_ERROR EXCEPTION;
	demarcacionNULL DemarcacionElectoral.ID_Demarcacion%TYPE;
    provinciaNULL Localidad.ID_Provincia%TYPE;
    localidadNULL Localidad.ID_Localidad%TYPE;
	nombreCiudad Localidad.nombreLoc%TYPE;
    totalPersonas INTEGER;
    bucle INTEGER;
    bucleMesa INTEGER;
    P_totalCentros INTEGER;
    P_totalMesas INTEGER;
    
BEGIN
    paramEntrada:= P_IDDemarcacion||' - lugar de realizacin: '||P_Provincia||P_Localidad;
	
    /* Comprobamos que la demarcacin y la localidad no sean nulos */
    IF (P_IDDemarcacion IS NULL) OR (P_Provincia IS NULL) OR (P_Localidad IS NULL)
    THEN
        RAISE VACIO_ERROR;
    END IF;  
    
    /* Comprobamos que exista la demarcacin  */
    BEGIN
        SELECT ID_Demarcacion
        INTO demarcacionNULL
        FROM DemarcacionElectoral
        WHERE ID_Demarcacion = P_IDDemarcacion
        AND ROWNUM = 1;
        EXCEPTION
            WHEN NO_DATA_FOUND THEN
            RAISE NO_DATA_FOUND;
    END;
	
	/* Comprobamos que exista la localidad y seleccionamos su nombre */
    BEGIN
        SELECT ID_Provincia, ID_Localidad, nombreLoc
        INTO provinciaNULL, localidadNULL, nombreCiudad
        FROM Localidad
        WHERE ID_Provincia = P_Provincia
		AND ID_Localidad = P_Localidad
        AND ROWNUM = 1;
        EXCEPTION
            WHEN NO_DATA_FOUND THEN
            RAISE NO_DATA_FOUND;
    END;
        
    /* Si ha pasado las comprobaciones anteriores, creamos los centros */
	
    /* Calculamos cuantos centros necesitamos segn la poblacin, uno cada 20.000 */
    SELECT count(*)
    INTO totalPersonas
    FROM Direccion 
    WHERE nombreCiudad = EntidadSingular;
        
	DECLARE
	paramSalida VARCHAR2(500);
	RSP VARCHAR2(500);
	BEGIN
		bucle:= TRUNC(totalPersonas/20000)+1;
        P_totalCentros:=0;
		P_totalMesas:=0;
		WHILE (bucle > 0) LOOP
			P_nombre:= 'Centro '||bucle;
			P_direccion:= 'Calle de '||P_nombre;
			altaCentroVotacion(Sec_DemarcacionElectoral.NEXTVAL, P_nombre, P_direccion, P_Provincia, P_Localidad, P_IDDemarcacion, RSP);
                /* creamos Mesas Electorales para cada centro en funcion de 1 mesa cada 2000 personas */
                IF (bucle>1)
                THEN
                    bucleMesa:=10;
                ELSE
                    bucleMesa:=TRUNC(totalPersonas/2000)+1-(TRUNC(totalPersonas/20000)*10);
                END IF;  
                
                WHILE (bucleMesa > 0) LOOP
                    P_nombreMesa:='Mesa '||bucleMesa; 
                    altaMesaElectoral(Sec_MesaElectoral.NEXTVAL,P_nombreMesa, Sec_DemarcacionElectoral.CURRVAL, RSP);
                    bucleMesa:=bucleMesa-1;
                    P_totalMesas:=P_totalMesas+1;
                END LOOP;
			bucle:= bucle-1;
			P_totalCentros:= P_totalCentros+1;
		END LOOP;
	END;
    paramSalida:= 'Centros: '||P_totalCentros||' - Mesas: '||P_totalMesas||' - Personas: '||totalPersonas||' - Ciudad: '||nombreCiudad; 
    RSP:= 'OK';
    COMMIT;
    INSERT INTO LogOperaciones VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, paramSalida, RSP);
    COMMIT;
    
EXCEPTION
    WHEN VACIO_ERROR THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL VALOR NO PUEDE SER NULO';
    ROLLBACK;
    INSERT INTO LogOperaciones VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, paramSalida, RSP);
    COMMIT;
    
    WHEN NO_DATA_FOUND THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL VALOR NO SE ENCUENTRA - NO EXISTE';
    ROLLBACK;
    INSERT INTO LogOperaciones VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, paramSalida, RSP);
    COMMIT;
    
    WHEN OTHERS THEN
    numERROR:= SQLCODE;
    tipoERROR:= SUBSTR(SQLERRM, 1, 90);
    RSP := TO_CHAR(numERROR)||' - '||tipoERROR;
    ROLLBACK;
    INSERT INTO LogOperaciones VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, paramSalida, RSP);
    COMMIT;
END;
/


/************************************************************
* Procedimiento para dar de la comunicacin BOE de un proceso  
* electoral de cualquier tipo de mbito.
*
* Entrada: ID_Proceso
* RSP salida: OK si se realiza con xito
*             ERROR+TIPO DE ERROR en caso contrario
* Salida: ninguna.
* Pre-condicin: el proceso electoral, las reglas de los partidos, 
* el mtodo de clculo, demarcaciones y escaos deben existir.
* Post-condicin: se crean las Mesas Electorales en esa poblacin.
************************************************************/

CREATE OR REPLACE PROCEDURE crearComBOE(
	PIDProceso IN ProcesoElectoral.ID_Proceso%TYPE,
    paramSalida OUT VARCHAR2,
	RSP OUT VARCHAR2) 
IS
    paramEntrada VARCHAR2(500);
	numERROR INTEGER;
	tipoERROR VARCHAR2(100);
    VACIO_ERROR EXCEPTION;
	procesoNULL ProcesoElectoral.ID_Proceso%TYPE;
	P_ambito ProcesoElectoral.ambito%TYPE;
	P_fecha ProcesoElectoral.fechaElecciones%TYPE;
	P_IDGobierno ProcesoElectoral.ID_Gobierno%TYPE;
	P_demarcacion VARCHAR2(1000);
	P_escanosNum VARCHAR2(1000);
	P_metodo MetodoCalculo.nombreMetodo%TYPE;
	P_Reglas VARCHAR2(1000);
    
BEGIN
    paramEntrada:= 'Proceso Electoral: '||PIDProceso;
	
    /* Comprobamos que el proceso electoral no sea nulo */
    IF (PIDProceso IS NULL) 
    THEN
        RAISE VACIO_ERROR;
    END IF;  
	
	/* Comprobamos que exista el proceso  */
    BEGIN
        SELECT ID_Proceso
        INTO procesoNULL
        FROM ProcesoElectoral
        WHERE ID_Proceso = PIDProceso
        AND ROWNUM = 1;
        EXCEPTION
            WHEN NO_DATA_FOUND THEN
            RAISE NO_DATA_FOUND;
    END;
    
	/* calculamos el mbito, fecha electoral y gobierno */
	SELECT ambito, fechaElecciones, ID_Gobierno
	INTO P_ambito, P_fecha, P_IDGobierno
	FROM ProcesoElectoral 
	WHERE ID_Proceso = PIDProceso; 
	
	/* Seleccionamos las demarcaciones, las agrupamos y eliminamos repetidas */  
    SELECT Demarcaciones
	INTO P_demarcacion
    FROM (SELECT ID_Proceso, REGEXP_REPLACE(LISTAGG(nombreDemarcacion, ',') 
        WITHIN GROUP (ORDER BY nombreDemarcacion, ID_Proceso),'([^,]+)(,\1)*(,|$)', '\1\3') As Demarcaciones
        FROM DemarcacionElectoral
        GROUP BY ID_Proceso
        HAVING ID_Proceso = PIDProceso);
    
	/* Seleccionamos los escaos los agrupamos y eliminamos repetidas */   
    SELECT Escanos
	INTO P_escanosNum
	FROM (SELECT ID_Proceso, REGEXP_REPLACE(LISTAGG(numEscanos, ',') 
        WITHIN GROUP (ORDER BY numEscanos, ID_Proceso),'([^,]+)(,\1)*(,|$)', '\1\3') As Escanos
        FROM DemarcacionElectoral
        GROUP BY ID_Proceso
        HAVING ID_Proceso = PIDProceso);
    
	/* Seleccionamos las reglas de los partidos, las agrupamos y eliminamos repetidas  */
    SELECT ReglasPartidos
	INTO P_Reglas
	FROM (SELECT ID_Proceso, REGEXP_REPLACE(LISTAGG(ID_RPP, ',') 
    WITHIN GROUP (ORDER BY ID_RPP, ID_Proceso),'([^,]+)(,\1)*(,|$)', '\1\3') AS ReglasPartidos
		FROM ParticipaEn 
		GROUP BY ID_Proceso
        HAVING ID_Proceso = PIDProceso);
    
	/* Seleccionamos el mtodo de clculo */
	SELECT nombreMetodo
	INTO P_metodo
	FROM MetodoCalculo
	WHERE ID_Metodo = 1; 
	
	/* Creamos la comunicacin en el BOE */
	altaComunicacionBOE(Sec_ComunicacionBOE.NEXTVAL,P_ambito, P_fecha, P_Reglas, P_metodo, P_demarcacion, P_escanosNum, P_IDGobierno, RSP);
    RSP:= 'OK';
    COMMIT;
    INSERT INTO LogOperaciones VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, paramSalida, RSP);
    COMMIT;
    
EXCEPTION
    WHEN VACIO_ERROR THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL VALOR NO PUEDE SER NULO';
    ROLLBACK;
    INSERT INTO LogOperaciones VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, paramSalida, RSP);
    COMMIT;
    
    WHEN NO_DATA_FOUND THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL VALOR NO SE ENCUENTRA - NO EXISTE';
    ROLLBACK;
    INSERT INTO LogOperaciones VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, paramSalida, RSP);
    COMMIT;
    
    WHEN OTHERS THEN
    numERROR:= SQLCODE;
    tipoERROR:= SUBSTR(SQLERRM, 1, 90);
    RSP := TO_CHAR(numERROR)||' - '||tipoERROR;
    ROLLBACK;
    INSERT INTO LogOperaciones VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, paramSalida, RSP);
    COMMIT;
END;
/


/************************************************************
* Procedimiento para dar de alta un Censo Electoral en funcin
* del mbito del Proceso Electoral a la hora de aplicar unas
* reglas determinadas.
*
* Ambito General: Reglas mayora de edad y nacionalidad espaola
* mbito Autonmico: Edad, nacionalidad y empadronamiento
* mbito Local: Edad, nacionalidad Europea y empadronamiento
*
* Entrada: ID_Proceso
* RSP salida: OK si se realiza con xito
*             ERROR+TIPO DE ERROR en caso contrario
* Salida: Nmero de personas insertadas, Nmero de personas posibles 
*         y nmero de fallos
* Pre-condicin: el proceso electoral, las reglas y las personas deben existir.
* Post-condicin: se crea el censo electoral de ese proceso.
************************************************************/

CREATE OR REPLACE PROCEDURE crearCenso(
	P_IDProceso IN ProcesoElectoral.ID_Proceso%TYPE,
    paramSalida OUT VARCHAR2,
	RSP OUT VARCHAR2) 
IS
    paramEntrada VARCHAR2(500);
	numERROR INTEGER;
	tipoERROR VARCHAR2(100);
    VACIO_ERROR EXCEPTION;
    procesoExiste ProcesoElectoral.ID_Proceso%TYPE;
    P_centroNum INTEGER;
    P_ambito ProcesoElectoral.ambito%TYPE;
    P_fecha ProcesoElectoral.fechaElecciones%TYPE;
    P_centro CentroVotacion.ID_Centro%TYPE;
    P_mesa MesaElectoral.ID_Mesa%TYPE;
    P_demarcacion DemarcacionElectoral.ID_Demarcacion%TYPE;
	totalInserciones NUMBER;
	totalPersonas NUMBER;
	totalFallos NUMBER;
	cuentaCentro INTEGER;
	cuentaMesa INTEGER;
    
BEGIN
    paramEntrada:= P_IDProceso;
	
    /* Comprobamos que el ID no sea nulo */
    IF (P_IDProceso IS NULL)
    THEN
        RAISE VACIO_ERROR;
    END IF;  
    
    /* Comprobamos que exista el Proceso Electoral */ 
    BEGIN
        SELECT ID_Proceso
        INTO procesoExiste
        FROM ProcesoElectoral
        WHERE ID_Proceso = P_IDProceso
        AND ROWNUM = 1;
        EXCEPTION
            WHEN NO_DATA_FOUND THEN
            RAISE NO_DATA_FOUND;
    END;
    
    /* seleccionamos datos del proceso para el alta */
    SELECT ambito, fechaElecciones
    INTO P_ambito, P_fecha
    FROM ProcesoElectoral
    WHERE ID_Proceso = P_IDProceso;
        
    /* Si ha pasado las comprobaciones anteriores, inscribimos en el censo */   
    IF (P_ambito = 'General')
    THEN   
		/* Pendiende de realizar para cuando se haga un ejemplo de votacin general */
        /* Realizamos una inscripcin por persona */
        altaCensoElectoral (1, 1, 1, P_IDProceso, P_fecha, 'S', 30, 29, NULL, RSP);
        
    ELSIF (P_ambito = 'Local') THEN

        /* Seleccionamos la Demarcacin */
        SELECT Id_Demarcacion
        INTO P_demarcacion
        FROM DemarcacionElectoral
        WHERE ID_Proceso = P_IDProceso;
		
		/* Seleccionamos el centro de votacin inicial */
		SELECT ID_Centro
		INTO P_centro
		FROM CentroVotacion
		WHERE ID_Demarcacion = P_demarcacion
		AND ROWNUM = 1
		ORDER BY ID_Centro;
		
		/* Seleccionamos la Mesa */
        SELECT Id_Mesa
        INTO P_mesa
        FROM MesaElectoral
        WHERE ID_Centro = P_centro
		AND ROWNUM = 1
		ORDER BY Id_Mesa;
        
		/* mbito Local: Edad, nacionalidad Europea y empadronamiento */
		totalInserciones:= 0;
        cuentaCentro:= 0;
        cuentaMesa:= 0;
		
		SELECT COUNT(*)
        INTO totalPersonas
        FROM Persona;
		
        DECLARE
        CURSOR C_personas IS
            SELECT numDoc, letraDoc 
            FROM Persona
            WHERE municipio='Zaragoza'
            AND (months_between(SYSDATE,FECHANAC)/12)>17
            AND NACIONALIDAD IN ('Espaola','Europea');
        BEGIN
			FOR p IN C_personas
            LOOP
            /* realizamos la inscripcin en el Censo Electoral */
            
			altaCensoElectoral (p.numDoc, p.letraDoc, '1, 3, 4', P_IDProceso, P_fecha, 'S', P_mesa, P_centro, NULL, RSP);
			totalInserciones:= totalInserciones+1;
			cuentaCentro:= cuentaCentro+1;
			cuentaMesa:= cuentaMesa+1;
            
			/* actualizamos Centro */
			IF (cuentaCentro = 20000)
			THEN
				P_centro:=P_centro+1;
                cuentaCentro:= 0;
			END IF; 
            
			/* actualizamos Mesa */
			IF (cuentaMesa = 2000)
			THEN
				P_mesa:=P_mesa+1;
                cuentaMesa:= 0;
			END IF;  
			
            END LOOP;
        END;
        
        /* seleccionamos el centro de votacin */

    ELSE 
		/* Pendiende de realizar para cuando se haga un ejemplo de votacin autonomica */
        /* Realizamos una inscripcin por persona */
        altaCensoElectoral (1, 1, 1, P_IDProceso, P_fecha, 'S', 30, 29, NULL, RSP);
        
    END IF;
	
	/* Contamos los fallos */
	SELECT COUNT(*)
	INTO totalFallos
	FROM CensoElectoral
	WHERE ID_Proceso = P_IDProceso;
	totalFallos:= totalInserciones-totalFallos;
	
    paramSalida:='Total Censados: '||totalInserciones||' de un total de '||totalPersonas||' personas'||' y fallos: '||totalFallos;
    RSP:= 'OK';
    COMMIT;
    INSERT INTO LogOperaciones VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, paramSalida, RSP);
    COMMIT;
   
EXCEPTION
    WHEN VACIO_ERROR THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL VALOR DEL PROCEDIMEINTO NO PUEDE SER NULO';
    ROLLBACK;
    INSERT INTO LogOperaciones VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, paramSalida, RSP);
    COMMIT;
    
    WHEN NO_DATA_FOUND THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL PROCEDIMIENTO NO SE ENCUENTRA - NO EXISTE';
    ROLLBACK;
    INSERT INTO LogOperaciones VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, paramSalida, RSP);
    COMMIT;
    
    WHEN OTHERS THEN
    numERROR:= SQLCODE;
    tipoERROR:= SUBSTR(SQLERRM, 1, 90);
    RSP := TO_CHAR(numERROR)||' - '||tipoERROR;
    ROLLBACK;
    INSERT INTO LogOperaciones VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, PARAMsALIDA, RSP);
    COMMIT;
END;
/


/************************************************************
* Procedimiento para enviar las comunicaciones a los votantes
* en un procedimiento electoral.
*
* Entrada: ID_Proceso
* RSP salida: OK si se realiza con xito
*             ERROR+TIPO DE ERROR en caso contrario
* Salida: Nmero de comunicaciones realizadas, Nmero de personas  
*         censadas y nmero de fallos
* Pre-condicin: el proceso electoral, las reglas y las personas deben existir.
* Post-condicin: se crea el censo electoral de ese proceso.
************************************************************/

CREATE OR REPLACE PROCEDURE crearComVot(
	P_IDProceso IN ProcesoElectoral.ID_Proceso%TYPE,
    paramSalida OUT VARCHAR2,
	RSP OUT VARCHAR2) 
IS
    paramEntrada VARCHAR2(500);
	numERROR INTEGER;
	tipoERROR VARCHAR2(100);
    VACIO_ERROR EXCEPTION;
    procesoExiste ProcesoElectoral.ID_Proceso%TYPE;
	
    P_centro CentroVotacion.ID_Centro%TYPE;
    P_mesa MesaElectoral.ID_Mesa%TYPE;

	totalComunicaciones NUMBER;
	totalCensadas NUMBER;
	totalFallos NUMBER;

    
BEGIN
    paramEntrada:= P_IDProceso;
	
    /* Comprobamos que el ID no sea nulo */
    IF (P_IDProceso IS NULL)
    THEN
        RAISE VACIO_ERROR;
    END IF;  
    
    /* Comprobamos que exista el Proceso Electoral */ 
    BEGIN
        SELECT ID_Proceso
        INTO procesoExiste
        FROM ProcesoElectoral
        WHERE ID_Proceso = P_IDProceso
        AND ROWNUM = 1;
        EXCEPTION
            WHEN NO_DATA_FOUND THEN
            RAISE NO_DATA_FOUND;
    END;
    
    /* Si ha pasado las comprobaciones anteriores, creamos las comunicaciones */   

	totalComunicaciones:= 0;
	
	SELECT COUNT(*)
	INTO totalCensadas
	FROM CensoElectoral
	WHERE ID_Proceso = P_IDProceso;
	
	DECLARE
	CURSOR C_comunicaciones IS
		SELECT numDoc, letraDoc, mesaElectoral, centroVotacion 
		FROM CensoElectoral
		WHERE ID_Proceso = P_IDProceso;
	BEGIN
		FOR p IN C_comunicaciones
		LOOP
		/* realizamos la inscripcin en el Censo Electoral */
		altaComunicacionVotantes (Sec_ComunicacionVotantes.NEXTVAL, p.numDoc, p.letraDoc, p.mesaElectoral, p.centroVotacion,'Aqui van las instrucciones del voto y del voto telemtico.', P_IDProceso, RSP);
		totalComunicaciones:= totalComunicaciones+1;
		END LOOP;
	END;
	
	/* Contamos los fallos */
	SELECT COUNT(*)
	INTO totalFallos
	FROM ComunicacionVotantes
	WHERE ID_Proceso = P_IDProceso;
	totalFallos:= totalComunicaciones-totalFallos;
	
    paramSalida:='Total comunicaciones: '||totalComunicaciones||' de un total de '||totalCensadas||' personas'||' y fallos: '||totalFallos;
    RSP:= 'OK';
    COMMIT;
    INSERT INTO LogOperaciones VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, paramSalida, RSP);
    COMMIT;
   
EXCEPTION
    WHEN VACIO_ERROR THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL VALOR DEL PROCEDIMEINTO NO PUEDE SER NULO';
    ROLLBACK;
    INSERT INTO LogOperaciones VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, paramSalida, RSP);
    COMMIT;
    
    WHEN NO_DATA_FOUND THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL PROCEDIMIENTO NO SE ENCUENTRA - NO EXISTE';
    ROLLBACK;
    INSERT INTO LogOperaciones VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, paramSalida, RSP);
    COMMIT;
    
    WHEN OTHERS THEN
    numERROR:= SQLCODE;
    tipoERROR:= SUBSTR(SQLERRM, 1, 90);
    RSP := TO_CHAR(numERROR)||' - '||tipoERROR;
    ROLLBACK;
    INSERT INTO LogOperaciones VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, PARAMsALIDA, RSP);
    COMMIT;
END;
/


/************************************************************
* Procedimiento para seleccionar los miembros de las mesas
* en un procedimiento electoral.
*
* Entrada: ID_Proceso
* RSP salida: OK si se realiza con xito
*             ERROR+TIPO DE ERROR en caso contrario
* Salida: Nmero de miembros seleccionados
* Pre-condicin: el proceso electoral, las reglas y las personas deben existir.
* Post-condicin: se crea el censo electoral de ese proceso.
************************************************************/

CREATE OR REPLACE PROCEDURE seleccionarMiembros(
	P_IDProceso IN ProcesoElectoral.ID_Proceso%TYPE,
    paramSalida OUT VARCHAR2,
	RSP OUT VARCHAR2) 
IS
    paramEntrada VARCHAR2(500);
	numERROR INTEGER;
	tipoERROR VARCHAR2(100);
    VACIO_ERROR EXCEPTION;
    procesoExiste ProcesoElectoral.ID_Proceso%TYPE;
    P_mesa MesaElectoral.ID_Mesa%TYPE;
	totalSeleccionado NUMBER;

    
BEGIN
    paramEntrada:= P_IDProceso;
	
    /* Comprobamos que el ID no sea nulo */
    IF (P_IDProceso IS NULL)
    THEN
        RAISE VACIO_ERROR;
    END IF;  
    
    /* Comprobamos que exista el Proceso Electoral */ 
    BEGIN
        SELECT ID_Proceso
        INTO procesoExiste
        FROM ProcesoElectoral
        WHERE ID_Proceso = P_IDProceso
        AND ROWNUM = 1;
        EXCEPTION
            WHEN NO_DATA_FOUND THEN
            RAISE NO_DATA_FOUND;
    END;
    
    /* Si ha pasado las comprobaciones anteriores, seleccionamos */   

	totalSeleccionado:= 0;
	
	DECLARE
	CURSOR C_seleccion IS
		SELECT ID_Mesa
		FROM MesaElectoral;
	BEGIN
		FOR p IN C_seleccion
		LOOP
		/* realizamos la seleccion de los miembros del censo */	
       
		INSERT INTO MiembroMesa 
		SELECT numDoc, letraDoc, ID_RP, P_IDProceso,'3, 4, 5', mesaElectoral, fechaElecciones, 'suplente', 'N', 'N', 'S' 
		FROM 
			(SELECT *
			FROM CensoElectoral
			ORDER BY DBMS_RANDOM.VALUE) 
		WHERE mesaElectoral = p.ID_Mesa
		AND ID_Proceso = P_IDProceso
		AND ROWNUM <= 6;
		
		totalSeleccionado:= totalSeleccionado+6;
		END LOOP;
	END;
	
    paramSalida:='Total miembros seleccionados: '||totalSeleccionado;
    RSP:= 'OK';
    COMMIT;
    INSERT INTO LogOperaciones VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, paramSalida, RSP);
    COMMIT;
   
EXCEPTION
    WHEN VACIO_ERROR THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL VALOR DEL PROCEDIMEINTO NO PUEDE SER NULO';
    ROLLBACK;
    INSERT INTO LogOperaciones VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, paramSalida, RSP);
    COMMIT;
    
    WHEN NO_DATA_FOUND THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL PROCEDIMIENTO NO SE ENCUENTRA - NO EXISTE';
    ROLLBACK;
    INSERT INTO LogOperaciones VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, paramSalida, RSP);
    COMMIT;
    
    WHEN OTHERS THEN
    numERROR:= SQLCODE;
    tipoERROR:= SUBSTR(SQLERRM, 1, 90);
    RSP := TO_CHAR(numERROR)||' - '||tipoERROR;
    ROLLBACK;
    INSERT INTO LogOperaciones VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, PARAMsALIDA, RSP);
    COMMIT;
END;
/

/************************************************************
* Procedimiento para asignar a los miembros de las mesas
* los estados apto/alegaciones/localizado y el cargo.
* Se entiende que es una simulacin promedio.
* No localizados: 2% de probabilidad
* Alegaciones: 25% probabilidad
* No aptos: 20% de los que han presentado alegaciones
*
* Entrada: ID_Proceso
* RSP salida: OK si se realiza con xito
*             ERROR+TIPO DE ERROR en caso contrario
* Salida: Nmero de miembros modificados, total no localizados,
* 		 total no aptos, total alegaciones
* Pre-condicin: el proceso electoral, las reglas y las personas deben existir.
* Post-condicin: se crea el censo electoral de ese proceso.
************************************************************/

CREATE OR REPLACE PROCEDURE simularEstados(
	P_IDProceso IN ProcesoElectoral.ID_Proceso%TYPE,
    paramSalida OUT VARCHAR2,
	RSP OUT VARCHAR2) 
IS
    paramEntrada VARCHAR2(500);
	numERROR INTEGER;
	tipoERROR VARCHAR2(100);
    VACIO_ERROR EXCEPTION;
    procesoExiste ProcesoElectoral.ID_Proceso%TYPE;
	cargoMiembro MiembroMesa.cargo%TYPE;
	P_localizado MiembroMesa.localizado%TYPE;
	P_alegaciones MiembroMesa.alegaciones%TYPE;
	P_apto MiembroMesa.apto%TYPE;
	totalSeleccionado NUMBER;
	totalAptos NUMBER;
	totalLocalizados NUMBER;
	totalAlegaciones NUMBER;
	contadorSuplente INTEGER;
	numAleatorio INTEGER;

    
BEGIN
    paramEntrada:= P_IDProceso;
	
    /* Comprobamos que el ID no sea nulo */
    IF (P_IDProceso IS NULL)
    THEN
        RAISE VACIO_ERROR;
    END IF;  
    
    /* Comprobamos que exista el Proceso Electoral */ 
    BEGIN
        SELECT ID_Proceso
        INTO procesoExiste
        FROM ProcesoElectoral
        WHERE ID_Proceso = P_IDProceso
        AND ROWNUM = 1;
        EXCEPTION
            WHEN NO_DATA_FOUND THEN
            RAISE NO_DATA_FOUND;
    END;
    
    /* Si ha pasado las comprobaciones anteriores, modificamos */   

	totalSeleccionado:= 0;
	totalAptos:= 0;
	totalLocalizados:= 0;
	totalAlegaciones:= 0;
	cargoMiembro:='presidente';
	contadorSuplente:= 1;
    
	DECLARE
	CURSOR C_asignar IS
		SELECT *
		FROM MiembroMesa;
	BEGIN
		FOR p IN C_asignar
		LOOP
		
		/* asignamos No localizados, alegaciones y aptos */
		numAleatorio:= ROUND(DBMS_RANDOM.VALUE(1,100));
		IF (numAleatorio <= 2)
		THEN
			/* si no se le localiza tampoco puede presentar alegaciones */
			P_localizado:='N';
			P_alegaciones:='N';
			P_apto:='S';
			totalLocalizados:=totalLocalizados+1;
		ELSE
			P_localizado:='S';
			numAleatorio:= ROUND(DBMS_RANDOM.VALUE(1,100));
			IF (numAleatorio <= 25)
			THEN
				P_alegaciones:='S';
				totalAlegaciones:=totalAlegaciones+1;
				numAleatorio:= ROUND(DBMS_RANDOM.VALUE(1,100));
					IF (numAleatorio <= 20)
					THEN
						P_apto:='N';
						totalAptos:=totalAptos+1;
					END IF;
			ELSE
				P_alegaciones:='N';
				P_apto:='S';
			END IF;
		END IF;
        
		/* realizamos la modificacin de los miembros del censo */
		modMiembroMesa(p.numDoc, p.letraDoc, p.ID_RP, p.ID_Proceso, p.ID_RS, p.ID_Mesa, p.fechaElecciones, cargoMiembro, P_localizado, P_alegaciones, P_apto, RSP);
		IF (cargoMiembro = 'presidente')
		THEN
			cargoMiembro:='secretario';
		ELSIF (cargoMiembro = 'secretario') THEN
			cargoMiembro:='vocal';
		ELSIF (cargoMiembro = 'vocal') THEN 
			cargoMiembro:='suplente';
		ELSIF (cargoMiembro = 'suplente') AND (contadorSuplente=1) THEN
			contadorSuplente:=2;
		ELSIF (cargoMiembro = 'suplente') AND (contadorSuplente=2) THEN
			contadorSuplente:=3;
		ELSIF (cargoMiembro = 'suplente') AND (contadorSuplente=3) THEN
			cargoMiembro:='presidente';
			contadorSuplente:=1;
		END IF;
		totalSeleccionado:= totalSeleccionado+1;
		END LOOP;
	END;
	
    paramSalida:='Total miembros: '||totalSeleccionado||' - Total NO Aptos: '||totalAptos||' - Total NO Localizados: '||totalLocalizados||' - Total Alegaciones'||totalAlegaciones;
    RSP:= 'OK';
    COMMIT;
    INSERT INTO LogOperaciones VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, paramSalida, RSP);
    COMMIT;
   
EXCEPTION
    WHEN VACIO_ERROR THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL VALOR DEL PROCEDIMEINTO NO PUEDE SER NULO';
    ROLLBACK;
    INSERT INTO LogOperaciones VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, paramSalida, RSP);
    COMMIT;
    
    WHEN NO_DATA_FOUND THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL PROCEDIMIENTO NO SE ENCUENTRA - NO EXISTE';
    ROLLBACK;
    INSERT INTO LogOperaciones VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, paramSalida, RSP);
    COMMIT;
    
    WHEN OTHERS THEN
    numERROR:= SQLCODE;
    tipoERROR:= SUBSTR(SQLERRM, 1, 90);
    RSP := TO_CHAR(numERROR)||' - '||tipoERROR;
    ROLLBACK;
    INSERT INTO LogOperaciones VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, PARAMsALIDA, RSP);
    COMMIT;
END;
/

/************************************************************
* Procedimiento para asignar los votos simulando los porcentajes
* de las elecciones municipales del 2019 de Zaragoza.
*
* Entrada: ID_Proceso
* RSP salida: OK si se realiza con xito
*             ERROR+TIPO DE ERROR en caso contrario
* Salida: Nmero de votos emitidos.
* Pre-condicin: el proceso electoral, las reglas y las personas deben existir.
* Post-condicin: se crea el censo electoral de ese proceso.
************************************************************/

CREATE OR REPLACE PROCEDURE asignarVotos2019(
	P_IDProceso IN ProcesoElectoral.ID_Proceso%TYPE,
    paramSalida OUT VARCHAR2,
	RSP OUT VARCHAR2) 
IS
    paramEntrada VARCHAR2(500);
	numERROR INTEGER;
	tipoERROR VARCHAR2(100);
    VACIO_ERROR EXCEPTION;
    procesoExiste ProcesoElectoral.ID_Proceso%TYPE;
	totalVotos NUMBER;
	probTelematico INTEGER;
	P_modalidad voto.modalidad%TYPE;
    
BEGIN
    paramEntrada:= P_IDProceso;
	
    /* Comprobamos que el ID no sea nulo */
    IF (P_IDProceso IS NULL)
    THEN
        RAISE VACIO_ERROR;
    END IF;  
    
    /* Comprobamos que exista el Proceso Electoral */ 
    BEGIN
        SELECT ID_Proceso
        INTO procesoExiste
        FROM ProcesoElectoral
        WHERE ID_Proceso = P_IDProceso
        AND ROWNUM = 1;
        EXCEPTION
            WHEN NO_DATA_FOUND THEN
            RAISE NO_DATA_FOUND;
    END;
    
    /* Si ha pasado las comprobaciones anteriores, modificamos */   

	totalVotos:= 0;
	
	FOR v IN 1..9593
	LOOP
		P_modalidad:='presencial';
		probTelematico:= ROUND(DBMS_RANDOM.VALUE(1,100));
		IF (probTelematico <= 10)
		THEN
			P_modalidad:='telematico';
		END IF;
		altaVoto(Sec_Voto.NEXTVAL, P_modalidad, 20, 'normal', ROUND(DBMS_RANDOM.VALUE(2,28)), RSP);
		totalVotos:=totalVotos+1;
	END LOOP;
	
	FOR v IN 1..7551
	LOOP
		P_modalidad:='presencial';
		probTelematico:= ROUND(DBMS_RANDOM.VALUE(1,100));
		IF (probTelematico <= 10)
		THEN
			P_modalidad:='telematico';
		END IF;
		altaVoto(Sec_Voto.NEXTVAL, P_modalidad, 10, 'normal', ROUND(DBMS_RANDOM.VALUE(2,28)), RSP);
		totalVotos:=totalVotos+1;
	END LOOP;
	
	FOR v IN 1..6258
	LOOP
		P_modalidad:='presencial';
		probTelematico:= ROUND(DBMS_RANDOM.VALUE(1,100));
		IF (probTelematico <= 10)
		THEN
			P_modalidad:='telematico';
		END IF;
		altaVoto( Sec_Voto.NEXTVAL, P_modalidad, 17, 'normal', ROUND(DBMS_RANDOM.VALUE(2,28)), RSP);
		totalVotos:=totalVotos+1;
	END LOOP;
	
	FOR v IN 1..3454
	LOOP
		P_modalidad:='presencial';
		probTelematico:= ROUND(DBMS_RANDOM.VALUE(1,100));
		IF (probTelematico <= 10)
		THEN
			P_modalidad:='telematico';
		END IF;
		altaVoto( Sec_Voto.NEXTVAL, P_modalidad, 46, 'normal', ROUND(DBMS_RANDOM.VALUE(2,28)), RSP);
		totalVotos:=totalVotos+1;
	END LOOP;
	
	FOR v IN 1..2124
	LOOP
		P_modalidad:='presencial';
		probTelematico:= ROUND(DBMS_RANDOM.VALUE(1,100));
		IF (probTelematico <= 10)
		THEN
			P_modalidad:='telematico';
		END IF;
		altaVoto( Sec_Voto.NEXTVAL, P_modalidad, 24, 'normal', ROUND(DBMS_RANDOM.VALUE(2,28)), RSP);
		totalVotos:=totalVotos+1;
	END LOOP;
	
	FOR v IN 1..2114
	LOOP
		P_modalidad:='presencial';
		probTelematico:= ROUND(DBMS_RANDOM.VALUE(1,100));
		IF (probTelematico <= 10)
		THEN
			P_modalidad:='telematico';
		END IF;
		altaVoto( Sec_Voto.NEXTVAL, P_modalidad, 1, 'normal', ROUND(DBMS_RANDOM.VALUE(2,28)), RSP);
		totalVotos:=totalVotos+1;
	END LOOP;
	
	FOR v IN 1..1582
	LOOP
		P_modalidad:='presencial';
		probTelematico:= ROUND(DBMS_RANDOM.VALUE(1,100));
		IF (probTelematico <= 10)
		THEN
			P_modalidad:='telematico';
		END IF;
		altaVoto( Sec_Voto.NEXTVAL, P_modalidad, 35, 'normal', ROUND(DBMS_RANDOM.VALUE(2,28)), RSP);
		totalVotos:=totalVotos+1;
	END LOOP;
	
	FOR v IN 1..580
	LOOP
		P_modalidad:='presencial';
		probTelematico:= ROUND(DBMS_RANDOM.VALUE(1,100));
		IF (probTelematico <= 10)
		THEN
			P_modalidad:='telematico';
		END IF;
		altaVoto( Sec_Voto.NEXTVAL, P_modalidad, 47, 'normal', ROUND(DBMS_RANDOM.VALUE(2,28)), RSP);
		totalVotos:=totalVotos+1;
	END LOOP;
	
	FOR v IN 1..267
	LOOP
		P_modalidad:='presencial';
		probTelematico:= ROUND(DBMS_RANDOM.VALUE(1,100));
		IF (probTelematico <= 10)
		THEN
			P_modalidad:='telematico';
		END IF;
		altaVoto( Sec_Voto.NEXTVAL, P_modalidad, 48, 'normal', ROUND(DBMS_RANDOM.VALUE(2,28)), RSP);
		totalVotos:=totalVotos+1;
	END LOOP;
	
	FOR v IN 1..215
	LOOP
		P_modalidad:='presencial';
		probTelematico:= ROUND(DBMS_RANDOM.VALUE(1,100));
		IF (probTelematico <= 10)
		THEN
			P_modalidad:='telematico';
		END IF;
		altaVoto( Sec_Voto.NEXTVAL, P_modalidad, 49, 'normal', ROUND(DBMS_RANDOM.VALUE(2,28)), RSP);
		totalVotos:=totalVotos+1;
	END LOOP;
	
	FOR v IN 1..81
	LOOP
		P_modalidad:='presencial';
		probTelematico:= ROUND(DBMS_RANDOM.VALUE(1,100));
		IF (probTelematico <= 10)
		THEN
			P_modalidad:='telematico';
		END IF;
		altaVoto( Sec_Voto.NEXTVAL, P_modalidad, 50, 'normal', ROUND(DBMS_RANDOM.VALUE(2,28)), RSP);
		totalVotos:=totalVotos+1;
	END LOOP;
	
	FOR v IN 1..79
	LOOP
		P_modalidad:='presencial';
		probTelematico:= ROUND(DBMS_RANDOM.VALUE(1,100));
		IF (probTelematico <= 10)
		THEN
			P_modalidad:='telematico';
		END IF;
		altaVoto( Sec_Voto.NEXTVAL, P_modalidad, 51, 'normal', ROUND(DBMS_RANDOM.VALUE(2,28)), RSP);
		totalVotos:=totalVotos+1;
	END LOOP;
	
	FOR v IN 1..24
	LOOP
		P_modalidad:='presencial';
		probTelematico:= ROUND(DBMS_RANDOM.VALUE(1,100));
		IF (probTelematico <= 10)
		THEN
			P_modalidad:='telematico';
		END IF;
		altaVoto( Sec_Voto.NEXTVAL, P_modalidad, 52, 'normal', ROUND(DBMS_RANDOM.VALUE(2,28)), RSP);
		totalVotos:=totalVotos+1;
	END LOOP;
	
	FOR v IN 1..23
	LOOP
		P_modalidad:='presencial';
		probTelematico:= ROUND(DBMS_RANDOM.VALUE(1,100));
		IF (probTelematico <= 10)
		THEN
			P_modalidad:='telematico';
		END IF;
		altaVoto( Sec_Voto.NEXTVAL, P_modalidad, 53, 'normal', ROUND(DBMS_RANDOM.VALUE(2,28)), RSP);
		totalVotos:=totalVotos+1;
	END LOOP;
	
	FOR v IN 1..22
	LOOP
		P_modalidad:='presencial';
		probTelematico:= ROUND(DBMS_RANDOM.VALUE(1,100));
		IF (probTelematico <= 10)
		THEN
			P_modalidad:='telematico';
		END IF;
		altaVoto( Sec_Voto.NEXTVAL, P_modalidad, 2, 'normal', ROUND(DBMS_RANDOM.VALUE(2,28)), RSP);
		totalVotos:=totalVotos+1;
	END LOOP;
	
	FOR v IN 1..19
	LOOP
		P_modalidad:='presencial';
		probTelematico:= ROUND(DBMS_RANDOM.VALUE(1,100));
		IF (probTelematico <= 10)
		THEN
			P_modalidad:='telematico';
		END IF;
		altaVoto( Sec_Voto.NEXTVAL, P_modalidad, 54, 'normal', ROUND(DBMS_RANDOM.VALUE(2,28)), RSP);
		totalVotos:=totalVotos+1;
	END LOOP;
	
	FOR v IN 1..19
	LOOP
		P_modalidad:='presencial';
		probTelematico:= ROUND(DBMS_RANDOM.VALUE(1,100));
		IF (probTelematico <= 10)
		THEN
			P_modalidad:='telematico';
		END IF;
		altaVoto( Sec_Voto.NEXTVAL, P_modalidad, 55, 'normal', ROUND(DBMS_RANDOM.VALUE(2,28)), RSP);
		totalVotos:=totalVotos+1;
	END LOOP;
	
	FOR v IN 1..17
	LOOP
		P_modalidad:='presencial';
		probTelematico:= ROUND(DBMS_RANDOM.VALUE(1,100));
		IF (probTelematico <= 10)
		THEN
			P_modalidad:='telematico';
		END IF;
		altaVoto( Sec_Voto.NEXTVAL, P_modalidad, 56, 'normal', ROUND(DBMS_RANDOM.VALUE(2,28)), RSP);
		totalVotos:=totalVotos+1;
	END LOOP;
	
	FOR v IN 1..17
	LOOP
		P_modalidad:='presencial';
		probTelematico:= ROUND(DBMS_RANDOM.VALUE(1,100));
		IF (probTelematico <= 10)
		THEN
			P_modalidad:='telematico';
		END IF;
		altaVoto( Sec_Voto.NEXTVAL, P_modalidad, 57, 'normal', ROUND(DBMS_RANDOM.VALUE(2,28)), RSP);
		totalVotos:=totalVotos+1;
	END LOOP;
	
	/* En blanco */
	FOR v IN 1..218
	LOOP
		P_modalidad:='presencial';
		probTelematico:= ROUND(DBMS_RANDOM.VALUE(1,100));
		IF (probTelematico <= 10)
		THEN
			P_modalidad:='telematico';
		END IF;
		altaVoto( Sec_Voto.NEXTVAL, P_modalidad, NULL, 'enBlanco', ROUND(DBMS_RANDOM.VALUE(2,28)), RSP);
		totalVotos:=totalVotos+1;
	END LOOP;
	
	/* Nulos que slo pueden ser presenciales*/
	FOR v IN 1..139
	LOOP
		P_modalidad:='presencial';
		altaVoto( Sec_Voto.NEXTVAL, P_modalidad, NULL, 'nulo', ROUND(DBMS_RANDOM.VALUE(2,28)), RSP);
		totalVotos:=totalVotos+1;
	END LOOP;
	
	
    paramSalida:='Total votos: '||totalVotos;
    RSP:= 'OK';
    COMMIT;
    INSERT INTO LogOperaciones VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, paramSalida, RSP);
    COMMIT;
   
EXCEPTION
    WHEN VACIO_ERROR THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL VALOR DEL PROCEDIMEINTO NO PUEDE SER NULO';
    ROLLBACK;
    INSERT INTO LogOperaciones VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, paramSalida, RSP);
    COMMIT;
    
    WHEN NO_DATA_FOUND THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL PROCEDIMIENTO NO SE ENCUENTRA - NO EXISTE';
    ROLLBACK;
    INSERT INTO LogOperaciones VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, paramSalida, RSP);
    COMMIT;
    
    WHEN OTHERS THEN
    numERROR:= SQLCODE;
    tipoERROR:= SUBSTR(SQLERRM, 1, 90);
    RSP := TO_CHAR(numERROR)||' - '||tipoERROR;
    ROLLBACK;
    INSERT INTO LogOperaciones VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, PARAMsALIDA, RSP);
    COMMIT;
END;
/


/************************************************************
* Procedimiento para realizar el recuento con desglose
* de los votos de un proceso.
*
* Entrada: ID_Proceso
* RSP salida: OK si se realiza con xito
*             ERROR+TIPO DE ERROR en caso contrario
* Salida: Nmero de votos, Nmero de Mesas,
* Pre-condicin: el proceso electoral y los votos y mesas deben existir.
* Post-condicin: se crea el recuento por mesas de ese proceso.
************************************************************/

CREATE OR REPLACE PROCEDURE realizaRecuento(
	P_IDProceso IN ProcesoElectoral.ID_Proceso%TYPE,
    paramSalida OUT VARCHAR2,
	RSP OUT VARCHAR2) 
IS
    paramEntrada VARCHAR2(500);
	numERROR INTEGER;
	tipoERROR VARCHAR2(100);
    VACIO_ERROR EXCEPTION;
    procesoExiste ProcesoElectoral.ID_Proceso%TYPE;
	totalVotos NUMBER;
	totalMesas NUMBER;
	P_nulos INTEGER;
	P_enBlanco INTEGER;
	P_partido INTEGER;
	P_nombre PartidoPolitico.nombrePartido%TYPE;
    
BEGIN
    paramEntrada:= P_IDProceso;
	
    /* Comprobamos que el ID no sea nulo */
    IF (P_IDProceso IS NULL)
    THEN
        RAISE VACIO_ERROR;
    END IF;  
    
    /* Comprobamos que exista el Proceso Electoral */ 
    BEGIN
        SELECT ID_Proceso
        INTO procesoExiste
        FROM ProcesoElectoral
        WHERE ID_Proceso = P_IDProceso
        AND ROWNUM = 1;
        EXCEPTION
            WHEN NO_DATA_FOUND THEN
            RAISE NO_DATA_FOUND;
    END;
    
    /* Si ha pasado las comprobaciones anteriores, realizamos el recuento */   

    totalVotos:=0;
	totalMesas:=0;
	DECLARE
	CURSOR C_recuento IS
		SELECT *
		FROM (SELECT m.ID_Mesa 
            FROM MesaElectoral m
            JOIN CentroVotacion c ON m.ID_Centro = c.ID_Centro
            JOIN DemarcacionElectoral d ON c.ID_Demarcacion = d.ID_Demarcacion
            WHERE d.ID_Proceso = 3) mesas;
	BEGIN
		FOR r IN C_recuento
		LOOP
			SELECT COUNT(*) 
			INTO P_nulos
			FROM VOTO 
			WHERE tipoVoto='nulo' AND ID_Mesa=r.ID_Mesa;
			
			SELECT COUNT(*) 
			INTO P_enBlanco
			FROM VOTO 
			WHERE tipoVoto='enBlanco' AND ID_Mesa=r.ID_Mesa;
			
			/* damos de alta el recuento */
			altaRecuento(Sec_Recuento.NEXTVAL, P_nulos, P_enBlanco, r.ID_Mesa, RSP);
			
            
			DECLARE
			CURSOR C_desglose IS
				SELECT *
				FROM PartidoPolitico;
			BEGIN
				FOR d IN C_desglose
				LOOP
					SELECT COUNT(*) 
					INTO P_partido 
					FROM VOTO 
					WHERE partido=d.ID_Partido
                    AND ID_Mesa=r.ID_Mesa;

					/* verificamos que el Partido participe en ese Proceso */				
					IF (P_partido >= 1)
					THEN
						SELECT nombrePartido
						INTO P_nombre
						FROM PartidoPolitico
						WHERE ID_Partido = d.ID_Partido;
						/* damos de alta el desglose */
						altaDesglose(Sec_Recuento.CURRVAL, Sec_Desglose.NEXTVAL, P_nombre, P_partido, RSP);
						totalVotos:= totalVotos+P_partido;
					END IF;
				END LOOP;
			END; 

			totalVotos:= totalVotos+P_nulos+P_enBlanco;
			totalMesas:= totalMesas+1;
		END LOOP;
	END;
	
    paramSalida:='Total votos: '||totalVotos||' - Total mesas: '||totalMesas;
    RSP:= 'OK';
    COMMIT;
    INSERT INTO LogOperaciones VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, paramSalida, RSP);
    COMMIT;
   
EXCEPTION
    WHEN VACIO_ERROR THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL VALOR DEL PROCEDIMEINTO NO PUEDE SER NULO';
    ROLLBACK;
    INSERT INTO LogOperaciones VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, paramSalida, RSP);
    COMMIT;
    
    WHEN NO_DATA_FOUND THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL PROCEDIMIENTO NO SE ENCUENTRA - NO EXISTE';
    ROLLBACK;
    INSERT INTO LogOperaciones VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, paramSalida, RSP);
    COMMIT;
    
    WHEN OTHERS THEN
    numERROR:= SQLCODE;
    tipoERROR:= SUBSTR(SQLERRM, 1, 90);
    RSP := TO_CHAR(numERROR)||' - '||tipoERROR;
    ROLLBACK;
    INSERT INTO LogOperaciones VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, PARAMsALIDA, RSP);
    COMMIT;
END;
/

/************************************************************
* Procedimiento para  anotar los resultados de un proceso
*
* Entrada: ID_Proceso, IR_Metodo
* RSP salida: OK si se realiza con xito
*             ERROR+TIPO DE ERROR en caso contrario
* Salida: Nmero de inserciones
* Pre-condicin: el proceso electoral y los votos y mesas deben existir.
* Post-condicin: se crea el recuento por mesas de ese proceso.
************************************************************/


CREATE OR REPLACE PROCEDURE listarResultados(
	P_IDProceso IN ProcesoElectoral.ID_Proceso%TYPE,
	P_IDMetodo IN MetodoCalculo.ID_Metodo%TYPE,
    paramSalida OUT VARCHAR2,
	RSP OUT VARCHAR2) 
IS
    paramEntrada VARCHAR2(500);
	numERROR INTEGER;
	tipoERROR VARCHAR2(100);
    VACIO_ERROR EXCEPTION;
    procesoExiste ProcesoElectoral.ID_Proceso%TYPE;
	metodoExiste MetodoCalculo.ID_Metodo%TYPE;
	totalInserciones NUMBER;
    P_fecha ProcesoElectoral.fechaElecciones%TYPE;

    
BEGIN
    paramEntrada:= 'Proceso Electoral: '||P_IDProceso||' - Mtodo de calculo'||P_IDMetodo;
	
    /* Comprobamos que el ID no sea nulo */
    IF (P_IDProceso IS NULL) OR (P_IDMetodo IS NULL)
    THEN
        RAISE VACIO_ERROR;
    END IF;  
    
    /* Comprobamos que exista el Proceso Electoral */ 
    BEGIN
        SELECT ID_Proceso
        INTO procesoExiste
        FROM ProcesoElectoral
        WHERE ID_Proceso = P_IDProceso
        AND ROWNUM = 1;
        EXCEPTION
            WHEN NO_DATA_FOUND THEN
            RAISE NO_DATA_FOUND;
    END;
	
	/* Comprobamos que exista el Mtodo de Clculo */ 
    BEGIN
        SELECT ID_Metodo
        INTO metodoExiste
        FROM MetodoCalculo
        WHERE ID_Metodo = P_IDMetodo
        AND ROWNUM = 1;
        EXCEPTION
            WHEN NO_DATA_FOUND THEN
            RAISE NO_DATA_FOUND;
    END;
    
    /* Si ha pasado las comprobaciones anteriores, realizamos el recuento */   

    totalInserciones:=0;
	
	/* calculamos la fecha de las elecciones */
	SELECT fechaElecciones
	INTO P_fecha
	FROM ProcesoElectoral
	WHERE ID_Proceso = P_IDProceso;

	DECLARE
	CURSOR C_demarcaciones IS
		SELECT ID_Demarcacion
		FROM DemarcacionElectoral
		WHERE ID_Proceso = P_IDProceso;
	BEGIN
		FOR d IN C_demarcaciones
		LOOP		
			DECLARE
			CURSOR C_recuento IS
				SELECT ID_Recuento
				FROM Recuento
                WHERE ID_Mesa IN (SELECT m.ID_Mesa 
									FROM MesaElectoral m
									JOIN CentroVotacion c ON m.ID_Centro = c.ID_Centro
									JOIN DemarcacionElectoral d ON c.ID_Demarcacion = d.ID_Demarcacion
									WHERE d.ID_Proceso = 3);
			BEGIN
				FOR r IN C_recuento
				LOOP
					/* damos de alta el resultado */
					altaResultado( d.ID_Demarcacion,  r.ID_Recuento, P_IDMetodo, P_fecha, RSP);
					totalInserciones:= totalInserciones+1;
				END LOOP;
			END;
		END LOOP;
	END;
	
    paramSalida:='Total inserciones: '||totalInserciones;
    RSP:= 'OK';
    COMMIT;
    INSERT INTO LogOperaciones VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, paramSalida, RSP);
    COMMIT;
   
EXCEPTION
    WHEN VACIO_ERROR THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL VALOR DEL PROCEDIMEINTO NO PUEDE SER NULO';
    ROLLBACK;
    INSERT INTO LogOperaciones VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, paramSalida, RSP);
    COMMIT;
    
    WHEN NO_DATA_FOUND THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL PROCEDIMIENTO NO SE ENCUENTRA - NO EXISTE';
    ROLLBACK;
    INSERT INTO LogOperaciones VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, paramSalida, RSP);
    COMMIT;
    
    WHEN OTHERS THEN
    numERROR:= SQLCODE;
    tipoERROR:= SUBSTR(SQLERRM, 1, 90);
    RSP := TO_CHAR(numERROR)||' - '||tipoERROR;
    ROLLBACK;
    INSERT INTO LogOperaciones VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, PARAMsALIDA, RSP);
    COMMIT;
END;
/

/************************************************************
* Cclculo metodo de D Hondt en una demarcacin
* Entrada: ID_Proceso, ID_Demarcacion
*
* Umbral 3% para elecciones generales
* Umbral 5% para elecciones autonomicas y locales
*
* RSP salida: OK si se realiza con xito
*             ERROR+TIPO DE ERROR en caso contrario
* Salida: total escaos y total partidos
* Pre-condicin: el proceso electoral y la demarcacion deben existir.
* Post-condicin: se muestra el resultado en la tabla creada.
* 
************************************************************/
CREATE OR REPLACE PROCEDURE calculaHondt(
	P_IDProceso IN ProcesoElectoral.ID_Proceso%TYPE,
    P_IDDemarcacion IN DemarcacionElectoral.ID_Demarcacion%TYPE,
    paramSalida OUT VARCHAR2,
	RSP OUT VARCHAR2) 
IS
    paramEntrada VARCHAR2(500);
	numERROR INTEGER;
	tipoERROR VARCHAR2(100);
    VACIO_ERROR EXCEPTION;
    procesoExiste ProcesoElectoral.ID_Proceso%TYPE;
    demarcacionExiste DemarcacionElectoral.ID_Demarcacion%TYPE;
    P_escanos INTEGER;
    P_ambito ProcesoElectoral.ambito%TYPE;
    umbral INTEGER;
    totalEscanos INTEGER;
    totalPartidos INTEGER;
    
BEGIN
    paramEntrada:= 'Proceso Electoral: '||P_IDProceso||' - Demarcacion: '||P_IDDemarcacion;
	
    /* Comprobamos que el ID no sea nulo */
    IF (P_IDProceso IS NULL)
    THEN
        RAISE VACIO_ERROR;
    END IF;  
    
    /* Comprobamos que exista el Proceso Electoral */ 
    BEGIN
        SELECT ID_Proceso
        INTO procesoExiste
        FROM ProcesoElectoral
        WHERE ID_Proceso = P_IDProceso
        AND ROWNUM = 1;
        EXCEPTION
            WHEN NO_DATA_FOUND THEN
            RAISE NO_DATA_FOUND;
    END;
    
    /* Comprobamos que exista la Demarcacion */ 
    BEGIN
        SELECT ID_Demarcacion
        INTO demarcacionExiste
        FROM DemarcacionElectoral
        WHERE ID_Demarcacion = P_IDDemarcacion
        AND ROWNUM = 1;
        EXCEPTION
            WHEN NO_DATA_FOUND THEN
            RAISE NO_DATA_FOUND;
    END;

    /* calculamos el Umbral en funcin del mbito*/
    SELECT ambito 
    INTO P_ambito
    FROM ProcesoElectoral 
    WHERE ID_Proceso=P_IDProceso;
    
    IF (P_ambito='General') THEN
        umbral:=3;
    ELSE
        umbral:=5;
    END IF;
    
    /* seleccionamos los escaos de la demarcacion*/
    SELECT numEscanos 
    INTO P_escanos
    FROM DemarcacionElectoral 
    WHERE ID_Demarcacion=P_IDDemarcacion;

    totalEscanos:=0; 
    /* realizamos el clculo */
    BEGIN
        WHILE P_escanos >= 1
        LOOP
            UPDATE muestraHondt SET escanos=escanos+1 
            WHERE partido = (
                SELECT partido 
                FROM (
                    SELECT partido, TRUNC(votos/(escanos+1)) 
                    FROM muestraHondt 
                    WHERE votos>((
                        SELECT sum(votos) 
                        FROM muestraHondt)*umbral/100) 
                    ORDER BY TRUNC(votos/(escanos+1)) DESC) 
                WHERE ROWNUM <=1);
                P_escanos:=P_escanos-1;
                totalEscanos:=totalEscanos+1;          
        END LOOP;
    END;
    /*calculamos el total de partidos con escaos */
    SELECT COUNT(*)
    INTO totalPartidos
    FROM muestraHondt
    WHERE escanos>0;
    
    paramSalida:='Se han adjudicado un total de '||totalEscanos||' escanos para un total de '||totalPartidos||' partidos.';
    RSP:= 'OK';
    COMMIT;
    INSERT INTO LogOperaciones VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, paramSalida, RSP);
    COMMIT;
   
EXCEPTION
    WHEN VACIO_ERROR THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL VALOR DEL PROCEDIMEINTO NO PUEDE SER NULO';
    ROLLBACK;
    INSERT INTO LogOperaciones VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, paramSalida, RSP);
    COMMIT;
    
    WHEN NO_DATA_FOUND THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL PROCEDIMIENTO NO SE ENCUENTRA - NO EXISTE';
    ROLLBACK;
    INSERT INTO LogOperaciones VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, paramSalida, RSP);
    COMMIT;
    
    WHEN OTHERS THEN
    numERROR:= SQLCODE;
    tipoERROR:= SUBSTR(SQLERRM, 1, 90);
    RSP := TO_CHAR(numERROR)||' - '||tipoERROR;
    ROLLBACK;
    INSERT INTO LogOperaciones VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, PARAMsALIDA, RSP);
    COMMIT;
END;
/

/************************************************************
* Cclculo metodo de Sainte-Lagu en una demarcacin
* Entrada: ID_Proceso, ID_Demarcacion
*
* Umbral 3% para elecciones generales
* Umbral 5% para elecciones autonomicas y locales
*
* RSP salida: OK si se realiza con xito
*             ERROR+TIPO DE ERROR en caso contrario
* Salida: total escaos y total partidos
* Pre-condicin: el proceso electoral y la demarcacion deben existir.
* Post-condicin: se muestra el resultado en la tabla creada.
* 
************************************************************/
CREATE OR REPLACE PROCEDURE calculaSainte(
	P_IDProceso IN ProcesoElectoral.ID_Proceso%TYPE,
    P_IDDemarcacion IN DemarcacionElectoral.ID_Demarcacion%TYPE,
    paramSalida OUT VARCHAR2,
	RSP OUT VARCHAR2) 
IS
    paramEntrada VARCHAR2(500);
	numERROR INTEGER;
	tipoERROR VARCHAR2(100);
    VACIO_ERROR EXCEPTION;
    procesoExiste ProcesoElectoral.ID_Proceso%TYPE;
    demarcacionExiste DemarcacionElectoral.ID_Demarcacion%TYPE;
    P_escanos INTEGER;
    P_ambito ProcesoElectoral.ambito%TYPE;
    umbral INTEGER;
    totalEscanos INTEGER;
    totalPartidos INTEGER;
    
BEGIN
    paramEntrada:= 'Proceso Electoral: '||P_IDProceso||' - Demarcacion: '||P_IDDemarcacion;
	
    /* Comprobamos que el ID no sea nulo */
    IF (P_IDProceso IS NULL)
    THEN
        RAISE VACIO_ERROR;
    END IF;  
    
    /* Comprobamos que exista el Proceso Electoral */ 
    BEGIN
        SELECT ID_Proceso
        INTO procesoExiste
        FROM ProcesoElectoral
        WHERE ID_Proceso = P_IDProceso
        AND ROWNUM = 1;
        EXCEPTION
            WHEN NO_DATA_FOUND THEN
            RAISE NO_DATA_FOUND;
    END;
    
    /* Comprobamos que exista la Demarcacion */ 
    BEGIN
        SELECT ID_Demarcacion
        INTO demarcacionExiste
        FROM DemarcacionElectoral
        WHERE ID_Demarcacion = P_IDDemarcacion
        AND ROWNUM = 1;
        EXCEPTION
            WHEN NO_DATA_FOUND THEN
            RAISE NO_DATA_FOUND;
    END;

    /* calculamos el Umbral en funcin del mbito*/
    SELECT ambito 
    INTO P_ambito
    FROM ProcesoElectoral 
    WHERE ID_Proceso=P_IDProceso;
    
    IF (P_ambito='General') THEN
        umbral:=3;
    ELSE
        umbral:=5;
    END IF;
    
    /* seleccionamos los escaos de la demarcacion*/
    SELECT numEscanos 
    INTO P_escanos
    FROM DemarcacionElectoral 
    WHERE ID_Demarcacion=P_IDDemarcacion;

    totalEscanos:=0; 
    /* realizamos el clculo */
    BEGIN
        WHILE P_escanos >= 1
        LOOP
            UPDATE muestraSainte SET escanos=escanos+1 
            WHERE partido = (
                SELECT partido 
                FROM (
                    SELECT partido, TRUNC(votos/((2*escanos)+1)) 
                    FROM muestraSainte 
                    WHERE votos>((
                        SELECT sum(votos) 
                        FROM muestraSainte)*umbral/100) 
                    ORDER BY TRUNC(votos/((2*escanos)+1)) DESC) 
                WHERE ROWNUM <=1);
                P_escanos:=P_escanos-1;
                totalEscanos:=totalEscanos+1;          
        END LOOP;
    END;
    /*calculamos el total de partidos con escaos */
    SELECT COUNT(*)
    INTO totalPartidos
    FROM muestraSainte
    WHERE escanos>0;
    
    paramSalida:='Se han adjudicado un total de '||totalEscanos||' escanos para un total de '||totalPartidos||' partidos.';
    RSP:= 'OK';
    COMMIT;
    INSERT INTO LogOperaciones VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, paramSalida, RSP);
    COMMIT;
   
EXCEPTION
    WHEN VACIO_ERROR THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL VALOR DEL PROCEDIMEINTO NO PUEDE SER NULO';
    ROLLBACK;
    INSERT INTO LogOperaciones VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, paramSalida, RSP);
    COMMIT;
    
    WHEN NO_DATA_FOUND THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL PROCEDIMIENTO NO SE ENCUENTRA - NO EXISTE';
    ROLLBACK;
    INSERT INTO LogOperaciones VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, paramSalida, RSP);
    COMMIT;
    
    WHEN OTHERS THEN
    numERROR:= SQLCODE;
    tipoERROR:= SUBSTR(SQLERRM, 1, 90);
    RSP := TO_CHAR(numERROR)||' - '||tipoERROR;
    ROLLBACK;
    INSERT INTO LogOperaciones VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, PARAMsALIDA, RSP);
    COMMIT;
END;
/




