/************************************************************
* Este es el fichero de carga de los procedimientos CRUD/ABM
*
* Como mnimo disponen de un parmetro de salida llamado RSP, de tipo string, 
* que indicar si la ejecucin ha finalizado correctamente (valor OK) 
* o si ha fracasado (valor ERROR+TIPO DE ERROR).
* Dispondrn de tratamiento de excepciones.
*
* Fecha creacin: 24/03/2021
* 
* TFG: Creacin base de datos de procesos electorales.
*
* Autor: Francisco Jos Martnez del Amo
* Tutor: Jordi Ferrer Durn
************************************************************/


/************************************************************
* Procedimiento para AADIR un Gobierno a la BD
* Entrada: ID_Gobierno, ambito, nombreGob
* Salida: OK si se realiza con xito
*         ERROR+TIPO DE ERROR en caso contrario
* Pre-condicin: el Gobierno no debe existir 
* Post-condicin: el Gobierno es dado de alta
************************************************************/

CREATE OR REPLACE PROCEDURE altaGobierno(
	P_ID IN Gobierno.ID_Gobierno%TYPE,
	P_Ambito IN Gobierno.ambito%TYPE,
	P_NombreGob IN Gobierno.nombreGob%TYPE,
	RSP OUT VARCHAR2) 
IS
    paramEntrada VARCHAR2(500);
	numERROR INTEGER;
	tipoERROR VARCHAR2(100);
    VACIO_ERROR EXCEPTION;

BEGIN
    paramEntrada:= P_ID||'-Ambito: '||P_Ambito||' - Nombre: '||P_NombreGob;
    /* Comprobamos que ningn valor sea nulo */
    IF (P_ID IS NULL) OR (P_Ambito IS NULL) OR (P_NombreGob IS NULL)
    THEN
        RAISE VACIO_ERROR;
    END IF;
    
    INSERT INTO Gobierno VALUES (P_ID, P_Ambito, P_NombreGob);
    RSP:= 'OK';
    COMMIT;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
EXCEPTION  
    WHEN DUP_VAL_ON_INDEX THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - GOBIERNO DUPLICADO / YA EXISTE';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN VACIO_ERROR THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL VALOR NO PUEDE SER NULO';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN OTHERS THEN
    numERROR:= SQLCODE;
    tipoERROR:= SUBSTR(SQLERRM, 1, 90);
    RSP := TO_CHAR(numERROR)||' - '||tipoERROR;
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
END;
/


/************************************************************
* Procedimiento para dar de BAJA un Gobierno EN la BD
* Entrada: ID_Gobierno, 
* Salida: OK si se realiza con xito
*         ERROR+TIPO DE ERROR en caso contrario
* Pre-condicin: el Gobierno DEBE existir 
* Post-condicin: el Gobierno es ELIMINADO
************************************************************/

CREATE OR REPLACE PROCEDURE bajaGobierno(
	P_ID IN Gobierno.ID_Gobierno%TYPE,
	RSP OUT VARCHAR2) 
IS
    paramEntrada VARCHAR2(500);
	numERROR INTEGER;
	tipoERROR VARCHAR2(100);
    VACIO_ERROR EXCEPTION;
    ID_NULL Gobierno.ID_Gobierno%TYPE;

BEGIN
    paramEntrada:= P_ID;
    
    /* Comprobamos que ningn valor sea nulo */
    IF (P_ID IS NULL)
    THEN
        RAISE VACIO_ERROR;
    END IF;
    /* Comprobamos que no exista ya ese gobierno  */
    BEGIN
        SELECT ID_Gobierno
        INTO ID_NULL
        FROM Gobierno
        WHERE ID_Gobierno = P_ID
        AND ROWNUM = 1;
        EXCEPTION
            WHEN NO_DATA_FOUND THEN
            RAISE NO_DATA_FOUND;
        END;
    /* borramo si ha pasado las anteriores omprobaciones */
    DELETE FROM Gobierno WHERE ID_Gobierno= P_ID;
    RSP:= 'OK';
    COMMIT;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
EXCEPTION
    WHEN VACIO_ERROR THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL VALOR NO PUEDE SER NULO';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN NO_DATA_FOUND THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL VALOR NO EXISTE';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN OTHERS THEN
    numERROR:= SQLCODE;
    tipoERROR:= SUBSTR(SQLERRM, 1, 90);
    RSP := TO_CHAR(numERROR)||' - '||tipoERROR;
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
END;
/

/************************************************************
* Procedimiento para MODIFICAR un Gobierno EN la BD
* Entrada: ID_Gobierno, ambito, nombreGob
* Salida: OK si se realiza con xito
*         ERROR+TIPO DE ERROR en caso contrario
* Pre-condicin: el Gobierno DEBE existir 
* Post-condicin: el Gobierno es dado MODIFICADO
************************************************************/

CREATE OR REPLACE PROCEDURE modGobierno(
	P_ID IN Gobierno.ID_Gobierno%TYPE,
	P_Ambito IN Gobierno.ambito%TYPE,
	P_NombreGob IN Gobierno.nombreGob%TYPE,
	RSP OUT VARCHAR2) 
IS
    paramEntrada VARCHAR2(500);
	numERROR INTEGER;
	tipoERROR VARCHAR2(100);
    VACIO_ERROR EXCEPTION;
    nombreDUPLICADO Gobierno.nombreGob%TYPE;

BEGIN
    paramEntrada:= P_ID||'-Ambito: '||P_Ambito||' - Nombre: '||P_NombreGob;
    
    /* Comprobamos que el ID no sea nulo o bien que ambito y nombre no sean nulos a la vez */
    IF (P_ID IS NULL) OR ((P_Ambito IS NULL) AND (P_NombreGob IS NULL))
    THEN
        RAISE VACIO_ERROR;
    END IF;
    
    /* Comprobamos que exista el gobierno a modificar  */
    BEGIN
        SELECT nombreGob
        INTO nombreDUPLICADO
        FROM Gobierno
        WHERE ID_Gobierno = P_ID
        AND ROWNUM = 1;
        EXCEPTION
            WHEN NO_DATA_FOUND THEN
            RAISE DUP_VAL_ON_INDEX;
    END;
    
    /* si ha pasado las comprobaciones anteriores, actualizamos */
    UPDATE Gobierno SET ambito= P_Ambito, nombreGob= P_NombreGob WHERE ID_Gobierno= P_ID;
    RSP:= 'OK';
    COMMIT;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
EXCEPTION
    WHEN VACIO_ERROR THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL VALOR NO PUEDE SER NULO';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN DUP_VAL_ON_INDEX THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - GOBIERNO DUPLICADO / YA EXISTE';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN OTHERS THEN
    numERROR:= SQLCODE;
    tipoERROR:= SUBSTR(SQLERRM, 1, 90);
    RSP := TO_CHAR(numERROR)||' - '||tipoERROR;
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
END;
/

/************************************************************
* Procedimiento para AADIR un Proceso Electoral a la BD
* Entrada: ID_Proceso, ambito, fechaElecciones, nombreProceso, 
*          ID_Gobierno
* Salida: OK si se realiza con xito
*         ERROR+TIPO DE ERROR en caso contrario
* Pre-condicin: el Gobierno DEBE existir 
                 el Proceso Electoral NO DEBE existir
                 incluye el ID o el mbito+fechaElecciones
* Post-condicin: el Proceso Electoral se CREA
************************************************************/
CREATE OR REPLACE PROCEDURE altaProcesoElectoral(
	P_ID IN ProcesoElectoral.ID_Proceso%TYPE,
	P_Ambito IN ProcesoElectoral.ambito%TYPE,
	P_Fecha IN ProcesoElectoral.fechaElecciones%TYPE,
    P_Nombre IN ProcesoElectoral.nombreProceso%TYPE,
    P_Gobierno IN ProcesoElectoral.ID_Gobierno%TYPE,
	RSP OUT VARCHAR2) 
IS
    paramEntrada VARCHAR2(500);
	numERROR INTEGER;
	tipoERROR VARCHAR2(100);
    VACIO_ERROR EXCEPTION;
    ambitoU1 ProcesoElectoral.ambito%TYPE; 
    fechaU1 ProcesoElectoral.fechaElecciones%TYPE;

BEGIN
    paramEntrada:= P_ID||'-Ambito: '||P_Ambito||'-Fecha: '||P_Fecha||'-Nombre: '||P_Nombre||' - Gobierno: '||P_Gobierno;
    /* Comprobamos que ningn valor sea nulo */
    IF (P_ID IS NULL) OR (P_Ambito IS NULL) OR (P_Fecha IS NULL) OR (P_Nombre IS NULL) OR (P_Gobierno IS NULL)
    THEN
        RAISE VACIO_ERROR;
    END IF;
    
    /* Comprobamos la clave alternativa Ambito y Fecha */
    BEGIN
        SELECT ambito, fechaElecciones 
        INTO ambitoU1, fechaU1
        FROM ProcesoElectoral 
        WHERE ambito=P_Ambito AND fechaElecciones=P_Fecha
        AND ROWNUM = 1;
        RAISE DUP_VAL_ON_INDEX;
    END;
    
    INSERT INTO ProcesoElectoral VALUES (P_ID, P_Ambito, P_Fecha,P_Nombre, P_Gobierno);
    RSP:= 'OK';
    COMMIT;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
EXCEPTION  
    WHEN NO_DATA_FOUND THEN
    INSERT INTO ProcesoElectoral VALUES (P_ID, P_Ambito, P_Fecha,P_Nombre, P_Gobierno);
    RSP:= 'OK';
    COMMIT;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN DUP_VAL_ON_INDEX THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - PROCESO ELECTORAL DUPLICADO O YA EXISTE';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN VACIO_ERROR THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL VALOR NO PUEDE SER NULO';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN OTHERS THEN
    numERROR:= SQLCODE;
    tipoERROR:= SUBSTR(SQLERRM, 1, 90);
    RSP := TO_CHAR(numERROR)||' - '||tipoERROR;
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
END;
/


/************************************************************
* Procedimiento para dar de BAJA un Proceso Electoral EN la BD
* Entrada: ID_Proceso
* Salida: OK si se realiza con xito
*         ERROR+TIPO DE ERROR en caso contrario
* Pre-condicin: el Proceso Electoral DEBE existir
* Post-condicin: el Proceso Electoral se BORRA
************************************************************/
CREATE OR REPLACE PROCEDURE bajaProcesoElectoral(
	P_ID IN ProcesoElectoral.ID_Proceso%TYPE,
	RSP OUT VARCHAR2) 
IS
    paramEntrada VARCHAR2(500);
	numERROR INTEGER;
	tipoERROR VARCHAR2(100);
    VACIO_ERROR EXCEPTION;
    ID_NULL Gobierno.ID_Gobierno%TYPE;

BEGIN
    paramEntrada:= P_ID;
    
    /* Comprobamos que ningn valor sea nulo */
    IF (P_ID IS NULL)
    THEN
        RAISE VACIO_ERROR;
    END IF;
    /* Comprobamos exista ese Proceso Electoral*/
    BEGIN
        SELECT ID_Proceso
        INTO ID_NULL
        FROM ProcesoElectoral
        WHERE ID_Proceso = P_ID
        AND ROWNUM = 1;
        EXCEPTION
            WHEN NO_DATA_FOUND THEN
            RAISE NO_DATA_FOUND;
    END;
    /* borramos si ha pasado las anteriores omprobaciones */
    DELETE FROM ProcesoElectoral WHERE ID_Proceso= P_ID;
    RSP:= 'OK';
    COMMIT;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
EXCEPTION
    WHEN VACIO_ERROR THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL VALOR NO PUEDE SER NULO';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN NO_DATA_FOUND THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL VALOR NO EXISTE EN LA LISTA DE PROCESOS ELECTORALES';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN OTHERS THEN
    numERROR:= SQLCODE;
    tipoERROR:= SUBSTR(SQLERRM, 1, 90);
    RSP := TO_CHAR(numERROR)||' - '||tipoERROR;
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
END;
/

/************************************************************
* Procedimiento para MODIFICAR un Proceso Electoral EN la BD
* Entrada: ID_Proceso, ambito, fechaElecciones, nombreProceso, 
*          ID_Gobierno
* Salida: OK si se realiza con xito
*         ERROR+TIPO DE ERROR en caso contrario
* Pre-condicin: el Proceso Electoral  DEBE existir
* Post-condicin: el Proceso Electoral se MODIFICA
************************************************************/
CREATE OR REPLACE PROCEDURE modProcesoElectoral(
	P_ID IN ProcesoElectoral.ID_Proceso%TYPE,
	P_Ambito IN ProcesoElectoral.ambito%TYPE,
	P_Fecha IN ProcesoElectoral.fechaElecciones%TYPE,
    P_Nombre IN ProcesoElectoral.nombreProceso%TYPE,
    P_Gobierno IN ProcesoElectoral.ID_Gobierno%TYPE,
	RSP OUT VARCHAR2) 
IS
    paramEntrada VARCHAR2(500);
	numERROR INTEGER;
	tipoERROR VARCHAR2(100);
    VACIO_ERROR EXCEPTION;
    procesoDUPLICADO ProcesoElectoral.ID_Proceso%TYPE;
    ambitoU1 ProcesoElectoral.ambito%TYPE; 
    fechaU1 ProcesoElectoral.fechaElecciones%TYPE;

BEGIN
    paramEntrada:= P_ID||'-Ambito: '||P_Ambito||'-Fecha: '||P_Fecha||'-Nombre: '||P_Nombre||' - Gobierno: '||P_Gobierno;
    
    /* Comprobamos que ningn valor sea nulo */
    IF (P_ID IS NULL) OR ((P_Ambito IS NULL) AND (P_Fecha IS NULL) AND (P_Nombre IS NULL) AND (P_Gobierno IS NULL))
    THEN
        RAISE VACIO_ERROR;
    END IF;
    
    /* Comprobamos si exite ese proceso  */
    BEGIN
        SELECT ID_Proceso
        INTO procesoDUPLICADO
        FROM ProcesoElectoral
        WHERE ID_Proceso = P_ID
        AND ROWNUM = 1;
        EXCEPTION
            WHEN NO_DATA_FOUND THEN
            RAISE DUP_VAL_ON_INDEX;
    END;
    
    /* Comprobamos la clave alternativa Ambito y Fecha */
    BEGIN
        SELECT ambito, fechaElecciones 
        INTO ambitoU1, fechaU1
        FROM ProcesoElectoral 
        WHERE ambito=P_Ambito AND fechaElecciones=P_Fecha AND ID_Proceso NOT LIKE P_ID
        AND ROWNUM = 1;
        RAISE DUP_VAL_ON_INDEX;
    END;
    
    /* si ha pasado las comprobaciones anteriores, actualizamos */
    UPDATE ProcesoElectoral SET ambito= P_Ambito, fechaElecciones= P_Fecha, nombreProceso= P_Nombre, ID_Gobierno= P_Gobierno WHERE ID_Proceso= P_ID;
    RSP:= 'OK';
    COMMIT;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
EXCEPTION
    WHEN NO_DATA_FOUND THEN
    UPDATE ProcesoElectoral SET ambito= P_Ambito, fechaElecciones= P_Fecha, nombreProceso= P_Nombre, ID_Gobierno= P_Gobierno WHERE ID_Proceso= P_ID;
    RSP:= 'OK';
    COMMIT;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN VACIO_ERROR THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL VALOR NO PUEDE SER NULO';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN DUP_VAL_ON_INDEX THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - PROCESO ELECTORAL DUPLICADO O YA EXISTE';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN OTHERS THEN
    numERROR:= SQLCODE;
    tipoERROR:= SUBSTR(SQLERRM, 1, 90);
    RSP := TO_CHAR(numERROR)||' - '||tipoERROR;
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
END;
/

/************************************************************
* Procedimiento para AADIR una Provincia a la BD
* Entrada: ID_Provincia, nombrePro, EscanosGen, EscanosAut
* Salida: OK si se realiza con xito
*         ERROR+TIPO DE ERROR en caso contrario
* Pre-condicin: la provincia NO debe existir 
* Post-condicin: la provincia es dada de ALTA
************************************************************/

CREATE OR REPLACE PROCEDURE altaProvincia(
	P_ID IN Provincia.ID_Provincia%TYPE,
	P_Nombre IN Provincia.nombrePro%TYPE,
	P_EscanosGen IN Provincia.EscanosGen%TYPE,
	P_EscanosAut IN Provincia.EscanosAut%TYPE,
	RSP OUT VARCHAR2) 
IS
    paramEntrada VARCHAR2(500);
	numERROR INTEGER;
	tipoERROR VARCHAR2(100);
    VACIO_ERROR EXCEPTION;

BEGIN
    paramEntrada:= P_ID||'- Nombre: '||P_Nombre||' - EscaosGen: '||P_EscanosGen||' - EscaosAut'||P_EscanosAut;
	
    /* Comprobamos que ningn valor sea nulo */
    IF (P_ID IS NULL) OR (P_Nombre IS NULL) OR (P_EscanosGen IS NULL) OR (P_EscanosAut IS NULL)
    THEN
        RAISE VACIO_ERROR;
    END IF;
	  
	/*Realizamos la insercin del valor*/
    INSERT INTO Provincia VALUES (P_ID, P_Nombre, P_EscanosGen, P_EscanosAut);
    RSP:= 'OK';
    COMMIT;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
EXCEPTION  
    WHEN DUP_VAL_ON_INDEX THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - LA PROVINCIA EST DUPLICADA / YA EXISTE';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN VACIO_ERROR THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL VALOR NO PUEDE SER NULO';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN OTHERS THEN
    numERROR:= SQLCODE;
    tipoERROR:= SUBSTR(SQLERRM, 1, 90);
    RSP := TO_CHAR(numERROR)||' - '||tipoERROR;
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
END;
/


/************************************************************
* Procedimiento para dar de BAJA una provincia EN la BD
* Entrada: ID_Provincia 
* Salida: OK si se realiza con xito
*         ERROR+TIPO DE ERROR en caso contrario
* Pre-condicin: la Provincia DEBE existir 
* Post-condicin: la Provincia se ELIMINA
************************************************************/

CREATE OR REPLACE PROCEDURE bajaProvincia(
	P_ID IN Provincia.ID_Provincia%TYPE,
	RSP OUT VARCHAR2) 
IS
    paramEntrada VARCHAR2(500);
	numERROR INTEGER;
	tipoERROR VARCHAR2(100);
    VACIO_ERROR EXCEPTION;
    ID_NULL Provincia.ID_Provincia%TYPE;

BEGIN
    paramEntrada:= P_ID;
    
    /* Comprobamos que ningn valor sea nulo */
    IF (P_ID IS NULL)
    THEN
        RAISE VACIO_ERROR;
    END IF;
    /* Comprobamos que exista esa Provincia  */
    BEGIN
        SELECT ID_Provincia
        INTO ID_NULL
        FROM Provincia
        WHERE ID_Provincia = P_ID
        AND ROWNUM = 1;
        EXCEPTION
            WHEN NO_DATA_FOUND THEN
            RAISE NO_DATA_FOUND;
        END;
    /* Borramos si ha pasado las anteriores comprobaciones */
    DELETE FROM Provincia WHERE ID_Provincia= P_ID;
    RSP:= 'OK';
    COMMIT;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
EXCEPTION
    WHEN VACIO_ERROR THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL VALOR NO PUEDE SER NULO';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN NO_DATA_FOUND THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - LA PROVINCIA NO EXISTE';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN OTHERS THEN
    numERROR:= SQLCODE;
    tipoERROR:= SUBSTR(SQLERRM, 1, 90);
    RSP := TO_CHAR(numERROR)||' - '||tipoERROR;
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
END;
/

/************************************************************
* Procedimiento para MODIFICAR una Provincia en la BD
* Entrada: ID_Provincia, nombrePro, EscanosGen, EscanosGen
* Salida: OK si se realiza con xito
*         ERROR+TIPO DE ERROR en caso contrario
* Pre-condicin: la Provincia DEBE existir 
* Post-condicin: la Provincia es MODIFICADA
************************************************************/

CREATE OR REPLACE PROCEDURE modProvincia(
	P_ID IN Provincia.ID_Provincia%TYPE,
	P_Nombre IN Provincia.nombrePro %TYPE,
	P_EscanosGen IN Provincia.EscanosGen%TYPE,
	P_EscanosAut IN Provincia.EscanosAut%TYPE,
	RSP OUT VARCHAR2) 
IS
    paramEntrada VARCHAR2(500);
	numERROR INTEGER;
	tipoERROR VARCHAR2(100);
    VACIO_ERROR EXCEPTION;
    nombreDUPLICADO Provincia.nombrePro%TYPE;

BEGIN
    paramEntrada:= P_ID||'- Nombre: '||P_Nombre||' - EscaosGen: '||P_EscanosGen||' - EscaosAut'||P_EscanosAut;
    
    /* Comprobamos que el ID no sea nulo o bien que los otros atributos no sean nulos a la vez */
    IF (P_ID IS NULL) OR ((P_Nombre IS NULL) AND (P_EscanosGen IS NULL) AND (P_EscanosAut IS NULL))
    THEN
        RAISE VACIO_ERROR;
    END IF;
    
    /* Comprobamos que exista el Provincia a modificar  */
    BEGIN
        SELECT nombrePro
        INTO nombreDUPLICADO
        FROM Provincia
        WHERE ID_Provincia = P_ID
        AND ROWNUM = 1;
        EXCEPTION
            WHEN NO_DATA_FOUND THEN
            RAISE DUP_VAL_ON_INDEX;
    END;
    
    /* Si ha pasado las comprobaciones anteriores, actualizamos */
    UPDATE Provincia SET nombrePro= P_Nombre, escanosGen= P_EscanosGen, escanosAut= P_EscanosAut WHERE ID_Provincia= P_ID;
    RSP:= 'OK';
    COMMIT;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
EXCEPTION
    WHEN VACIO_ERROR THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL VALOR NO PUEDE SER NULO';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN DUP_VAL_ON_INDEX THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - PROVINCIA DUPLICADA / YA EXISTE';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN OTHERS THEN
    numERROR:= SQLCODE;
    tipoERROR:= SUBSTR(SQLERRM, 1, 90);
    RSP := TO_CHAR(numERROR)||' - '||tipoERROR;
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
END;
/

/************************************************************
* Procedimiento para AADIR una Localidad a la BD
* Entrada: ID_Provincia, ID_Localidad, nombreLoc, EscanosLoc
* Salida: OK si se realiza con xito
*         ERROR+TIPO DE ERROR en caso contrario
* Pre-condicin: la Localidad NO debe existir 
* Post-condicin: la Localidad es dada de ALTA
************************************************************/

CREATE OR REPLACE PROCEDURE altaLocalidad(
	P_IDPro IN Localidad.ID_Provincia%TYPE,
	P_IDLoc IN Localidad.ID_Localidad%TYPE,
	P_Nombre IN Localidad.nombreLoc%TYPE,
	P_EscanosLoc IN Localidad.EscanosLoc%TYPE,
	RSP OUT VARCHAR2) 
IS
    paramEntrada VARCHAR2(500);
	numERROR INTEGER;
	tipoERROR VARCHAR2(100);
    VACIO_ERROR EXCEPTION;

BEGIN
    paramEntrada:= P_IDPro||'-'||P_IDLoc||'- Nombre: '||P_Nombre||' - EscaosLoc: '||P_EscanosLoc;
	
    /* Comprobamos que ningn valor sea nulo */
    IF (P_IDPro IS NULL) OR (P_IDLoc IS NULL) OR (P_Nombre IS NULL) OR (P_EscanosLoc IS NULL)
    THEN
        RAISE VACIO_ERROR;
    END IF;
	  
	/*Realizamos la insercin del valor*/
    INSERT INTO Localidad VALUES (P_IDPro, P_IDLoc, P_Nombre, P_EscanosLoc);
    RSP:= 'OK';
    COMMIT;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
EXCEPTION  
    WHEN DUP_VAL_ON_INDEX THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - LA LOCALIDAD EST DUPLICADA / YA EXISTE';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN VACIO_ERROR THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL VALOR NO PUEDE SER NULO';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN OTHERS THEN
    numERROR:= SQLCODE;
    tipoERROR:= SUBSTR(SQLERRM, 1, 90);
    RSP := TO_CHAR(numERROR)||' - '||tipoERROR;
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
END;
/


/************************************************************
* Procedimiento para dar de BAJA una Localidad EN la BD
* Entrada: ID_Provincia, ID_Localidad
* Salida: OK si se realiza con xito
*         ERROR+TIPO DE ERROR en caso contrario
* Pre-condicin: la Localidad DEBE existir 
* Post-condicin: la Localidad se ELIMINA
************************************************************/

CREATE OR REPLACE PROCEDURE bajaLocalidad(
	P_IDPro IN Localidad.ID_Provincia%TYPE,
	P_IDLoc IN Localidad.ID_Localidad%TYPE,
	RSP OUT VARCHAR2) 
IS
    paramEntrada VARCHAR2(500);
	numERROR INTEGER;
	tipoERROR VARCHAR2(100);
    VACIO_ERROR EXCEPTION;
    ID_NULLPro Localidad.ID_Provincia%TYPE;
	ID_NULLLoc Localidad.ID_Localidad%TYPE;

BEGIN
    paramEntrada:= P_IDPro||P_IDLoc;
    
    /* Comprobamos que ningn valor sea nulo */
    IF (P_IDPro IS NULL) OR (P_IDLoc IS NULL)
    THEN
        RAISE VACIO_ERROR;
    END IF;
    /* Comprobamos que exista esa Localidad  */
    BEGIN
        SELECT ID_Provincia, ID_Localidad
        INTO ID_NULLPro, ID_NULLLoc
        FROM Localidad
        WHERE ID_Localidad = P_IDLoc AND ID_Provincia = P_IDPro
        AND ROWNUM = 1;
        EXCEPTION
            WHEN NO_DATA_FOUND THEN
            RAISE NO_DATA_FOUND;
        END;
		
    /* Borramos si ha pasado las anteriores comprobaciones */
    DELETE FROM Localidad WHERE ID_Provincia = P_IDPro AND ID_Localidad= P_IDLoc;
    RSP:= 'OK';
    COMMIT;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
EXCEPTION
    WHEN VACIO_ERROR THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL VALOR NO PUEDE SER NULO';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN NO_DATA_FOUND THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - LA LOCALIDAD NO EXISTE';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN OTHERS THEN
    numERROR:= SQLCODE;
    tipoERROR:= SUBSTR(SQLERRM, 1, 90);
    RSP := TO_CHAR(numERROR)||' - '||tipoERROR;
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
END;
/

/************************************************************
* Procedimiento para MODIFICAR una Localidad en la BD
* Entrada: ID_Provincia, ID_Localidad, nombreLoc, EscanosLoc
* Salida: OK si se realiza con xito
*         ERROR+TIPO DE ERROR en caso contrario
* Pre-condicin: la Localidad DEBE existir 
* Post-condicin: la Localidad es MODIFICADA
************************************************************/

CREATE OR REPLACE PROCEDURE modLocalidad(
	P_IDPro IN Localidad.ID_Provincia%TYPE,
	P_IDLoc IN Localidad.ID_Localidad%TYPE,
	P_Nombre IN Localidad.nombreLoc%TYPE,
	P_EscanosLoc IN Localidad.EscanosLoc%TYPE,
	RSP OUT VARCHAR2) 
IS
    paramEntrada VARCHAR2(500);
	numERROR INTEGER;
	tipoERROR VARCHAR2(100);
    VACIO_ERROR EXCEPTION;
    nombreDUPLICADO Localidad.nombreLoc%TYPE;

BEGIN
    paramEntrada:= P_IDPro||'-'||P_IDLoc||'- Nombre: '||P_Nombre||' - EscaosLoc: '||P_EscanosLoc;
	
    /* Comprobamos que el ID no sea nulo o bien que los otros atributos no sean nulos a la vez */
    IF (P_IDPro IS NULL) OR (P_IDLoc IS NULL) OR ((P_Nombre IS NULL) AND (P_EscanosLoc IS NULL))
    THEN
        RAISE VACIO_ERROR;
    END IF;
    
    /* Comprobamos que exista el Localidad a modificar  */
    BEGIN
        SELECT nombreLoc
        INTO nombreDUPLICADO
        FROM Localidad
        WHERE ID_Localidad = P_IDLoc AND ID_Provincia = P_IDPro
        AND ROWNUM = 1;
        EXCEPTION
            WHEN NO_DATA_FOUND THEN
            RAISE NO_DATA_FOUND;
        END;
    
    /* Si ha pasado las comprobaciones anteriores, actualizamos */
    UPDATE Localidad SET nombreLoc= P_Nombre, escanosLoc= P_EscanosLoc WHERE ID_Localidad= P_IDLoc AND ID_Provincia = P_IDPro;
    RSP:= 'OK';
    COMMIT;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
EXCEPTION
    WHEN VACIO_ERROR THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL VALOR NO PUEDE SER NULO';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN NO_DATA_FOUND THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - LOCALIDAD NO ENCONTRADA - NO EXISTE';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN OTHERS THEN
    numERROR:= SQLCODE;
    tipoERROR:= SUBSTR(SQLERRM, 1, 90);
    RSP := TO_CHAR(numERROR)||' - '||tipoERROR;
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
END;
/


/************************************************************
* Procedimiento para AADIR un PartidoPolitico a la BD
* Entrada: ID_Partido, nombrePartido, siglas
* Salida: OK si se realiza con xito
*         ERROR+TIPO DE ERROR en caso contrario
* Pre-condicin: el PartidoPolitico NO debe existir 
* Post-condicin: el PartidoPolitico es dada de ALTA
************************************************************/

CREATE OR REPLACE PROCEDURE altaPartidoPolitico(
	P_IDPartido IN PartidoPolitico.ID_Partido%TYPE,
	P_nombre IN PartidoPolitico.nombrePartido%TYPE,
	P_siglas IN PartidoPolitico.siglas%TYPE,
	RSP OUT VARCHAR2) 
IS
    paramEntrada VARCHAR2(500);
	numERROR INTEGER;
	tipoERROR VARCHAR2(100);
    VACIO_ERROR EXCEPTION;

BEGIN
    paramEntrada:= P_IDPartido||'- Nombre:'||P_nombre||'- siglas: '||P_siglas;
	
    /* Comprobamos que ningn valor sea nulo */
    IF (P_IDPartido IS NULL) OR (P_nombre IS NULL) OR (P_siglas IS NULL)
    THEN
        RAISE VACIO_ERROR;
    END IF;
	  
	/*Realizamos la insercin del valor*/
    INSERT INTO PartidoPolitico VALUES (P_IDPartido, P_nombre, P_siglas);
    RSP:= 'OK';
    COMMIT;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
EXCEPTION  
    WHEN DUP_VAL_ON_INDEX THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - El PARTIDO EST DUPLICADO / YA EXISTE';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN VACIO_ERROR THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL VALOR NO PUEDE SER NULO';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN OTHERS THEN
    numERROR:= SQLCODE;
    tipoERROR:= SUBSTR(SQLERRM, 1, 90);
    RSP := TO_CHAR(numERROR)||' - '||tipoERROR;
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
END;
/


/************************************************************
* Procedimiento para dar de BAJA un Partido Politico EN la BD
* Entrada: ID_Partido
* Salida: OK si se realiza con xito
*         ERROR+TIPO DE ERROR en caso contrario
* Pre-condicin: el Partido Politico DEBE existir 
* Post-condicin: el Partido Politico se ELIMINA
************************************************************/

CREATE OR REPLACE PROCEDURE bajaPartidoPolitico(
	P_IDPartido IN PartidoPolitico.ID_Partido%TYPE,
	RSP OUT VARCHAR2) 
IS
    paramEntrada VARCHAR2(500);
	numERROR INTEGER;
	tipoERROR VARCHAR2(100);
    VACIO_ERROR EXCEPTION;
    ID_NULLPartido PartidoPolitico.ID_Partido%TYPE;

BEGIN
    paramEntrada:= P_IDPartido;
    
    /* Comprobamos que el valor no sea nulo */
    IF (P_IDPartido IS NULL) 
    THEN
        RAISE VACIO_ERROR;
    END IF;
    /* Comprobamos que exista ese Partido Politico  */
    BEGIN
        SELECT ID_Partido
        INTO ID_NULLPartido
        FROM PartidoPolitico
        WHERE ID_Partido = P_IDPartido
        AND ROWNUM = 1;
        EXCEPTION
            WHEN NO_DATA_FOUND THEN
            RAISE NO_DATA_FOUND;
        END;
		
    /* Borramos si ha pasado las anteriores comprobaciones */
    DELETE FROM PartidoPolitico WHERE ID_Partido = P_IDPartido;
    RSP:= 'OK';
    COMMIT;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
EXCEPTION
    WHEN VACIO_ERROR THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL VALOR NO PUEDE SER NULO';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN NO_DATA_FOUND THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL PARTIDO NO EXISTE';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN OTHERS THEN
    numERROR:= SQLCODE;
    tipoERROR:= SUBSTR(SQLERRM, 1, 90);
    RSP := TO_CHAR(numERROR)||' - '||tipoERROR;
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
END;
/

/************************************************************
* Procedimiento para MODIFICAR un Partido Politico en la BD
* Entrada: ID_Partido, nombrePartido, siglas
* Salida: OK si se realiza con xito
*         ERROR+TIPO DE ERROR en caso contrario
* Pre-condicin: El Partido Politico DEBE existir 
* Post-condicin: El Partido Politico es MODIFICADO
************************************************************/

CREATE OR REPLACE PROCEDURE modPartidoPolitico(
	P_IDPartido IN PartidoPolitico.ID_Partido%TYPE,
	P_nombre IN PartidoPolitico.nombrePartido%TYPE,
	P_siglas IN PartidoPolitico.siglas%TYPE,
	RSP OUT VARCHAR2) 
IS
    paramEntrada VARCHAR2(500);
	numERROR INTEGER;
	tipoERROR VARCHAR2(100);
    VACIO_ERROR EXCEPTION;
    nombreDUPLICADO PartidoPolitico.nombrePartido%TYPE;

BEGIN
    paramEntrada:= P_IDPartido||'- Nombre:'||P_nombre||'- siglas: '||P_siglas;
	
    /* Comprobamos que el ID no sea nulo o bien que los otros atributos no sean nulos a la vez */
    IF (P_IDPartido IS NULL) OR ((P_nombre IS NULL) AND (P_siglas IS NULL))
    THEN
        RAISE VACIO_ERROR;
    END IF;
    
    /* Comprobamos que exista el PartidoPolitico a modificar  */
    BEGIN
        SELECT nombrePartido
        INTO nombreDUPLICADO
        FROM PartidoPolitico
        WHERE ID_Partido = P_IDPartido
        AND ROWNUM = 1;
        EXCEPTION
            WHEN NO_DATA_FOUND THEN
            RAISE DUP_VAL_ON_INDEX;
        END;
    
    /* Si ha pasado las comprobaciones anteriores, actualizamos */
    UPDATE PartidoPolitico SET nombrePartido= P_nombre, siglas= P_siglas WHERE ID_Partido= P_IDPartido;
    RSP:= 'OK';
    COMMIT;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
EXCEPTION
    WHEN VACIO_ERROR THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL VALOR NO PUEDE SER NULO';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN DUP_VAL_ON_INDEX THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL PARTIDO ESTA DUPLICADO';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN OTHERS THEN
    numERROR:= SQLCODE;
    tipoERROR:= SUBSTR(SQLERRM, 1, 90);
    RSP := TO_CHAR(numERROR)||' - '||tipoERROR;
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
END;
/


/************************************************************
* Procedimiento para AADIR un Metodo de Calculo a la BD
* Entrada: ID_Metodo, nombreMetodo
* Salida: OK si se realiza con xito
*         ERROR+TIPO DE ERROR en caso contrario
* Pre-condicin: el Metodo de Calculo NO debe existir 
* Post-condicin: el Metodo de Calculo es dada de ALTA
************************************************************/

CREATE OR REPLACE PROCEDURE altaMetodoCalculo(
	P_IDMetodo IN MetodoCalculo.ID_Metodo%TYPE,
	P_nombre IN MetodoCalculo.nombreMetodo%TYPE,
	RSP OUT VARCHAR2) 
IS
    paramEntrada VARCHAR2(500);
	numERROR INTEGER;
	tipoERROR VARCHAR2(100);
    VACIO_ERROR EXCEPTION;

BEGIN
    paramEntrada:= P_IDMetodo||'- Nombre:'||P_nombre;
	
    /* Comprobamos que ningn valor sea nulo */
    IF (P_IDMetodo IS NULL) OR (P_nombre IS NULL)
    THEN
        RAISE VACIO_ERROR;
    END IF;
	  
	/*Realizamos la insercin del valor*/
    INSERT INTO MetodoCalculo VALUES (P_IDMetodo, P_nombre);
    RSP:= 'OK';
    COMMIT;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
EXCEPTION  
    WHEN DUP_VAL_ON_INDEX THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - El METODO EST DUPLICADO / YA EXISTE';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN VACIO_ERROR THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL VALOR NO PUEDE SER NULO';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN OTHERS THEN
    numERROR:= SQLCODE;
    tipoERROR:= SUBSTR(SQLERRM, 1, 90);
    RSP := TO_CHAR(numERROR)||' - '||tipoERROR;
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
END;
/


/************************************************************
* Procedimiento para dar de BAJA un mtodo de clculo EN la BD
* Entrada: ID_Metodo
* Salida: OK si se realiza con xito
*         ERROR+TIPO DE ERROR en caso contrario
* Pre-condicin: el Mtodo de clculo DEBE existir 
* Post-condicin: el Mtodo de clculo se ELIMINA
************************************************************/

CREATE OR REPLACE PROCEDURE bajaMetodoCalculo(
	P_IDMetodo IN MetodoCalculo.ID_Metodo%TYPE,
	RSP OUT VARCHAR2) 
IS
    paramEntrada VARCHAR2(500);
	numERROR INTEGER;
	tipoERROR VARCHAR2(100);
    VACIO_ERROR EXCEPTION;
    ID_NULLMetodo MetodoCalculo.ID_Metodo%TYPE;

BEGIN
    paramEntrada:= P_IDMetodo;
    
    /* Comprobamos que el valor no sea nulo */
    IF (P_IDMetodo IS NULL) 
    THEN
        RAISE VACIO_ERROR;
    END IF;
    /* Comprobamos que exista ese Partido Politico  */
    BEGIN
        SELECT ID_Metodo
        INTO ID_NULLMetodo
        FROM MetodoCalculo
        WHERE ID_Metodo = P_IDMetodo
        AND ROWNUM = 1;
        EXCEPTION
            WHEN NO_DATA_FOUND THEN
            RAISE NO_DATA_FOUND;
        END;
		
    /* Borramos si ha pasado las anteriores comprobaciones */
    DELETE FROM MetodoCalculo WHERE ID_Metodo = P_IDMetodo;
    RSP:= 'OK';
    COMMIT;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
EXCEPTION
    WHEN VACIO_ERROR THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL VALOR NO PUEDE SER NULO';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN NO_DATA_FOUND THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL METODO DE CALCULO NO EXISTE';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN OTHERS THEN
    numERROR:= SQLCODE;
    tipoERROR:= SUBSTR(SQLERRM, 1, 90);
    RSP := TO_CHAR(numERROR)||' - '||tipoERROR;
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
END;
/

/************************************************************
* Procedimiento para MODIFICAR un Mtodo de Clculo en la BD
* Entrada: ID_Metodo, nombreMetodo
* Salida: OK si se realiza con xito
*         ERROR+TIPO DE ERROR en caso contrario
* Pre-condicin: El  Mtodo de Clculo DEBE existir 
* Post-condicin: El  Mtodo de Clculo es MODIFICADO
************************************************************/

CREATE OR REPLACE PROCEDURE modMetodoCalculo(
	P_IDMetodo IN MetodoCalculo.ID_Metodo%TYPE,
	P_nombre IN MetodoCalculo.nombreMetodo%TYPE,
	RSP OUT VARCHAR2) 
IS
    paramEntrada VARCHAR2(500);
	numERROR INTEGER;
	tipoERROR VARCHAR2(100);
    VACIO_ERROR EXCEPTION;
    nombreDUPLICADO MetodoCalculo.nombreMetodo%TYPE;

BEGIN
    paramEntrada:= P_IDMetodo||'- Nombre:'||P_nombre;
	
    /* Comprobamos que el ID no sea nulo o bien que los otros atributos no sean nulos a la vez */
    IF (P_IDMetodo IS NULL) OR (P_nombre IS NULL)
    THEN
        RAISE VACIO_ERROR;
    END IF;
    
    /* Comprobamos que exista el MetodoCalculo a modificar  */
    BEGIN
        SELECT nombreMetodo
        INTO nombreDUPLICADO
        FROM MetodoCalculo
        WHERE ID_Metodo = P_IDMetodo
        AND ROWNUM = 1;
        EXCEPTION
            WHEN NO_DATA_FOUND THEN
            RAISE NO_DATA_FOUND;
        END;
    
    /* Si ha pasado las comprobaciones anteriores, actualizamos */
    UPDATE MetodoCalculo SET nombreMetodo= P_nombre WHERE ID_Metodo= P_IDMetodo;
    RSP:= 'OK';
    COMMIT;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
EXCEPTION
    WHEN VACIO_ERROR THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL VALOR NO PUEDE SER NULO';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN NO_DATA_FOUND THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL METODO DE CALCULO NO SE ENCUENTRA - NO EXISTE';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN OTHERS THEN
    numERROR:= SQLCODE;
    tipoERROR:= SUBSTR(SQLERRM, 1, 90);
    RSP := TO_CHAR(numERROR)||' - '||tipoERROR;
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
END;
/


/************************************************************
* Procedimiento para AADIR una Regla de Proceso Electoral a la BD
* Entrada: ID_RP, nombreRP, descripcionRP
* Salida: OK si se realiza con xito
*         ERROR+TIPO DE ERROR en caso contrario
* Pre-condicin: la Regla de Proceso NO debe existir 
* Post-condicin: la Regla de Proceso es dada de ALTA
************************************************************/

CREATE OR REPLACE PROCEDURE altaReglaProceso(
	P_IDRP IN ReglaProceso.ID_RP%TYPE,
	P_nombre IN ReglaProceso.nombreRP%TYPE,
	P_descripcion IN ReglaProceso.descripcionRP%TYPE,
	RSP OUT VARCHAR2) 
IS
    paramEntrada VARCHAR2(500);
	numERROR INTEGER;
	tipoERROR VARCHAR2(100);
    VACIO_ERROR EXCEPTION;

BEGIN
    paramEntrada:= P_IDRP||'- Nombre:'||P_nombre||' - Descripcin: '||P_descripcion;
	
    /* Comprobamos que ningn valor sea nulo */
    IF (P_IDRP IS NULL) OR (P_nombre IS NULL) OR (P_descripcion IS NULL)
    THEN
        RAISE VACIO_ERROR;
    END IF;
	  
	/*Realizamos la insercin del valor*/
    INSERT INTO ReglaProceso VALUES (P_IDRP, P_nombre, P_descripcion);
    RSP:= 'OK';
    COMMIT;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
EXCEPTION  
    WHEN DUP_VAL_ON_INDEX THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - LA REGLA DE PROCESO EST DUPLICADA / YA EXISTE';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN VACIO_ERROR THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL VALOR NO PUEDE SER NULO';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN OTHERS THEN
    numERROR:= SQLCODE;
    tipoERROR:= SUBSTR(SQLERRM, 1, 90);
    RSP := TO_CHAR(numERROR)||' - '||tipoERROR;
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
END;
/


/************************************************************
* Procedimiento para dar de BAJA una Regla de Proceso Electoral EN la BD
* Entrada: ID_RP
* Salida: OK si se realiza con xito
*         ERROR+TIPO DE ERROR en caso contrario
* Pre-condicin: la Regla de Proceso DEBE existir 
* Post-condicin: la Regla de Proceso se ELIMINA
************************************************************/

CREATE OR REPLACE PROCEDURE bajaReglaProceso(
	P_IDRP IN ReglaProceso.ID_RP%TYPE,
	RSP OUT VARCHAR2) 
IS
    paramEntrada VARCHAR2(500);
	numERROR INTEGER;
	tipoERROR VARCHAR2(100);
    VACIO_ERROR EXCEPTION;
    ID_NULL ReglaProceso.ID_RP%TYPE;

BEGIN
    paramEntrada:= P_IDRP;
    
    /* Comprobamos que el valor no sea nulo */
    IF (P_IDRP IS NULL) 
    THEN
        RAISE VACIO_ERROR;
    END IF;
    /* Comprobamos que exista ese Partido Politico  */
    BEGIN
        SELECT ID_RP
        INTO ID_NULL
        FROM ReglaProceso
        WHERE ID_RP = P_IDRP
        AND ROWNUM = 1;
        EXCEPTION
            WHEN NO_DATA_FOUND THEN
            RAISE NO_DATA_FOUND;
        END;
		
    /* Borramos si ha pasado las anteriores comprobaciones */
    DELETE FROM ReglaProceso WHERE ID_RP = P_IDRP;
    RSP:= 'OK';
    COMMIT;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
EXCEPTION
    WHEN VACIO_ERROR THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL VALOR NO PUEDE SER NULO';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN NO_DATA_FOUND THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - la Regla de Proceso NO EXISTE';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN OTHERS THEN
    numERROR:= SQLCODE;
    tipoERROR:= SUBSTR(SQLERRM, 1, 90);
    RSP := TO_CHAR(numERROR)||' - '||tipoERROR;
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
END;
/

/************************************************************
* Procedimiento para MODIFICAR una Regla de Proceso Electoral en la BD
* Entrada: ID_RP, nombreRP, descripcionRP
* Salida: OK si se realiza con xito
*         ERROR+TIPO DE ERROR en caso contrario
* Pre-condicin: la Regla de Proceso DEBE existir 
* Post-condicin: la Regla de Proceso es MODIFICADO
************************************************************/

CREATE OR REPLACE PROCEDURE modReglaProceso(
	P_IDRP IN ReglaProceso.ID_RP%TYPE,
	P_nombre IN ReglaProceso.nombreRP%TYPE,
	P_descripcion IN ReglaProceso.descripcionRP%TYPE,
	RSP OUT VARCHAR2) 
IS
    paramEntrada VARCHAR2(500);
	numERROR INTEGER;
	tipoERROR VARCHAR2(100);
    VACIO_ERROR EXCEPTION;
    nombreDUPLICADO ReglaProceso.nombreRP%TYPE;

BEGIN
    paramEntrada:= P_IDRP||'- Nombre:'||P_nombre||' - Descripcin: '||P_descripcion;
	
    /* Comprobamos que el ID no sea nulo o bien que los otros atributos no sean nulos a la vez */
    IF (P_IDRP IS NULL) OR ((P_nombre IS NULL) AND (P_descripcion IS NULL))
    THEN
        RAISE VACIO_ERROR;
    END IF;
    
    /* Comprobamos que exista el ReglaProceso a modificar  */
    BEGIN
        SELECT nombreRP
        INTO nombreDUPLICADO
        FROM ReglaProceso
        WHERE ID_RP = P_IDRP
        AND ROWNUM = 1;
        EXCEPTION
            WHEN NO_DATA_FOUND THEN
            RAISE NO_DATA_FOUND;
        END;
    
    /* Si ha pasado las comprobaciones anteriores, actualizamos */
    UPDATE ReglaProceso SET nombreRP= P_nombre, descripcionRP= P_descripcion WHERE ID_RP= P_IDRP;
    RSP:= 'OK';
    COMMIT;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
EXCEPTION
    WHEN VACIO_ERROR THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL VALOR NO PUEDE SER NULO';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN NO_DATA_FOUND THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - la Regla de Proceso NO SE ENCUENTRA - NO EXISTE';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN OTHERS THEN
    numERROR:= SQLCODE;
    tipoERROR:= SUBSTR(SQLERRM, 1, 90);
    RSP := TO_CHAR(numERROR)||' - '||tipoERROR;
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
END;
/


/************************************************************
* Procedimiento para AADIR una Regla de Partido Poltico a la BD
* Entrada: ID_RPP, nombreRPP, descripcionRPP
* Salida: OK si se realiza con xito
*         ERROR+TIPO DE ERROR en caso contrario
* Pre-condicin: la Regla de Partido Politico NO debe existir 
* Post-condicin: la Regla de Partido Politico es dada de ALTA
************************************************************/

CREATE OR REPLACE PROCEDURE altaReglasPartidosPoliticos(
	P_IDRP IN ReglasPartidosPoliticos.ID_RPP%TYPE,
	P_nombre IN ReglasPartidosPoliticos.nombreRPP%TYPE,
	P_descripcion IN ReglasPartidosPoliticos.descripcionRPP%TYPE,
	RSP OUT VARCHAR2) 
IS
    paramEntrada VARCHAR2(500);
	numERROR INTEGER;
	tipoERROR VARCHAR2(100);
    VACIO_ERROR EXCEPTION;

BEGIN
    paramEntrada:= P_IDRP||'- Nombre:'||P_nombre||' - Descripcin: '||P_descripcion;
	
    /* Comprobamos que ningn valor sea nulo */
    IF (P_IDRP IS NULL) OR (P_nombre IS NULL) OR (P_descripcion IS NULL)
    THEN
        RAISE VACIO_ERROR;
    END IF;
	  
	/*Realizamos la insercin del valor*/
    INSERT INTO ReglasPartidosPoliticos VALUES (P_IDRP, P_nombre, P_descripcion);
    RSP:= 'OK';
    COMMIT;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
EXCEPTION  
    WHEN DUP_VAL_ON_INDEX THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - LA REGLA DE PARTIDO POLITICO EST DUPLICADA / YA EXISTE';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN VACIO_ERROR THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL VALOR NO PUEDE SER NULO';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN OTHERS THEN
    numERROR:= SQLCODE;
    tipoERROR:= SUBSTR(SQLERRM, 1, 90);
    RSP := TO_CHAR(numERROR)||' - '||tipoERROR;
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
END;
/


/************************************************************
* Procedimiento para dar de BAJA una Regla de Partido Poltico EN la BD
* Entrada: ID_RPP
* Salida: OK si se realiza con xito
*         ERROR+TIPO DE ERROR en caso contrario
* Pre-condicin: la Regla de Partido Politico DEBE existir 
* Post-condicin: la Regla de Partido Politico se ELIMINA
************************************************************/

CREATE OR REPLACE PROCEDURE bajaReglasPartidosPoliticos(
	P_IDRP IN ReglasPartidosPoliticos.ID_RPP%TYPE,
	RSP OUT VARCHAR2) 
IS
    paramEntrada VARCHAR2(500);
	numERROR INTEGER;
	tipoERROR VARCHAR2(100);
    VACIO_ERROR EXCEPTION;
    ID_NULL ReglasPartidosPoliticos.ID_RPP%TYPE;

BEGIN
    paramEntrada:= P_IDRP;
    
    /* Comprobamos que el valor no sea nulo */
    IF (P_IDRP IS NULL) 
    THEN
        RAISE VACIO_ERROR;
    END IF;
    /* Comprobamos que exista ese Partido Politico  */
    BEGIN
        SELECT ID_RPP
        INTO ID_NULL
        FROM ReglasPartidosPoliticos
        WHERE ID_RPP = P_IDRP
        AND ROWNUM = 1;
        EXCEPTION
            WHEN NO_DATA_FOUND THEN
            RAISE NO_DATA_FOUND;
        END;
		
    /* Borramos si ha pasado las anteriores comprobaciones */
    DELETE FROM ReglasPartidosPoliticos WHERE ID_RPP = P_IDRP;
    RSP:= 'OK';
    COMMIT;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
EXCEPTION
    WHEN VACIO_ERROR THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL VALOR NO PUEDE SER NULO';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN NO_DATA_FOUND THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - la Regla de Partido Politico NO EXISTE';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN OTHERS THEN
    numERROR:= SQLCODE;
    tipoERROR:= SUBSTR(SQLERRM, 1, 90);
    RSP := TO_CHAR(numERROR)||' - '||tipoERROR;
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
END;
/

/************************************************************
* Procedimiento para MODIFICAR una Regla de Partido Poltico en la BD
* Entrada: ID_RPP, nombreRPP, descripcionRPP
* Salida: OK si se realiza con xito
*         ERROR+TIPO DE ERROR en caso contrario
* Pre-condicin: la Regla de Partido Politico DEBE existir 
* Post-condicin: la Regla de Partido Politico es MODIFICADO
************************************************************/

CREATE OR REPLACE PROCEDURE modReglasPartidosPoliticos(
	P_IDRP IN ReglasPartidosPoliticos.ID_RPP%TYPE,
	P_nombre IN ReglasPartidosPoliticos.nombreRPP%TYPE,
	P_descripcion IN ReglasPartidosPoliticos.descripcionRPP%TYPE,
	RSP OUT VARCHAR2) 
IS
    paramEntrada VARCHAR2(500);
	numERROR INTEGER;
	tipoERROR VARCHAR2(100);
    VACIO_ERROR EXCEPTION;
    nombreDUPLICADO ReglasPartidosPoliticos.nombreRPP%TYPE;

BEGIN
    paramEntrada:= P_IDRP||'- Nombre:'||P_nombre||' - Descripcin: '||P_descripcion;
	
    /* Comprobamos que el ID no sea nulo o bien que los otros atributos no sean nulos a la vez */
    IF (P_IDRP IS NULL) OR ((P_nombre IS NULL) AND (P_descripcion IS NULL))
    THEN
        RAISE VACIO_ERROR;
    END IF;
    
    /* Comprobamos que exista el ReglaPartido Politico a modificar  */
    BEGIN
        SELECT nombreRPP
        INTO nombreDUPLICADO
        FROM ReglasPartidosPoliticos
        WHERE ID_RPP = P_IDRP
        AND ROWNUM = 1;
        EXCEPTION
            WHEN NO_DATA_FOUND THEN
            RAISE NO_DATA_FOUND;
        END;
    
    /* Si ha pasado las comprobaciones anteriores, actualizamos */
    UPDATE ReglasPartidosPoliticos SET nombreRPP= P_nombre, descripcionRPP= P_descripcion WHERE ID_RPP= P_IDRP;
    RSP:= 'OK';
    COMMIT;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
EXCEPTION
    WHEN VACIO_ERROR THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL VALOR NO PUEDE SER NULO';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN NO_DATA_FOUND THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - la Regla de Partido Politico NO SE ENCUENTRA - NO EXISTE';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN OTHERS THEN
    numERROR:= SQLCODE;
    tipoERROR:= SUBSTR(SQLERRM, 1, 90);
    RSP := TO_CHAR(numERROR)||' - '||tipoERROR;
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
END;
/



/************************************************************
* Procedimiento para AADIR una Demarcacin electoral a la BD
* Entrada: ID_Demarcacion, nombreDemarcacion, numEscanos, ID_Proceso
* Salida: OK si se realiza con xito
*         ERROR+TIPO DE ERROR en caso contrario
* Pre-condicin: la Demarcacin Electoral NO debe existir 
* Post-condicin: la Demarcacin Electoral es dada de ALTA
************************************************************/

CREATE OR REPLACE PROCEDURE altaDemarcacionElectoral(
	P_IDDemarcacion IN DemarcacionElectoral.ID_Demarcacion%TYPE,
	P_nombre IN DemarcacionElectoral.nombreDemarcacion%TYPE,
	P_numEscanos IN DemarcacionElectoral.numEscanos%TYPE,
	P_proceso IN DemarcacionElectoral.ID_Proceso%TYPE,
	RSP OUT VARCHAR2) 
IS
    paramEntrada VARCHAR2(500);
	numERROR INTEGER;
	tipoERROR VARCHAR2(100);
    VACIO_ERROR EXCEPTION;

BEGIN
    paramEntrada:= P_IDDemarcacion||'- Nombre:'||P_nombre||' - Escaos: '||P_numEscanos||' - Proceso Electoral'||P_proceso;
	
    /* Comprobamos que ningn valor sea nulo */
    IF (P_IDDemarcacion IS NULL) OR (P_nombre IS NULL) OR (P_numEscanos IS NULL) OR (P_proceso IS NULL)
    THEN
        RAISE VACIO_ERROR;
    END IF;
	  
	/*Realizamos la insercin del valor*/
    INSERT INTO DemarcacionElectoral VALUES (P_IDDemarcacion, P_nombre, P_numEscanos, P_proceso);
    RSP:= 'OK';
    COMMIT;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
EXCEPTION  
    WHEN DUP_VAL_ON_INDEX THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - LA Demarcacin Electoral EST DUPLICADA / YA EXISTE';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN VACIO_ERROR THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL VALOR NO PUEDE SER NULO';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN OTHERS THEN
    numERROR:= SQLCODE;
    tipoERROR:= SUBSTR(SQLERRM, 1, 90);
    RSP := TO_CHAR(numERROR)||' - '||tipoERROR;
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
END;
/


/************************************************************
* Procedimiento para dar de BAJA una Demarcacin electoral EN la BD
* Entrada: ID_Demarcacion
* Salida: OK si se realiza con xito
*         ERROR+TIPO DE ERROR en caso contrario
* Pre-condicin: la Demarcacin Electoral DEBE existir 
* Post-condicin: la Demarcacin Electoral se ELIMINA
************************************************************/

CREATE OR REPLACE PROCEDURE bajaDemarcacionElectoral(
	P_IDDemarcacion IN DemarcacionElectoral.ID_Demarcacion%TYPE,
	RSP OUT VARCHAR2) 
IS
    paramEntrada VARCHAR2(500);
	numERROR INTEGER;
	tipoERROR VARCHAR2(100);
    VACIO_ERROR EXCEPTION;
    ID_NULL DemarcacionElectoral.ID_Demarcacion%TYPE;

BEGIN
    paramEntrada:= P_IDDemarcacion;
    
    /* Comprobamos que el valor no sea nulo */
    IF (P_IDDemarcacion IS NULL) 
    THEN
        RAISE VACIO_ERROR;
    END IF;
    /* Comprobamos que exista esa demarcacin  */
    BEGIN
        SELECT ID_Demarcacion
        INTO ID_NULL
        FROM DemarcacionElectoral
        WHERE ID_Demarcacion = P_IDDemarcacion
        AND ROWNUM = 1;
        EXCEPTION
            WHEN NO_DATA_FOUND THEN
            RAISE NO_DATA_FOUND;
        END;
		
    /* Borramos si ha pasado las anteriores comprobaciones */
    DELETE FROM DemarcacionElectoral WHERE ID_Demarcacion = P_IDDemarcacion;
    RSP:= 'OK';
    COMMIT;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
EXCEPTION
    WHEN VACIO_ERROR THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL VALOR NO PUEDE SER NULO';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN NO_DATA_FOUND THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - la Demarcacin Electoral NO EXISTE';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN OTHERS THEN
    numERROR:= SQLCODE;
    tipoERROR:= SUBSTR(SQLERRM, 1, 90);
    RSP := TO_CHAR(numERROR)||' - '||tipoERROR;
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
END;
/

/************************************************************
* Procedimiento para MODIFICAR una Demarcacin electoral en la BD
* Entrada: ID_Demarcacion, nombreDemarcacion, numEscanos, ID_Proceso
* Salida: OK si se realiza con xito
*         ERROR+TIPO DE ERROR en caso contrario
* Pre-condicin: la Demarcacin Electoral DEBE existir 
* Post-condicin: la Demarcacin Electoral es MODIFICADO
************************************************************/

CREATE OR REPLACE PROCEDURE modDemarcacionElectoral(
	P_IDDemarcacion IN DemarcacionElectoral.ID_Demarcacion%TYPE,
	P_nombre IN DemarcacionElectoral.nombreDemarcacion%TYPE,
	P_numEscanos IN DemarcacionElectoral.numEscanos%TYPE,
	P_proceso IN DemarcacionElectoral.ID_Proceso%TYPE,
	RSP OUT VARCHAR2) 
IS
    paramEntrada VARCHAR2(500);
	numERROR INTEGER;
	tipoERROR VARCHAR2(100);
    VACIO_ERROR EXCEPTION;
	ID_duplicado DemarcacionElectoral.ID_Demarcacion%TYPE;

BEGIN
    paramEntrada:= P_IDDemarcacion||'- Nombre:'||P_nombre||' - Escaos: '||P_numEscanos||' - Proceso Electoral'||P_proceso;
	
    /* Comprobamos que el ID no sea nulo o bien que los otros atributos no sean nulos a la vez */
    IF (P_IDDemarcacion IS NULL) OR ((P_nombre IS NULL) AND (P_numEscanos IS NULL) AND (P_proceso IS NULL))
    THEN
        RAISE VACIO_ERROR;
    END IF;
    
    /* Comprobamos que exista la demarcacin electoral a modificar  */
    BEGIN
        SELECT ID_Demarcacion
        INTO ID_duplicado
        FROM DemarcacionElectoral
        WHERE ID_Demarcacion = P_IDDemarcacion
        AND ROWNUM = 1;
        EXCEPTION
            WHEN NO_DATA_FOUND THEN
            RAISE NO_DATA_FOUND;
        END;
    
    /* Si ha pasado las comprobaciones anteriores, actualizamos */
    UPDATE DemarcacionElectoral SET nombreDemarcacion= P_nombre, numEscanos= P_numEscanos, ID_Proceso= P_proceso WHERE ID_Demarcacion= P_IDDemarcacion;
    RSP:= 'OK';
    COMMIT;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
EXCEPTION
    WHEN VACIO_ERROR THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL VALOR NO PUEDE SER NULO';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN NO_DATA_FOUND THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - la Demarcacin Electoral NO SE ENCUENTRA - NO EXISTE';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN OTHERS THEN
    numERROR:= SQLCODE;
    tipoERROR:= SUBSTR(SQLERRM, 1, 90);
    RSP := TO_CHAR(numERROR)||' - '||tipoERROR;
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
END;
/


/************************************************************
* Procedimiento para AADIR un Centro de Votacin a la BD
* Entrada: ID_Centro, nombreCentro, direccionCentro, 
*          ID_Provincia, ID_Localidad, ID_Demarcacion
* Salida: OK si se realiza con xito
*         ERROR+TIPO DE ERROR en caso contrario
* Pre-condicin: el Centro de Votacin NO debe existir,
*               la localidad debe existir
* Post-condicin: el Centro de Votacin es dada de ALTA
************************************************************/

CREATE OR REPLACE PROCEDURE altaCentroVotacion(
	P_IDCentro IN CentroVotacion.ID_Centro%TYPE,
	P_nombre IN CentroVotacion.nombreCentro%TYPE,
	P_direccionCentro IN CentroVotacion.direccionCentro%TYPE,
	P_Provincia IN CentroVotacion.ID_Provincia%TYPE,
	P_Localidad IN CentroVotacion.ID_Localidad%TYPE,
	P_Demarcacion IN CentroVotacion.ID_Demarcacion%TYPE,
	RSP OUT VARCHAR2) 
IS
    paramEntrada VARCHAR2(500);
	numERROR INTEGER;
	tipoERROR VARCHAR2(100);
    VACIO_ERROR EXCEPTION;

BEGIN
    paramEntrada:= P_IDCentro||'- Nombre:'||P_nombre||' - Direccion: '||P_direccionCentro||' - Localidad'||P_Provincia||P_Localidad||' - Demarcacin:'||P_Demarcacion;
	
    /* Comprobamos que ningn valor sea nulo */
    IF (P_IDCentro IS NULL) OR (P_nombre IS NULL) OR (P_direccionCentro IS NULL) OR (P_Provincia IS NULL) OR (P_Localidad IS NULL) OR (P_Demarcacion IS NULL)
    THEN
        RAISE VACIO_ERROR;
    END IF;
	  
	/*Realizamos la insercin del valor*/
    INSERT INTO CentroVotacion VALUES (P_IDCentro, P_nombre, P_direccionCentro, P_Provincia, P_Localidad, P_Demarcacion);
    RSP:= 'OK';
    COMMIT;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
EXCEPTION  
    WHEN DUP_VAL_ON_INDEX THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - el Centro de Votacin EST DUPLICADA / YA EXISTE';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN VACIO_ERROR THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL VALOR NO PUEDE SER NULO';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN OTHERS THEN
    numERROR:= SQLCODE;
    tipoERROR:= SUBSTR(SQLERRM, 1, 90);
    RSP := TO_CHAR(numERROR)||' - '||tipoERROR;
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
END;
/


/************************************************************
* Procedimiento para dar de BAJA un Centro de Votacin EN la BD
* Entrada: ID_Centro
* Salida: OK si se realiza con xito
*         ERROR+TIPO DE ERROR en caso contrario
* Pre-condicin: el Centro de Votacin DEBE existir 
* Post-condicin: el Centro de Votacin se ELIMINA
************************************************************/

CREATE OR REPLACE PROCEDURE bajaCentroVotacion(
	P_IDCentro IN CentroVotacion.ID_Centro%TYPE,
	RSP OUT VARCHAR2) 
IS
    paramEntrada VARCHAR2(500);
	numERROR INTEGER;
	tipoERROR VARCHAR2(100);
    VACIO_ERROR EXCEPTION;
    ID_NULL CentroVotacion.ID_Centro%TYPE;

BEGIN
    paramEntrada:= P_IDCentro;
    
    /* Comprobamos que el valor no sea nulo */
    IF (P_IDCentro IS NULL) 
    THEN
        RAISE VACIO_ERROR;
    END IF;
    /* Comprobamos que exista ese centro  */
    BEGIN
        SELECT ID_Centro
        INTO ID_NULL
        FROM CentroVotacion
        WHERE ID_Centro = P_IDCentro
        AND ROWNUM = 1;
        EXCEPTION
            WHEN NO_DATA_FOUND THEN
            RAISE NO_DATA_FOUND;
        END;
		
    /* Borramos si ha pasado las anteriores comprobaciones */
    DELETE FROM CentroVotacion WHERE ID_Centro = P_IDCentro;
    RSP:= 'OK';
    COMMIT;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
EXCEPTION
    WHEN VACIO_ERROR THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL VALOR NO PUEDE SER NULO';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN NO_DATA_FOUND THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - el Centro de Votacin NO EXISTE';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN OTHERS THEN
    numERROR:= SQLCODE;
    tipoERROR:= SUBSTR(SQLERRM, 1, 90);
    RSP := TO_CHAR(numERROR)||' - '||tipoERROR;
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
END;
/

/************************************************************
* Procedimiento para MODIFICAR un Centro de Votacin en la BD
* Entrada: ID_Centro, nombreCentro, direccionCentro, 
*          ID_Provincia, ID_Localidad, ID_Demarcacion
* Salida: OK si se realiza con xito
*         ERROR+TIPO DE ERROR en caso contrario
* Pre-condicin: el Centro de Votacin DEBE existir 
* Post-condicin: el Centro de Votacin es MODIFICADO
************************************************************/

CREATE OR REPLACE PROCEDURE modCentroVotacion(
	P_IDCentro IN CentroVotacion.ID_Centro%TYPE,
	P_nombre IN CentroVotacion.nombreCentro%TYPE,
	P_direccionCentro IN CentroVotacion.direccionCentro%TYPE,
	P_Provincia IN CentroVotacion.ID_Provincia%TYPE,
	P_Localidad IN CentroVotacion.ID_Localidad%TYPE,
	P_Demarcacion IN CentroVotacion.ID_Demarcacion%TYPE,
	RSP OUT VARCHAR2) 
IS
    paramEntrada VARCHAR2(500);
	numERROR INTEGER;
	tipoERROR VARCHAR2(100);
    VACIO_ERROR EXCEPTION;
	ID_duplicado CentroVotacion.ID_Centro%TYPE;

BEGIN
    paramEntrada:= P_IDCentro||'- Nombre:'||P_nombre||' - Direccion: '||P_direccionCentro||' - Localidad'||P_Provincia||P_Localidad||' - Demarcacin:'||P_Demarcacion;
	
    /* Comprobamos que el ID no sea nulo o bien que los otros atributos no sean nulos a la vez */
    IF (P_IDCentro IS NULL) OR ((P_nombre IS NULL) AND (P_direccionCentro IS NULL) AND (P_Provincia IS NULL) AND (P_Localidad IS NULL) AND (P_Demarcacion IS NULL))
    THEN
        RAISE VACIO_ERROR;
    END IF;
    
    /* Comprobamos que exista el Centro de Votacin a modificar  */
    BEGIN
        SELECT ID_Centro
        INTO ID_duplicado
        FROM CentroVotacion
        WHERE ID_Centro = P_IDCentro
        AND ROWNUM = 1;
        EXCEPTION
            WHEN NO_DATA_FOUND THEN
            RAISE NO_DATA_FOUND;
        END;
    
    /* Si ha pasado las comprobaciones anteriores, actualizamos */
    UPDATE CentroVotacion SET nombreCentro= P_nombre, direccionCentro= P_direccionCentro, ID_Provincia= P_Provincia WHERE ID_Centro= P_IDCentro;
    RSP:= 'OK';
    COMMIT;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
EXCEPTION
    WHEN VACIO_ERROR THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL VALOR NO PUEDE SER NULO';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN NO_DATA_FOUND THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - el Centro de Votacin NO SE ENCUENTRA - NO EXISTE';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN OTHERS THEN
    numERROR:= SQLCODE;
    tipoERROR:= SUBSTR(SQLERRM, 1, 90);
    RSP := TO_CHAR(numERROR)||' - '||tipoERROR;
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
END;
/



/************************************************************
* Procedimiento para AADIR una Mesa Electoral a la BD
* Entrada: ID_Mesa, nombreMesa, ID_Centro
* Salida: OK si se realiza con xito
*         ERROR+TIPO DE ERROR en caso contrario
* Pre-condicin: la Mesa Electoral NO debe existir 
* Post-condicin: la Mesa Electoral es dada de ALTA
************************************************************/

CREATE OR REPLACE PROCEDURE altaMesaElectoral(
	P_IDMesa IN MesaElectoral.ID_Mesa%TYPE,
	P_nombre IN MesaElectoral.nombreMesa%TYPE,
	P_Centro IN MesaElectoral.ID_Centro%TYPE,
	RSP OUT VARCHAR2) 
IS
    paramEntrada VARCHAR2(500);
	numERROR INTEGER;
	tipoERROR VARCHAR2(100);
    VACIO_ERROR EXCEPTION;

BEGIN
    paramEntrada:= P_IDMesa||'- Nombre:'||P_nombre||' - Centro: '||P_Centro;
	
    /* Comprobamos que ningn valor sea nulo */
    IF (P_IDMesa IS NULL) OR (P_nombre IS NULL) OR (P_Centro IS NULL) 
    THEN
        RAISE VACIO_ERROR;
    END IF;
	  
	/*Realizamos la insercin del valor*/
    INSERT INTO MesaElectoral VALUES (P_IDMesa, P_nombre, P_Centro);
    RSP:= 'OK';
    COMMIT;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
EXCEPTION  
    WHEN DUP_VAL_ON_INDEX THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - la Mesa Electoral EST DUPLICADA / YA EXISTE';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN VACIO_ERROR THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL VALOR NO PUEDE SER NULO';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN OTHERS THEN
    numERROR:= SQLCODE;
    tipoERROR:= SUBSTR(SQLERRM, 1, 90);
    RSP := TO_CHAR(numERROR)||' - '||tipoERROR;
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
END;
/


/************************************************************
* Procedimiento para dar de BAJA una Mesa Electoral EN la BD
* Entrada: ID_Mesa
* Salida: OK si se realiza con xito
*         ERROR+TIPO DE ERROR en caso contrario
* Pre-condicin: la Mesa Electoral DEBE existir 
* Post-condicin: la Mesa Electoral se ELIMINA
************************************************************/

CREATE OR REPLACE PROCEDURE bajaMesaElectoral(
	P_IDMesa IN MesaElectoral.ID_Mesa%TYPE,
	RSP OUT VARCHAR2) 
IS
    paramEntrada VARCHAR2(500);
	numERROR INTEGER;
	tipoERROR VARCHAR2(100);
    VACIO_ERROR EXCEPTION;
    ID_NULL MesaElectoral.ID_Mesa%TYPE;

BEGIN
    paramEntrada:= P_IDMesa;
    
    /* Comprobamos que el valor no sea nulo */
    IF (P_IDMesa IS NULL) 
    THEN
        RAISE VACIO_ERROR;
    END IF;
    /* Comprobamos que exista ese centro  */
    BEGIN
        SELECT ID_Mesa
        INTO ID_NULL
        FROM MesaElectoral
        WHERE ID_Mesa = P_IDMesa
        AND ROWNUM = 1;
        EXCEPTION
            WHEN NO_DATA_FOUND THEN
            RAISE NO_DATA_FOUND;
        END;
		
    /* Borramos si ha pasado las anteriores comprobaciones */
    DELETE FROM MesaElectoral WHERE ID_Mesa = P_IDMesa;
    RSP:= 'OK';
    COMMIT;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
EXCEPTION
    WHEN VACIO_ERROR THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL VALOR NO PUEDE SER NULO';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN NO_DATA_FOUND THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - la Mesa Electoral NO EXISTE';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN OTHERS THEN
    numERROR:= SQLCODE;
    tipoERROR:= SUBSTR(SQLERRM, 1, 90);
    RSP := TO_CHAR(numERROR)||' - '||tipoERROR;
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
END;
/

/************************************************************
* Procedimiento para MODIFICAR una Mesa Electoral en la BD
* Entrada: ID_Mesa, nombreMesa, ID_Centro
* Salida: OK si se realiza con xito
*         ERROR+TIPO DE ERROR en caso contrario
* Pre-condicin: la Mesa Electoral DEBE existir 
* Post-condicin: la Mesa Electoral es MODIFICADO
************************************************************/

CREATE OR REPLACE PROCEDURE modMesaElectoral(
	P_IDMesa IN MesaElectoral.ID_Mesa%TYPE,
	P_nombre IN MesaElectoral.nombreMesa%TYPE,
	P_Centro IN MesaElectoral.ID_Centro%TYPE,
	RSP OUT VARCHAR2) 
IS
    paramEntrada VARCHAR2(500);
	numERROR INTEGER;
	tipoERROR VARCHAR2(100);
    VACIO_ERROR EXCEPTION;
	ID_duplicado MesaElectoral.ID_Mesa%TYPE;

BEGIN
    paramEntrada:= P_IDMesa||'- Nombre:'||P_nombre||' - Centro: '||P_Centro;
	
    /* Comprobamos que el ID no sea nulo o bien que los otros atributos no sean nulos a la vez */
    IF (P_IDMesa IS NULL) OR ((P_nombre IS NULL) AND (P_Centro IS NULL))
    THEN
        RAISE VACIO_ERROR;
    END IF;
    
    /* Comprobamos que exista la Mesa Electoral a modificar  */
    BEGIN
        SELECT ID_Mesa
        INTO ID_duplicado
        FROM MesaElectoral
        WHERE ID_Mesa = P_IDMesa
        AND ROWNUM = 1;
        EXCEPTION
            WHEN NO_DATA_FOUND THEN
            RAISE NO_DATA_FOUND;
        END;
    
    /* Si ha pasado las comprobaciones anteriores, actualizamos */
    UPDATE MesaElectoral SET nombreMesa= P_nombre, ID_Centro= P_Centro WHERE ID_Mesa= P_IDMesa;
    RSP:= 'OK';
    COMMIT;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
EXCEPTION
    WHEN VACIO_ERROR THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL VALOR NO PUEDE SER NULO';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN NO_DATA_FOUND THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - la Mesa Electoral NO SE ENCUENTRA - NO EXISTE';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN OTHERS THEN
    numERROR:= SQLCODE;
    tipoERROR:= SUBSTR(SQLERRM, 1, 90);
    RSP := TO_CHAR(numERROR)||' - '||tipoERROR;
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
END;
/



/************************************************************
* Procedimiento para AADIR una Comuniciacin en el BOE a la BD
* Entrada: ID_ComBOE, ambito, fechaElecciones, ReglasPartidos
* 		   metodoCalculo, demarcacion, escanosNum, ID_Gobierno 
* Salida: OK si se realiza con xito
*         ERROR+TIPO DE ERROR en caso contrario
* Pre-condicin: la Comuniciacin en el BOE NO debe existir 
*				 El gobierno debe existir
* Post-condicin: la Comuniciacin en el BOE es dada de ALTA
************************************************************/

CREATE OR REPLACE PROCEDURE altaComunicacionBOE(
	PIDComBOE IN ComunicacionBOE.ID_ComBOE%TYPE,
	P_ambito IN ComunicacionBOE.ambito%TYPE,
	P_fecha IN ComunicacionBOE.fechaElecciones%TYPE,
	P_Reglas IN ComunicacionBOE.ReglasPartidos%TYPE,
	P_metodo IN ComunicacionBOE.metodoCalculo%TYPE,
	P_demarcacion IN ComunicacionBOE.demarcacion%TYPE,
	P_escanosNum IN ComunicacionBOE.escanosNum%TYPE,
	P_IDGobierno IN ComunicacionBOE.ID_Gobierno%TYPE,
	RSP OUT VARCHAR2) 
IS
    paramEntrada VARCHAR2(4000);
	numERROR INTEGER;
	tipoERROR VARCHAR2(100);
    VACIO_ERROR EXCEPTION;

BEGIN
    paramEntrada:= PIDComBOE||'- Ambito:'||P_ambito||' - Fecha:'||P_fecha||' - Reglas:'||P_Reglas||' - Metodo:'||P_metodo||' - Demarcacion:'||P_demarcacion||' - Escaos:'||P_escanosNum||' - Gobierno:'||P_IDGobierno;
	
    /* Comprobamos que ningn valor sea nulo */
    IF (PIDComBOE IS NULL) OR (P_ambito IS NULL) OR (P_fecha IS NULL) OR (P_Reglas IS NULL) OR (P_metodo IS NULL) OR (P_demarcacion IS NULL) OR (P_escanosNum IS NULL) OR (P_IDGobierno IS NULL) 
    THEN
        RAISE VACIO_ERROR;
    END IF;
	  
	/*Realizamos la insercin del valor*/
    INSERT INTO ComunicacionBOE VALUES (PIDComBOE, P_ambito, P_fecha, P_Reglas, P_metodo, P_demarcacion, P_escanosNum, P_IDGobierno);
    RSP:= 'OK';
    COMMIT;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
EXCEPTION  
    WHEN DUP_VAL_ON_INDEX THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - la Comuniciacin en el BOE EST DUPLICADA / YA EXISTE';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN VACIO_ERROR THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL VALOR NO PUEDE SER NULO';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN OTHERS THEN
    numERROR:= SQLCODE;
    tipoERROR:= SUBSTR(SQLERRM, 1, 90);
    RSP := TO_CHAR(numERROR)||' - '||tipoERROR;
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
END;
/


/************************************************************
* Procedimiento para dar de BAJA una Comuniciacin en el BOE EN la BD
* Entrada: ID_ComBOE
* Salida: OK si se realiza con xito
*         ERROR+TIPO DE ERROR en caso contrario
* Pre-condicin: la Comuniciacin en el BOE DEBE existir 
* Post-condicin: la Comuniciacin en el BOE se ELIMINA
************************************************************/

CREATE OR REPLACE PROCEDURE bajaComunicacionBOE(
	PIDComBOE IN ComunicacionBOE.ID_ComBOE%TYPE,
	RSP OUT VARCHAR2) 
IS
    paramEntrada VARCHAR2(500);
	numERROR INTEGER;
	tipoERROR VARCHAR2(100);
    VACIO_ERROR EXCEPTION;
    ID_NULL ComunicacionBOE.ID_ComBOE%TYPE;

BEGIN
    paramEntrada:= PIDComBOE;
    
    /* Comprobamos que el valor no sea nulo */
    IF (PIDComBOE IS NULL) 
    THEN
        RAISE VACIO_ERROR;
    END IF;
    /* Comprobamos que exista ese centro  */
    BEGIN
        SELECT ID_ComBOE
        INTO ID_NULL
        FROM ComunicacionBOE
        WHERE ID_ComBOE = PIDComBOE
        AND ROWNUM = 1;
        EXCEPTION
            WHEN NO_DATA_FOUND THEN
            RAISE NO_DATA_FOUND;
        END;
		
    /* Borramos si ha pasado las anteriores comprobaciones */
    DELETE FROM ComunicacionBOE WHERE ID_ComBOE = PIDComBOE;
    RSP:= 'OK';
    COMMIT;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
EXCEPTION
    WHEN VACIO_ERROR THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL VALOR NO PUEDE SER NULO';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN NO_DATA_FOUND THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - la Comuniciacin en el BOE NO EXISTE';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN OTHERS THEN
    numERROR:= SQLCODE;
    tipoERROR:= SUBSTR(SQLERRM, 1, 90);
    RSP := TO_CHAR(numERROR)||' - '||tipoERROR;
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
END;
/

/************************************************************
* Procedimiento para MODIFICAR una Comuniciacin en el BOE en la BD
* Entrada: ID_ComBOE, ambito, fechaElecciones, ReglasPartidos
* 		   metodoCalculo, demarcacion, escanosNum, ID_Gobierno 
* Salida: OK si se realiza con xito
*         ERROR+TIPO DE ERROR en caso contrario
* Pre-condicin: la Comuniciacin en el BOE DEBE existir 
* Post-condicin: la Comuniciacin en el BOE es MODIFICADO
************************************************************/

CREATE OR REPLACE PROCEDURE modComunicacionBOE(
	PIDComBOE IN ComunicacionBOE.ID_ComBOE%TYPE,
	P_ambito IN ComunicacionBOE.ambito%TYPE,
	P_fecha IN ComunicacionBOE.fechaElecciones%TYPE,
	P_Reglas IN ComunicacionBOE.ReglasPartidos%TYPE,
	P_metodo IN ComunicacionBOE.metodoCalculo%TYPE,
	P_demarcacion IN ComunicacionBOE.demarcacion%TYPE,
	P_escanosNum IN ComunicacionBOE.escanosNum%TYPE,
	P_IDGobierno IN ComunicacionBOE.ID_Gobierno%TYPE,
	RSP OUT VARCHAR2) 
IS
    paramEntrada VARCHAR2(500);
	numERROR INTEGER;
	tipoERROR VARCHAR2(100);
    VACIO_ERROR EXCEPTION;
	ID_duplicado ComunicacionBOE.ID_ComBOE%TYPE;

BEGIN
    paramEntrada:= PIDComBOE||'- Ambito:'||P_ambito||' - Fecha:'||P_fecha||' - Reglas:'||P_Reglas||' - Metodo:'||P_metodo||' - Demarcacion:'||P_demarcacion||' - Escaos:'||P_escanosNum||' - Gobierno:'||P_IDGobierno;
	
    /* Comprobamos que el ID no sea nulo o bien que los otros atributos no sean nulos a la vez */
    IF (PIDComBOE IS NULL) OR ((P_ambito IS NULL) AND (P_fecha IS NULL) AND (P_Reglas IS NULL) AND (P_metodo IS NULL) AND (P_demarcacion IS NULL) AND (P_escanosNum IS NULL) AND (P_IDGobierno IS NULL) )
    THEN
        RAISE VACIO_ERROR;
    END IF;
    
    /* Comprobamos que exista la Comuniciacin en el BOE a modificar  */
    BEGIN
        SELECT ID_ComBOE
        INTO ID_duplicado
        FROM ComunicacionBOE
        WHERE ID_ComBOE = PIDComBOE
        AND ROWNUM = 1;
        EXCEPTION
            WHEN NO_DATA_FOUND THEN
            RAISE NO_DATA_FOUND;
        END;
    
    /* Si ha pasado las comprobaciones anteriores, actualizamos */
    UPDATE ComunicacionBOE SET ambito= P_ambito, fechaElecciones= P_fecha, ReglasPartidos=P_Reglas, metodoCalculo=P_metodo, demarcacion=P_demarcacion, escanosNum=P_escanosNum, ID_Gobierno=P_IDGobierno  WHERE ID_ComBOE= PIDComBOE;
    RSP:= 'OK';
    COMMIT;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
EXCEPTION
    WHEN VACIO_ERROR THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL VALOR NO PUEDE SER NULO';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN NO_DATA_FOUND THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - la Comuniciacin en el BOE NO SE ENCUENTRA - NO EXISTE';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN OTHERS THEN
    numERROR:= SQLCODE;
    tipoERROR:= SUBSTR(SQLERRM, 1, 90);
    RSP := TO_CHAR(numERROR)||' - '||tipoERROR;
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
END;
/



/************************************************************
* Procedimiento para AADIR que partidos participan en un 
* proceso electoral a la BD.
* Entrada: ID_Proceso, ID_RPP, ID_Partido 
* Salida: OK si se realiza con xito
*         ERROR+TIPO DE ERROR en caso contrario
* Pre-condicin: El proceso electoral, las reglas de partidos 
*				 y los partidos polticos deben existir.
* Post-condicin: los partidos que participan en un proceso 
* 				  electoral son dada de ALTA
************************************************************/

CREATE OR REPLACE PROCEDURE altaParticipaEn(
	P_Proceso IN ParticipaEn.ID_Proceso%TYPE,
	P_RPP IN ParticipaEn.ID_RPP%TYPE,
	P_Partido IN ParticipaEn.ID_Partido%TYPE,
	RSP OUT VARCHAR2) 
IS
    paramEntrada VARCHAR2(4000);
	numERROR INTEGER;
	tipoERROR VARCHAR2(100);
    VACIO_ERROR EXCEPTION;

BEGIN
    paramEntrada:= 'Proceso:'||P_Proceso||' - Regla:'||P_RPP||' - Partido:'||P_Partido;
	
    /* Comprobamos que ningn valor sea nulo */
    IF (P_Proceso IS NULL) OR (P_RPP IS NULL) OR (P_Partido IS NULL)  
    THEN
        RAISE VACIO_ERROR;
    END IF;
	  
	/*Realizamos la insercin del valor*/
    INSERT INTO ParticipaEn VALUES (P_Proceso, P_RPP, P_Partido);
    RSP:= 'OK';
    COMMIT;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
EXCEPTION  
    WHEN DUP_VAL_ON_INDEX THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' -  DUPLICADA / YA EXISTE';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN VACIO_ERROR THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL VALOR NO PUEDE SER NULO';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN OTHERS THEN
    numERROR:= SQLCODE;
    tipoERROR:= SUBSTR(SQLERRM, 1, 90);
    RSP := TO_CHAR(numERROR)||' - '||tipoERROR;
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
END;
/


/************************************************************
* Procedimiento para dar de BAJA que partidos participan en 
* un proceso electoral EN la BD
* Entrada: ID_Proceso, ID_RPP, ID_Partido 
* Salida: OK si se realiza con xito
*         ERROR+TIPO DE ERROR en caso contrario
* Pre-condicin: los partidos que participan en un proceso 
* 				 electoral DEBEN existir 
* Post-condicin: el partidos que participan en un proceso 
* 				  electoral se ELIMINA
************************************************************/

CREATE OR REPLACE PROCEDURE bajaParticipaEn(
	P_Proceso IN ParticipaEn.ID_Proceso%TYPE,
	P_RPP IN ParticipaEn.ID_RPP%TYPE,
	P_Partido IN ParticipaEn.ID_Partido%TYPE,
	RSP OUT VARCHAR2) 
IS
    paramEntrada VARCHAR2(500);
	numERROR INTEGER;
	tipoERROR VARCHAR2(100);
    VACIO_ERROR EXCEPTION;
    proceso_NULL ParticipaEn.ID_Proceso%TYPE;
	RPP_NULL ParticipaEn.ID_RPP%TYPE;
	partido_NULL ParticipaEn.ID_Partido%TYPE;

BEGIN
    paramEntrada:= 'Proceso:'||P_Proceso||' - Regla:'||P_RPP||' - Partido:'||P_Partido;
    
    /* Comprobamos que el valor no sea nulo */
    IF (P_Proceso IS NULL) OR (P_RPP IS NULL) OR (P_Partido IS NULL)  
    THEN
        RAISE VACIO_ERROR;
    END IF;
	
    /* Comprobamos que exista */
    BEGIN
        SELECT ID_Proceso, ID_RPP, ID_Partido
        INTO proceso_NULL, RPP_NULL, partido_NULL
        FROM ParticipaEn
        WHERE ID_Proceso = P_Proceso
		AND ID_RPP = P_RPP
		AND ID_Partido = P_Partido
        AND ROWNUM = 1;
        EXCEPTION
            WHEN NO_DATA_FOUND THEN
            RAISE NO_DATA_FOUND;
        END;
		
    /* Borramos si ha pasado las anteriores comprobaciones */
    DELETE FROM ParticipaEn WHERE ID_Proceso = P_Proceso AND ID_RPP = P_RPP AND ID_Partido = P_Partido;
    RSP:= 'OK';
    COMMIT;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
EXCEPTION
    WHEN VACIO_ERROR THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL VALOR NO PUEDE SER NULO';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN NO_DATA_FOUND THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' -  NO EXISTE';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN OTHERS THEN
    numERROR:= SQLCODE;
    tipoERROR:= SUBSTR(SQLERRM, 1, 90);
    RSP := TO_CHAR(numERROR)||' - '||tipoERROR;
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
END;
/


/************************************************************
* Procedimiento para AADIR las personas al censo electoral.
* Entrada:  numDoc, letraDoc, ID_RP, ID_Proceso, fechaElecciones
*			inscrito, mesaElectoral, centroVotacion, ID_Voto
* Salida: OK si se realiza con xito
*         ERROR+TIPO DE ERROR en caso contrario
* Pre-condicin: El proceso electoral, las reglas de participacin 
*				 y las personas deben existir.
* Post-condicin: el censo electoral es dado de ALTA
************************************************************/

CREATE OR REPLACE PROCEDURE altaCensoElectoral(
	P_numDoc IN CensoElectoral.numDoc%TYPE,
	P_letraDoc IN CensoElectoral.letraDoc%TYPE,
	P_IDRP IN CensoElectoral.ID_RP%TYPE,
	P_IDProceso IN CensoElectoral.ID_Proceso%TYPE,
	P_fechaElecciones IN CensoElectoral.fechaElecciones%TYPE,
	P_inscrito IN CensoElectoral.inscrito%TYPE,
	P_mesaElectoral IN CensoElectoral.mesaElectoral%TYPE,
	P_centroVotacion IN CensoElectoral.centroVotacion%TYPE,
	P_IDVoto IN CensoElectoral.ID_Voto%TYPE,
	RSP OUT VARCHAR2) 
IS
    paramEntrada VARCHAR2(4000);
	numERROR INTEGER;
	tipoERROR VARCHAR2(100);
    VACIO_ERROR EXCEPTION;

BEGIN
    paramEntrada:= 'DNI:'||P_numDoc||P_letraDoc||' - Regla:'||P_IDRP||' - Proceso:'||P_IDProceso||' - Fecha'||P_fechaElecciones||
                ' - Inscrito'||P_inscrito||' - Mesa'||P_mesaElectoral||' - Centro'||P_centroVotacion||' - Voto'||P_IDVoto;
	
    /* Comprobamos que ningn valor sea nulo, voto no se evala ya que es totalmente opcional */
    IF (P_numDoc IS NULL) OR (P_letraDoc IS NULL) OR (P_IDRP IS NULL) OR (P_IDProceso IS NULL) OR (P_fechaElecciones IS NULL) 
        OR (P_inscrito IS NULL) OR (P_mesaElectoral IS NULL) OR (P_centroVotacion IS NULL)  
    THEN
        RAISE VACIO_ERROR;
    END IF;
	  
	/*Realizamos la insercin del valor*/
    INSERT INTO CensoElectoral VALUES (P_numDoc, P_letraDoc, P_IDRP, P_IDProceso, P_fechaElecciones, P_inscrito, P_mesaElectoral, P_centroVotacion, P_IDVoto);
    RSP:= 'OK';
    COMMIT;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
EXCEPTION  
    WHEN DUP_VAL_ON_INDEX THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' -  DUPLICADA / YA EXISTE';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN VACIO_ERROR THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL VALOR NO PUEDE SER NULO';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN OTHERS THEN
    numERROR:= SQLCODE;
    tipoERROR:= SUBSTR(SQLERRM, 1, 90);
    RSP := TO_CHAR(numERROR)||' - '||tipoERROR;
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
END;
/


/************************************************************
* Procedimiento para dar de BAJA las personas en el censo electoral
* Entrada: ID_Proceso, ID_RPP, ID_Partido 
* Salida: OK si se realiza con xito
*         ERROR+TIPO DE ERROR en caso contrario
* Pre-condicin: las personas que participan en un proceso 
* 				 electoral DEBEN existir 
* Post-condicin: las personas se ELIMINAN del censo
************************************************************/

CREATE OR REPLACE PROCEDURE bajaCensoElectoral(
	P_numDoc IN CensoElectoral.numDoc%TYPE,
	P_letraDoc IN CensoElectoral.letraDoc%TYPE,
	P_IDRP IN CensoElectoral.ID_RP%TYPE,
	P_IDProceso IN CensoElectoral.ID_Proceso%TYPE,
	RSP OUT VARCHAR2) 
IS
    paramEntrada VARCHAR2(500);
	numERROR INTEGER;
	tipoERROR VARCHAR2(100);
    VACIO_ERROR EXCEPTION;
	numDoc_NULL CensoElectoral.numDoc%TYPE;
	letraDoc_NULL CensoElectoral.letraDoc%TYPE;
	IDRP_NULL CensoElectoral.ID_RP%TYPE;
	IDProceso_NULL CensoElectoral.ID_Proceso%TYPE;

BEGIN
    paramEntrada:= 'DNI:'||P_numDoc||P_letraDoc||' - Regla:'||P_IDRP||' - Proceso:'||P_IDProceso;
    
    /* Comprobamos que el valor no sea nulo */
    IF (P_numDoc IS NULL) OR (P_letraDoc IS NULL) OR (P_IDRP IS NULL) OR (P_IDProceso IS NULL)
    THEN
        RAISE VACIO_ERROR;
    END IF;
	
    /* Comprobamos que exista */
    BEGIN
        SELECT numDoc, letraDoc, ID_RP, ID_Proceso
        INTO numDoc_NULL, letraDoc_NULL, IDRP_NULL, IDProceso_NULL
        FROM CensoElectoral
        WHERE numDoc = numDoc_NULL
		AND letraDoc = letraDoc_NULL
		AND ID_RP = IDRP_NULL
		AND ID_Proceso = IDProceso_NULL
        AND ROWNUM = 1;
        EXCEPTION
            WHEN NO_DATA_FOUND THEN
            RAISE NO_DATA_FOUND;
    END;
		
    /* Borramos si ha pasado las anteriores comprobaciones */
    DELETE FROM CensoElectoral WHERE numDoc = P_numDoc AND letraDoc = P_letraDoc AND ID_RP = P_IDRP AND ID_Proceso = P_IDProceso;
    RSP:= 'OK';
    COMMIT;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
EXCEPTION
    WHEN VACIO_ERROR THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL VALOR NO PUEDE SER NULO';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN NO_DATA_FOUND THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' -  NO EXISTE';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN OTHERS THEN
    numERROR:= SQLCODE;
    tipoERROR:= SUBSTR(SQLERRM, 1, 90);
    RSP := TO_CHAR(numERROR)||' - '||tipoERROR;
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
END;
/

/************************************************************
* Procedimiento para MODIFICAR las personas inscritas en el
* censo electoral.
* Entrada:  numDoc, letraDoc, ID_RP, ID_Proceso, fechaElecciones
*			inscrito, mesaElectoral, centroVotacion, ID_Voto
* Salida: OK si se realiza con xito
*         ERROR+TIPO DE ERROR en caso contrario
* Pre-condicin: la persona, el proceso y las reglas 
* 				 DEBEN existir 
* Post-condicin: los datos de la persona son MODIFICADOS
************************************************************/

CREATE OR REPLACE PROCEDURE modCensoElectoral(
	P_numDoc IN CensoElectoral.numDoc%TYPE,
	P_letraDoc IN CensoElectoral.letraDoc%TYPE,
	P_IDRP IN CensoElectoral.ID_RP%TYPE,
	P_IDProceso IN CensoElectoral.ID_Proceso%TYPE,
	P_fechaElecciones IN CensoElectoral.fechaElecciones%TYPE,
	P_inscrito IN CensoElectoral.inscrito%TYPE,
	P_mesaElectoral IN CensoElectoral.mesaElectoral%TYPE,
	P_centroVotacion IN CensoElectoral.centroVotacion%TYPE,
	P_IDVoto IN CensoElectoral.ID_Voto%TYPE,
	RSP OUT VARCHAR2)
IS
    paramEntrada VARCHAR2(500);
	numERROR INTEGER;
	tipoERROR VARCHAR2(100);
    VACIO_ERROR EXCEPTION;
	numDoc_NULL CensoElectoral.numDoc%TYPE;
	letraDoc_NULL CensoElectoral.letraDoc%TYPE;
	IDRP_NULL CensoElectoral.ID_RP%TYPE;
	IDProceso_NULL CensoElectoral.ID_Proceso%TYPE;

BEGIN
    paramEntrada:= 'DNI:'||P_numDoc||P_letraDoc||' - Regla:'||P_IDRP||' - Proceso:'||P_IDProceso||' - Fecha'||P_fechaElecciones||' - Inscrito'||P_inscrito||
					' - Mesa'||P_mesaElectoral||' - Centro'||P_centroVotacion||' - Voto'||P_IDVoto;
	
    /* Comprobamos que ningn valor de clave primaria sea nulo, o todos los otros no sean nulos a la vez */
    IF (P_numDoc IS NULL) OR (P_letraDoc IS NULL) OR (P_IDRP IS NULL) OR (P_IDProceso IS NULL) OR 
		((P_fechaElecciones IS NULL) AND (P_inscrito IS NULL) AND (P_mesaElectoral IS NULL) AND (P_centroVotacion IS NULL) AND (P_IDVoto IS NULL))
    THEN
        RAISE VACIO_ERROR;
    END IF;
	
    /* Comprobamos que exista */
    BEGIN
        SELECT numDoc, letraDoc, ID_RP, ID_Proceso
        INTO numDoc_NULL, letraDoc_NULL, IDRP_NULL, IDProceso_NULL
        FROM CensoElectoral
        WHERE numDoc = numDoc_NULL
		AND letraDoc = letraDoc_NULL
		AND ID_RP = IDRP_NULL
		AND ID_Proceso = IDProceso_NULL
        AND ROWNUM = 1;
        EXCEPTION
            WHEN NO_DATA_FOUND THEN
            RAISE NO_DATA_FOUND;
    END;
    
    /* Si ha pasado las comprobaciones anteriores, actualizamos */
    UPDATE CensoElectoral SET fechaElecciones = P_fechaElecciones, inscrito = P_inscrito, mesaElectoral = P_mesaElectoral, centroVotacion = P_centroVotacion, ID_Voto = P_IDVoto
	WHERE numDoc = P_numDoc AND letraDoc = P_letraDoc AND ID_RP = P_IDRP AND ID_Proceso = P_IDProceso;
    RSP:= 'OK';
    COMMIT;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
EXCEPTION
    WHEN VACIO_ERROR THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL VALOR NO PUEDE SER NULO';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN NO_DATA_FOUND THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - NO SE ENCUENTRA - NO EXISTE';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN OTHERS THEN
    numERROR:= SQLCODE;
    tipoERROR:= SUBSTR(SQLERRM, 1, 90);
    RSP := TO_CHAR(numERROR)||' - '||tipoERROR;
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
END;
/

/************************************************************
* Procedimiento para AADIR las Comunicaciones a los Votantes.
* Entrada:  ID_CV, numDoc, letraDoc, ID_Proceso, 
*			 mesaElectoral, centroVotacion, InstruccionesCV
* Salida: OK si se realiza con xito
*         ERROR+TIPO DE ERROR en caso contrario
* Pre-condicin: El proceso electoral, y el censo deben existir.
* Post-condicin: la comunicacin es dada de ALTA
************************************************************/

CREATE OR REPLACE PROCEDURE altaComunicacionVotantes(
	P_IDCV IN ComunicacionVotantes.ID_CV%TYPE,
	P_numDoc IN ComunicacionVotantes.numDoc%TYPE,
	P_letraDoc IN ComunicacionVotantes.letraDoc%TYPE,
	P_mesaElectoral IN ComunicacionVotantes.mesaElectoral%TYPE,
	P_centroVotacion IN ComunicacionVotantes.centroVotacion%TYPE,
	P_instrucciones IN ComunicacionVotantes.InstruccionesCV%TYPE,
	P_IDProceso IN ComunicacionVotantes.ID_Proceso%TYPE,
	RSP OUT VARCHAR2) 
IS
    paramEntrada VARCHAR2(4000);
	numERROR INTEGER;
	tipoERROR VARCHAR2(100);
    VACIO_ERROR EXCEPTION;

BEGIN
    paramEntrada:=P_IDCV||'DNI:'||P_numDoc||P_letraDoc||' - Proceso:'||P_IDProceso||' - Mesa'||P_mesaElectoral||' - Centro'||P_centroVotacion||' - InstruccionesCV'||P_instrucciones;
	
    /* Comprobamos que ningn valor sea nulo, voto no se evala ya que es totalmente opcional */
    IF (P_IDCV IS NULL) OR (P_numDoc IS NULL) OR (P_letraDoc IS NULL) OR (P_IDProceso IS NULL) OR (P_mesaElectoral IS NULL) OR (P_centroVotacion IS NULL) OR (P_instrucciones IS NULL) 
    THEN
        RAISE VACIO_ERROR;
    END IF;
	  
	/*Realizamos la insercin del valor*/
    INSERT INTO ComunicacionVotantes VALUES (P_IDCV, P_numDoc, P_letraDoc, P_mesaElectoral, P_centroVotacion, P_instrucciones, P_IDProceso);
    RSP:= 'OK';
    COMMIT;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
EXCEPTION  
    WHEN DUP_VAL_ON_INDEX THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' -  DUPLICADA / YA EXISTE';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN VACIO_ERROR THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL VALOR NO PUEDE SER NULO';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN OTHERS THEN
    numERROR:= SQLCODE;
    tipoERROR:= SUBSTR(SQLERRM, 1, 90);
    RSP := TO_CHAR(numERROR)||' - '||tipoERROR;
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
END;
/


/************************************************************
* Procedimiento para dar de BAJA las Comunicaciones a los Votantes
* Entrada: ID_CV
* Salida: OK si se realiza con xito
*         ERROR+TIPO DE ERROR en caso contrario
* Pre-condicin: la comunicacin debe existir
* Post-condicin: la comunicacin se ELIMINA 
************************************************************/

CREATE OR REPLACE PROCEDURE bajaComunicacionVotantes(
	P_IDCV IN ComunicacionVotantes.ID_CV%TYPE,
	RSP OUT VARCHAR2) 
IS
    paramEntrada VARCHAR2(500);
	numERROR INTEGER;
	tipoERROR VARCHAR2(100);
    VACIO_ERROR EXCEPTION;
	IDCV_NULL ComunicacionVotantes.ID_CV%TYPE;


BEGIN
    paramEntrada:=' - Identificador:'||P_IDCV;
    
    /* Comprobamos que el valor no sea nulo */
    IF (P_IDCV IS NULL)
    THEN
        RAISE VACIO_ERROR;
    END IF;
	
    /* Comprobamos que exista */
    BEGIN
        SELECT ID_CV
        INTO IDCV_NULL
        FROM ComunicacionVotantes
        WHERE ID_CV = IDCV_NULL
        AND ROWNUM = 1;
        EXCEPTION
            WHEN NO_DATA_FOUND THEN
            RAISE NO_DATA_FOUND;
    END;
		
    /* Borramos si ha pasado las anteriores comprobaciones */
    DELETE FROM ComunicacionVotantes WHERE ID_CV = P_IDCV;
    RSP:= 'OK';
    COMMIT;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
EXCEPTION
    WHEN VACIO_ERROR THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL VALOR NO PUEDE SER NULO';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN NO_DATA_FOUND THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' -  NO EXISTE';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN OTHERS THEN
    numERROR:= SQLCODE;
    tipoERROR:= SUBSTR(SQLERRM, 1, 90);
    RSP := TO_CHAR(numERROR)||' - '||tipoERROR;
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
END;
/

/************************************************************
* Procedimiento para MODIFICAR las Comunicaciones a los Votantes
* Entrada:  ID_CV, numDoc, letraDoc, ID_Proceso, 
*			 mesaElectoral, centroVotacion, InstruccionesCV
* Salida: OK si se realiza con xito
*         ERROR+TIPO DE ERROR en caso contrario
* Pre-condicin: la comunicacin DEBEN existir 
* Post-condicin: la comunicacin es MODIFICADA
************************************************************/

CREATE OR REPLACE PROCEDURE modComunicacionVotantes(
	P_IDCV IN ComunicacionVotantes.ID_CV%TYPE,
	P_numDoc IN ComunicacionVotantes.numDoc%TYPE,
	P_letraDoc IN ComunicacionVotantes.letraDoc%TYPE,
	P_mesaElectoral IN ComunicacionVotantes.mesaElectoral%TYPE,
	P_centroVotacion IN ComunicacionVotantes.centroVotacion%TYPE,
	P_instrucciones IN ComunicacionVotantes.InstruccionesCV%TYPE,
	P_IDProceso IN ComunicacionVotantes.ID_Proceso%TYPE,
	RSP OUT VARCHAR2) 
IS
    paramEntrada VARCHAR2(500);
	numERROR INTEGER;
	tipoERROR VARCHAR2(100);
    VACIO_ERROR EXCEPTION;
	IDCV_NULL ComunicacionVotantes.ID_CV%TYPE;

BEGIN
    paramEntrada:=P_IDCV||'DNI:'||P_numDoc||P_letraDoc||' - Proceso:'||P_IDProceso||' - Mesa'||P_mesaElectoral||' - Centro'||P_centroVotacion||' - InstruccionesCV'||P_instrucciones;
	
    /* Comprobamos que ningn valor de clave primaria sea nulo, o todos los otros no sean nulos a la vez */
    IF (P_IDCV IS NULL) OR ((P_numDoc IS NULL) AND (P_letraDoc IS NULL) AND (P_IDProceso IS NULL) AND (P_mesaElectoral IS NULL) AND (P_centroVotacion IS NULL) AND (P_instrucciones IS NULL))
    THEN
        RAISE VACIO_ERROR;
    END IF;
	
    /* Comprobamos que exista */
    BEGIN
        SELECT ID_CV
        INTO IDCV_NULL
        FROM ComunicacionVotantes
        WHERE ID_CV = IDCV_NULL
        AND ROWNUM = 1;
        EXCEPTION
            WHEN NO_DATA_FOUND THEN
            RAISE NO_DATA_FOUND;
    END;
    
    /* Si ha pasado las comprobaciones anteriores, actualizamos */
    UPDATE ComunicacionVotantes SET numDoc = P_numDoc, letraDoc = P_letraDoc, mesaElectoral = P_mesaElectoral, centroVotacion = P_centroVotacion, instruccionesCV = P_instrucciones, ID_Proceso = P_IDProceso WHERE ID_CV = P_IDCV;
    RSP:= 'OK';
    COMMIT;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
EXCEPTION
    WHEN VACIO_ERROR THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL VALOR NO PUEDE SER NULO';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN NO_DATA_FOUND THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - NO SE ENCUENTRA - NO EXISTE';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN OTHERS THEN
    numERROR:= SQLCODE;
    tipoERROR:= SUBSTR(SQLERRM, 1, 90);
    RSP := TO_CHAR(numERROR)||' - '||tipoERROR;
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
END;
/

/************************************************************
* Procedimiento para AADIR las Listas de Candidatos.
* Entrada:  ID_Lista, aprobada, ID_Partido, ID_Demarcacion, 
*			 ID_Gobierno
* Salida: OK si se realiza con xito
*         ERROR+TIPO DE ERROR en caso contrario
* Pre-condicin: El partido, las demarcaciones y el gobierno 
* 				DEBEN existir.
* Post-condicin: la lista es dada de ALTA
************************************************************/

CREATE OR REPLACE PROCEDURE altaListaCandidatos(
	P_IDLista IN ListaCandidatos.ID_Lista%TYPE,
	P_aprobada IN ListaCandidatos.aprobada%TYPE,
	P_partido IN ListaCandidatos.ID_Partido%TYPE,
	P_demarcacion IN ListaCandidatos.ID_Demarcacion%TYPE,
	P_gobierno IN ListaCandidatos.ID_Gobierno%TYPE,
	RSP OUT VARCHAR2) 
IS
    paramEntrada VARCHAR2(4000);
	numERROR INTEGER;
	tipoERROR VARCHAR2(100);
    VACIO_ERROR EXCEPTION;

BEGIN
    paramEntrada:=P_IDLista||' - aprobada: '||P_aprobada||' - Partido: '||P_partido||' - Demarcacion: '||P_demarcacion||' - Gobierno'||P_gobierno;
	
    /* Comprobamos que ningn valor sea nulo */
    IF (P_IDLista IS NULL) OR (P_aprobada IS NULL) OR (P_partido IS NULL) OR (P_demarcacion IS NULL) OR (P_gobierno IS NULL)
    THEN
        RAISE VACIO_ERROR;
    END IF;
	  
	/*Realizamos la insercin del valor*/
    INSERT INTO ListaCandidatos VALUES (P_IDLista, P_aprobada, P_partido, P_demarcacion, P_gobierno);
    RSP:= 'OK';
    COMMIT;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
EXCEPTION  
    WHEN DUP_VAL_ON_INDEX THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' -  LISTA DUPLICADA / YA EXISTE';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN VACIO_ERROR THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL VALOR NO PUEDE SER NULO';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN OTHERS THEN
    numERROR:= SQLCODE;
    tipoERROR:= SUBSTR(SQLERRM, 1, 90);
    RSP := TO_CHAR(numERROR)||' - '||tipoERROR;
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
END;
/


/************************************************************
* Procedimiento para dar de BAJA la lista de candidatos
* Entrada: ID_Lista
* Salida: OK si se realiza con xito
*         ERROR+TIPO DE ERROR en caso contrario
* Pre-condicin: la lista debe existir
* Post-condicin: la lista se ELIMINA 
************************************************************/

CREATE OR REPLACE PROCEDURE bajaListaCandidatos(
	P_IDLista IN ListaCandidatos.ID_Lista%TYPE,
	RSP OUT VARCHAR2) 
IS
    paramEntrada VARCHAR2(500);
	numERROR INTEGER;
	tipoERROR VARCHAR2(100);
    VACIO_ERROR EXCEPTION;
	IDLista_NULL ListaCandidatos.ID_Lista%TYPE;


BEGIN
    paramEntrada:='Identificador lista:'||P_IDLista;
    
    /* Comprobamos que el valor no sea nulo */
    IF (P_IDLista IS NULL)
    THEN
        RAISE VACIO_ERROR;
    END IF;
	
    /* Comprobamos que exista */
    BEGIN
        SELECT ID_Lista
        INTO IDLista_NULL
        FROM ListaCandidatos
        WHERE ID_Lista = P_IDLista
        AND ROWNUM = 1;
        EXCEPTION
            WHEN NO_DATA_FOUND THEN
            RAISE NO_DATA_FOUND;
    END;
		
    /* Borramos si ha pasado las anteriores comprobaciones */
    DELETE FROM ListaCandidatos WHERE ID_Lista = P_IDLista;
    RSP:= 'OK';
    COMMIT;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
EXCEPTION
    WHEN VACIO_ERROR THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL VALOR NO PUEDE SER NULO';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN NO_DATA_FOUND THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - LA LISTA NO EXISTE';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN OTHERS THEN
    numERROR:= SQLCODE;
    tipoERROR:= SUBSTR(SQLERRM, 1, 90);
    RSP := TO_CHAR(numERROR)||' - '||tipoERROR;
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
END;
/

/************************************************************
* Procedimiento para MODIFICAR la lista de candidatos
* Entrada:  ID_Lista, aprobada, ID_Partido, ID_Demarcacion, 
*			 ID_Gobierno
* Salida: OK si se realiza con xito
*         ERROR+TIPO DE ERROR en caso contrario
* Pre-condicin: la lista DEBE existir 
* Post-condicin: la lista es MODIFICADA
************************************************************/

CREATE OR REPLACE PROCEDURE modListaCandidatos(
	P_IDLista IN ListaCandidatos.ID_Lista%TYPE,
	P_aprobada IN ListaCandidatos.aprobada%TYPE,
	P_partido IN ListaCandidatos.ID_Partido%TYPE,
	P_demarcacion IN ListaCandidatos.ID_Demarcacion%TYPE,
	P_gobierno IN ListaCandidatos.ID_Gobierno%TYPE,
	RSP OUT VARCHAR2) 
IS
    paramEntrada VARCHAR2(500);
	numERROR INTEGER;
	tipoERROR VARCHAR2(100);
    VACIO_ERROR EXCEPTION;
	IDLista_NULL ListaCandidatos.ID_Lista%TYPE;

BEGIN
    paramEntrada:=P_IDLista||' - aprobada: '||P_aprobada||' - Partido: '||P_partido||' - Demarcacion: '||P_demarcacion||' - Gobierno'||P_gobierno;
	
    /* Comprobamos que ningn valor de clave primaria sea nulo, o todos los otros no sean nulos a la vez */
    IF (P_IDLista IS NULL) OR ((P_aprobada IS NULL) AND (P_partido IS NULL) AND (P_demarcacion IS NULL) AND (P_gobierno IS NULL))
    THEN
        RAISE VACIO_ERROR;
    END IF;
	
    /* Comprobamos que exista */
    BEGIN
        SELECT ID_Lista
        INTO IDLista_NULL
        FROM ListaCandidatos
        WHERE ID_Lista = P_IDLista
        AND ROWNUM = 1;
        EXCEPTION
            WHEN NO_DATA_FOUND THEN
            RAISE NO_DATA_FOUND;
    END;
    
    /* Si ha pasado las comprobaciones anteriores, actualizamos */
    UPDATE ListaCandidatos SET aprobada = P_aprobada, ID_Partido = P_partido, ID_Demarcacion = P_demarcacion, ID_Gobierno = P_gobierno WHERE ID_Lista = P_IDLista;
    RSP:= 'OK';
    COMMIT;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
EXCEPTION
    WHEN VACIO_ERROR THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL VALOR NO PUEDE SER NULO';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN NO_DATA_FOUND THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - LA LISTA NO SE ENCUENTRA - NO EXISTE';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN OTHERS THEN
    numERROR:= SQLCODE;
    tipoERROR:= SUBSTR(SQLERRM, 1, 90);
    RSP := TO_CHAR(numERROR)||' - '||tipoERROR;
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
END;
/

/************************************************************
* Procedimiento para AADIR los candidatos.
* Entrada:  numDoc, letraDoc, ID_Partido, nombre, 
*			 apellido1, apellido2, ID_Lista
* Salida: OK si se realiza con xito
*         ERROR+TIPO DE ERROR en caso contrario
* Pre-condicin: El partido DEBE existir.
* Post-condicin: la lista es dada de ALTA
************************************************************/

CREATE OR REPLACE PROCEDURE altaCandidato(
	P_numDoc IN Candidato.numDoc%TYPE,
	P_letraDoc IN Candidato.letraDoc%TYPE,
	P_partido IN Candidato.ID_Partido%TYPE,
	P_nombre IN Candidato.nombre%TYPE,
	P_apellido1 IN Candidato.apellido1%TYPE,
	P_apellido2 IN Candidato.apellido2%TYPE,
	P_lista IN Candidato.ID_Lista%TYPE,
	RSP OUT VARCHAR2) 
IS
    paramEntrada VARCHAR2(4000);
	numERROR INTEGER;
	tipoERROR VARCHAR2(100);
    VACIO_ERROR EXCEPTION;

BEGIN
    paramEntrada:='DNI: '||P_numDoc||P_letraDoc||' - Partido: '||P_partido||' - Nombre: '||P_nombre||' - Apellido1: '||P_apellido1||' - Apellido2: '||P_apellido2||' - Lista: '||P_lista;
	
    /* Comprobamos que ningn valor sea nulo, lista no se evala ya que es totalmente opcional */
    IF (P_numDoc IS NULL) OR (P_letraDoc IS NULL) OR (P_partido IS NULL) OR (P_nombre IS NULL) OR (P_apellido1 IS NULL) OR (P_apellido2 IS NULL)
    THEN
        RAISE VACIO_ERROR;
    END IF;
	  
	/*Realizamos la insercin del valor*/
    INSERT INTO Candidato VALUES (P_numDoc, P_letraDoc, P_partido, P_nombre, P_apellido1, P_apellido1, P_lista);
    RSP:= 'OK';
    COMMIT;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
EXCEPTION  
    WHEN DUP_VAL_ON_INDEX THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' -  CANDIDATO DUPLICADO / YA EXISTE';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN VACIO_ERROR THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL VALOR NO PUEDE SER NULO';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN OTHERS THEN
    numERROR:= SQLCODE;
    tipoERROR:= SUBSTR(SQLERRM, 1, 90);
    RSP := TO_CHAR(numERROR)||' - '||tipoERROR;
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
END;
/


/************************************************************
* Procedimiento para dar de BAJA un candidato
* Entrada: numDoc, letraDoc
* Salida: OK si se realiza con xito
*         ERROR+TIPO DE ERROR en caso contrario
* Pre-condicin: el candidato debe existir
* Post-condicin: el candidato se ELIMINA 
************************************************************/

CREATE OR REPLACE PROCEDURE bajaCandidato(
	P_numDoc IN Candidato.numDoc%TYPE,
	P_letraDoc IN Candidato.letraDoc%TYPE,
	RSP OUT VARCHAR2) 
IS
    paramEntrada VARCHAR2(500);
	numERROR INTEGER;
	tipoERROR VARCHAR2(100);
    VACIO_ERROR EXCEPTION;
	numDoc_NULL Candidato.numDoc%TYPE;
	letraDoc_NULL Candidato.letraDoc%TYPE;


BEGIN
    paramEntrada:=' - Identificador:'||P_numDoc;
    
    /* Comprobamos que el valor no sea nulo */
    IF (P_numDoc IS NULL)
    THEN
        RAISE VACIO_ERROR;
    END IF;
	
    /* Comprobamos que exista */
    BEGIN
        SELECT numDoc, letraDoc
        INTO numDoc_NULL, letraDoc_NULL
        FROM Candidato
        WHERE numDoc = P_numDoc
		AND letraDoc = P_letraDoc
        AND ROWNUM = 1;
        EXCEPTION
            WHEN NO_DATA_FOUND THEN
            RAISE NO_DATA_FOUND;
    END;
		
    /* Borramos si ha pasado las anteriores comprobaciones */
    DELETE FROM Candidato WHERE numDoc = P_numDoc AND letraDoc = P_letraDoc;
    RSP:= 'OK';
    COMMIT;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
EXCEPTION
    WHEN VACIO_ERROR THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL VALOR NO PUEDE SER NULO';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN NO_DATA_FOUND THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL CANDIDATO NO EXISTE';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN OTHERS THEN
    numERROR:= SQLCODE;
    tipoERROR:= SUBSTR(SQLERRM, 1, 90);
    RSP := TO_CHAR(numERROR)||' - '||tipoERROR;
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
END;
/

/************************************************************
* Procedimiento para MODIFICAR un candidato
* Entrada:  numDoc, letraDoc, ID_Partido, nombre, 
*			 apellido1, apellido2, ID_Lista
* Salida: OK si se realiza con xito
*         ERROR+TIPO DE ERROR en caso contrario
* Pre-condicin: el candidato DEBE existir 
* Post-condicin: el candidato es MODIFICADO
************************************************************/

CREATE OR REPLACE PROCEDURE modCandidato(
	P_numDoc IN Candidato.numDoc%TYPE,
	P_letraDoc IN Candidato.letraDoc%TYPE,
	P_partido IN Candidato.ID_Partido%TYPE,
	P_nombre IN Candidato.nombre%TYPE,
	P_apellido1 IN Candidato.apellido1%TYPE,
	P_apellido2 IN Candidato.apellido2%TYPE,
	P_lista IN Candidato.ID_Lista%TYPE,
	RSP OUT VARCHAR2) 
IS
    paramEntrada VARCHAR2(500);
	numERROR INTEGER;
	tipoERROR VARCHAR2(100);
    VACIO_ERROR EXCEPTION;
	numDoc_NULL Candidato.numDoc%TYPE;
	letraDoc_NULL Candidato.letraDoc%TYPE;

BEGIN
    paramEntrada:='DNI: '||P_numDoc||P_letraDoc||' - Partido: '||P_partido||' - Nombre: '||P_nombre||' - Apellido1: '||P_apellido1||' - Apellido2: '||P_apellido2||' - Lista: '||P_lista;
	
    /* Comprobamos que ningn valor de clave primaria sea nulo, o todos los otros no sean nulos a la vez */
    IF (P_numDoc IS NULL) OR (P_letraDoc IS NULL) OR( (P_partido IS NULL) AND (P_nombre IS NULL) AND (P_apellido1 IS NULL) AND (P_apellido2 IS NULL) AND (P_lista IS NULL))
    THEN
        RAISE VACIO_ERROR;
    END IF;
	
    /* Comprobamos que exista */
    BEGIN
        SELECT numDoc, letraDoc
        INTO numDoc_NULL, letraDoc_NULL
        FROM Candidato
        WHERE numDoc = P_numDoc
		AND letraDoc = P_letraDoc
        AND ROWNUM = 1;
        EXCEPTION
            WHEN NO_DATA_FOUND THEN
            RAISE NO_DATA_FOUND;
    END;
    
    /* Si ha pasado las comprobaciones anteriores, actualizamos */
    UPDATE Candidato SET ID_Partido = P_partido, nombre = P_nombre, apellido1 = P_apellido1, apellido2 = P_apellido2, ID_Lista = P_lista WHERE numDoc = P_numDoc AND letraDoc = P_letraDoc;
    RSP:= 'OK';
    COMMIT;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
EXCEPTION
    WHEN VACIO_ERROR THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL VALOR NO PUEDE SER NULO';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN NO_DATA_FOUND THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL CANDIDATO NO SE ENCUENTRA - NO EXISTE';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN OTHERS THEN
    numERROR:= SQLCODE;
    tipoERROR:= SUBSTR(SQLERRM, 1, 90);
    RSP := TO_CHAR(numERROR)||' - '||tipoERROR;
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
END;
/

/************************************************************
* Procedimiento para AADIR una Regla de Seleecin a la BD
* Entrada: ID_RS, nombreRS, descripcionRS
* Salida: OK si se realiza con xito
*         ERROR+TIPO DE ERROR en caso contrario
* Pre-condicin: la Regla de Seleccion NO debe existir 
* Post-condicin: la Regla de Seleccion es dada de ALTA
************************************************************/

CREATE OR REPLACE PROCEDURE altaReglasSeleccion(
	P_IDRS IN ReglasSeleccion.ID_RS%TYPE,
	P_nombre IN ReglasSeleccion.nombreRS%TYPE,
	P_descripcion IN ReglasSeleccion.descripcionRS%TYPE,
	RSP OUT VARCHAR2) 
IS
    paramEntrada VARCHAR2(500);
	numERROR INTEGER;
	tipoERROR VARCHAR2(100);
    VACIO_ERROR EXCEPTION;

BEGIN
    paramEntrada:= P_IDRS||'- Nombre:'||P_nombre||' - Descripcin: '||P_descripcion;
	
    /* Comprobamos que ningn valor sea nulo */
    IF (P_IDRS IS NULL) OR (P_nombre IS NULL) OR (P_descripcion IS NULL)
    THEN
        RAISE VACIO_ERROR;
    END IF;
	  
	/*Realizamos la insercin del valor*/
    INSERT INTO ReglasSeleccion VALUES (P_IDRS, P_nombre, P_descripcion);
    RSP:= 'OK';
    COMMIT;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
EXCEPTION  
    WHEN DUP_VAL_ON_INDEX THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - LA REGLA DE SELECCION EST DUPLICADA / YA EXISTE';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN VACIO_ERROR THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL VALOR NO PUEDE SER NULO';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN OTHERS THEN
    numERROR:= SQLCODE;
    tipoERROR:= SUBSTR(SQLERRM, 1, 90);
    RSP := TO_CHAR(numERROR)||' - '||tipoERROR;
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
END;
/


/************************************************************
* Procedimiento para dar de BAJA una Regla de Seleccion EN la BD
* Entrada: ID_RS
* Salida: OK si se realiza con xito
*         ERROR+TIPO DE ERROR en caso contrario
* Pre-condicin: la Regla de Seleccion DEBE existir 
* Post-condicin: la Regla de Seleccion se ELIMINA
************************************************************/

CREATE OR REPLACE PROCEDURE bajaReglasSeleccion(
	P_IDRS IN ReglasSeleccion.ID_RS%TYPE,
	RSP OUT VARCHAR2) 
IS
    paramEntrada VARCHAR2(500);
	numERROR INTEGER;
	tipoERROR VARCHAR2(100);
    VACIO_ERROR EXCEPTION;
    ID_NULL ReglasSeleccion.ID_RS%TYPE;

BEGIN
    paramEntrada:= P_IDRS;
    
    /* Comprobamos que el valor no sea nulo */
    IF (P_IDRS IS NULL) 
    THEN
        RAISE VACIO_ERROR;
    END IF;
    /* Comprobamos que exista ese Partido Politico  */
    BEGIN
        SELECT ID_RS
        INTO ID_NULL
        FROM ReglasSeleccion
        WHERE ID_RS = P_IDRS
        AND ROWNUM = 1;
        EXCEPTION
            WHEN NO_DATA_FOUND THEN
            RAISE NO_DATA_FOUND;
        END;
		
    /* Borramos si ha pasado las anteriores comprobaciones */
    DELETE FROM ReglasSeleccion WHERE ID_RS = P_IDRS;
    RSP:= 'OK';
    COMMIT;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
EXCEPTION
    WHEN VACIO_ERROR THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL VALOR NO PUEDE SER NULO';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN NO_DATA_FOUND THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - la Regla de Seleccion NO EXISTE';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN OTHERS THEN
    numERROR:= SQLCODE;
    tipoERROR:= SUBSTR(SQLERRM, 1, 90);
    RSP := TO_CHAR(numERROR)||' - '||tipoERROR;
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
END;
/

/************************************************************
* Procedimiento para MODIFICAR una Regla de Seleccion en la BD
* Entrada: ID_RS, nombreRS
* Salida: OK si se realiza con xito
*         ERROR+TIPO DE ERROR en caso contrario
* Pre-condicin: la Regla de Seleccion DEBE existir 
* Post-condicin: la Regla de Seleccion es MODIFICADO
************************************************************/

CREATE OR REPLACE PROCEDURE modReglasSeleccion(
	P_IDRS IN ReglasSeleccion.ID_RS%TYPE,
	P_nombre IN ReglasSeleccion.nombreRS%TYPE,
	P_descripcion IN ReglasSeleccion.descripcionRS%TYPE,
	RSP OUT VARCHAR2) 
IS
    paramEntrada VARCHAR2(500);
	numERROR INTEGER;
	tipoERROR VARCHAR2(100);
    VACIO_ERROR EXCEPTION;
    nombreDUPLICADO ReglasSeleccion.nombreRS%TYPE;

BEGIN
    paramEntrada:= P_IDRS||'- Nombre:'||P_nombre||' - Descripcin: '||P_descripcion;
	
    /* Comprobamos que el ID no sea nulo o bien que los otros atributos no sean nulos a la vez */
    IF (P_IDRS IS NULL) OR ((P_nombre IS NULL) AND (P_descripcion IS NULL))
    THEN
        RAISE VACIO_ERROR;
    END IF;
    
    /* Comprobamos que exista el ReglasSeleccion a modificar  */
    BEGIN
        SELECT nombreRS
        INTO nombreDUPLICADO
        FROM ReglasSeleccion
        WHERE ID_RS = P_IDRS
        AND ROWNUM = 1;
        EXCEPTION
            WHEN NO_DATA_FOUND THEN
            RAISE NO_DATA_FOUND;
        END;
    
    /* Si ha pasado las comprobaciones anteriores, actualizamos */
    UPDATE ReglasSeleccion SET nombreRS= P_nombre, descripcionRS= P_descripcion WHERE ID_RS= P_IDRS;
    RSP:= 'OK';
    COMMIT;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
EXCEPTION
    WHEN VACIO_ERROR THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL VALOR NO PUEDE SER NULO';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN NO_DATA_FOUND THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - la Regla de Seleccion NO SE ENCUENTRA - NO EXISTE';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN OTHERS THEN
    numERROR:= SQLCODE;
    tipoERROR:= SUBSTR(SQLERRM, 1, 90);
    RSP := TO_CHAR(numERROR)||' - '||tipoERROR;
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
END;
/

/************************************************************
* Procedimiento para AADIR miembros a una mesa electoral
* Entrada: numDoc, letraDoc, ID_RP, ID_Proceso, ID_RS, ID_Mesa
* 		   fechaElecciones, cargo, localizado, alegaciones, apto
* Salida: OK si se realiza con xito
*         ERROR+TIPO DE ERROR en caso contrario
* Pre-condicin: el censo debe estar creado, las reglas y las
				 mesas deben existir.
* Post-condicin: el miembro de la mesa es dado de ALTA
************************************************************/

CREATE OR REPLACE PROCEDURE altaMiembroMesa(
	P_numDoc IN MiembroMesa.numDoc%TYPE,
	P_letraDoc IN MiembroMesa.letraDoc%TYPE,
	PIDRP IN MiembroMesa.ID_RP%TYPE,
	PIDProceso IN MiembroMesa.ID_Proceso%TYPE,
	PIDRS IN MiembroMesa.ID_RS%TYPE,
	PIDMesa IN MiembroMesa.ID_Mesa%TYPE,
	P_fecha IN MiembroMesa.fechaElecciones%TYPE,
	P_cargo IN MiembroMesa.cargo%TYPE,
	P_localizado IN MiembroMesa.localizado%TYPE,
	P_alegaciones IN MiembroMesa.alegaciones%TYPE,
	P_apto IN MiembroMesa.apto%TYPE,
	RSP OUT VARCHAR2) 
IS
    paramEntrada VARCHAR2(500);
	numERROR INTEGER;
	tipoERROR VARCHAR2(100);
    VACIO_ERROR EXCEPTION;

BEGIN
    paramEntrada:= 'DNI: '||P_numDoc||P_letraDoc||' - Reglas Proceso: '||PIDRP||' - Proceso: '||PIDProceso||' - Reglas de seleccion: : '||PIDRS||
		' - Mesa: '||PIDMesa||' - fecha elecciones: '||P_fecha||' - Cargo: '||P_cargo||' - Localizado: '||P_localizado||' - Alegaciones: '||
		P_alegaciones||' - Apto: '||P_apto;
	
    /* Comprobamos que ningn valor sea nulo */
    IF (P_numDoc IS NULL) OR (P_letraDoc IS NULL) OR (PIDRP IS NULL) OR (PIDProceso IS NULL) OR (PIDRS IS NULL) OR (PIDMesa IS NULL) 
		OR (P_fecha IS NULL) OR (P_cargo IS NULL) OR (P_localizado IS NULL) OR (P_apto IS NULL) OR (P_alegaciones IS NULL) 
    THEN
        RAISE VACIO_ERROR;
    END IF;
	  
	/*Realizamos la insercin del valor*/
    INSERT INTO MiembroMesa VALUES (P_numDoc, P_letraDoc, PIDRP, PIDProceso, PIDRS, PIDMesa, P_fecha, P_cargo, P_localizado, P_alegaciones, P_apto);
    RSP:= 'OK';
    COMMIT;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
EXCEPTION  
    WHEN DUP_VAL_ON_INDEX THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL MIEMBRO EST DUPLICADO / YA EXISTE';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN VACIO_ERROR THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL VALOR NO PUEDE SER NULO';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN OTHERS THEN
    numERROR:= SQLCODE;
    tipoERROR:= SUBSTR(SQLERRM, 1, 90);
    RSP := TO_CHAR(numERROR)||' - '||tipoERROR;
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
END;
/


/************************************************************
* Procedimiento para dar de BAJA una miembro en una mesa
* Entrada: numDoc, letraDoc, ID_RP, ID_Proceso, ID_RS, ID_Mesa
* Salida: OK si se realiza con xito
*         ERROR+TIPO DE ERROR en caso contrario
* Pre-condicin: el miembro DEBE existir 
* Post-condicin: el miembro se ELIMINA de la mesa
************************************************************/

CREATE OR REPLACE PROCEDURE bajaMiembroMesa(
	P_numDoc IN MiembroMesa.numDoc%TYPE,
	P_letraDoc IN MiembroMesa.letraDoc%TYPE,
	PIDRP IN MiembroMesa.ID_RP%TYPE,
	PIDProceso IN MiembroMesa.ID_Proceso%TYPE,
	PIDRS IN MiembroMesa.ID_RS%TYPE,
	PIDMesa IN MiembroMesa.ID_Mesa%TYPE,
	RSP OUT VARCHAR2) 
IS
    paramEntrada VARCHAR2(500);
	numERROR INTEGER;
	tipoERROR VARCHAR2(100);
    VACIO_ERROR EXCEPTION;
    numDoc_NULL MiembroMesa.numDoc%TYPE;
	letraDoc_NULL MiembroMesa.letraDoc%TYPE;
	IDRP_NULL MiembroMesa.ID_RP%TYPE;
	IDProceso_NULL MiembroMesa.ID_Proceso%TYPE;
	IDRS_NULL MiembroMesa.ID_RS%TYPE;
	IDMesa_NULL MiembroMesa.ID_Mesa%TYPE;

BEGIN
    paramEntrada:= 'DNI: '||P_numDoc||P_letraDoc||' - Reglas Proceso: '||PIDRP||' - Proceso: '||PIDProceso||' - Reglas de seleccion: : '||PIDRS||
		' - Mesa: '||PIDMesa;
    
    /* Comprobamos que el valor no sea nulo */
    IF (P_numDoc IS NULL) OR (P_letraDoc IS NULL) OR (PIDRP IS NULL) OR (PIDProceso IS NULL) OR (PIDRS IS NULL) OR (PIDMesa IS NULL)
    THEN
        RAISE VACIO_ERROR;
    END IF;
    /* Comprobamos que exista */
    BEGIN
        SELECT numDoc, letraDoc, ID_RP, ID_Proceso, ID_RS, ID_Mesa
        INTO numDoc_NULL, letraDoc_NULL, IDRP_NULL, IDProceso_NULL, IDRS_NULL, IDMesa_NULL
        FROM MiembroMesa
        WHERE numDoc = P_numDoc
		AND letraDoc = P_letraDoc
		AND ID_RP = PIDRP
		AND ID_Proceso = PIDProceso
		AND ID_RS = PIDRS
		AND ID_Mesa = PIDMesa
        AND ROWNUM = 1;
        EXCEPTION
            WHEN NO_DATA_FOUND THEN
            RAISE NO_DATA_FOUND;
     END;
		
    /* Borramos si ha pasado las anteriores comprobaciones */
    DELETE FROM MiembroMesa WHERE numDoc = P_numDoc AND letraDoc = P_letraDoc AND ID_RP = PIDRP AND ID_Proceso = PIDProceso AND ID_RS = PIDRS AND ID_Mesa = PIDMesa;
    RSP:= 'OK';
    COMMIT;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
EXCEPTION
    WHEN VACIO_ERROR THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL VALOR NO PUEDE SER NULO';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN NO_DATA_FOUND THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL MIEMBRO DE LA MESA NO EXISTE';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN OTHERS THEN
    numERROR:= SQLCODE;
    tipoERROR:= SUBSTR(SQLERRM, 1, 90);
    RSP := TO_CHAR(numERROR)||' - '||tipoERROR;
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
END;
/

/************************************************************
* Procedimiento para MODIFICAR miembro de una mesa
* Entrada: numDoc, letraDoc, ID_RP, ID_Proceso, ID_RS, ID_Mesa
* 		   fechaElecciones, cargo, localizado, alegaciones, apto
* Salida: OK si se realiza con xito
*         ERROR+TIPO DE ERROR en caso contrario
* Pre-condicin: el miembro DEBE existir 
* Post-condicin: los datos del miembro son MODIFICADOS
************************************************************/

CREATE OR REPLACE PROCEDURE modMiembroMesa(
	P_numDoc IN MiembroMesa.numDoc%TYPE,
	P_letraDoc IN MiembroMesa.letraDoc%TYPE,
	PIDRP IN MiembroMesa.ID_RP%TYPE,
	PIDProceso IN MiembroMesa.ID_Proceso%TYPE,
	PIDRS IN MiembroMesa.ID_RS%TYPE,
	PIDMesa IN MiembroMesa.ID_Mesa%TYPE,
	P_fecha IN MiembroMesa.fechaElecciones%TYPE,
	P_cargo IN MiembroMesa.cargo%TYPE,
	P_localizado IN MiembroMesa.localizado%TYPE,
	P_alegaciones IN MiembroMesa.alegaciones%TYPE,
	P_apto IN MiembroMesa.apto%TYPE,
	RSP OUT VARCHAR2)  
IS
    paramEntrada VARCHAR2(500);
	numERROR INTEGER;
	tipoERROR VARCHAR2(100);
    VACIO_ERROR EXCEPTION;
    numDoc_NULL MiembroMesa.numDoc%TYPE;
	letraDoc_NULL MiembroMesa.letraDoc%TYPE;
	IDRP_NULL MiembroMesa.ID_RP%TYPE;
	IDProceso_NULL MiembroMesa.ID_Proceso%TYPE;
	IDRS_NULL MiembroMesa.ID_RS%TYPE;
	IDMesa_NULL MiembroMesa.ID_Mesa%TYPE;

BEGIN
    paramEntrada:= 'DNI: '||P_numDoc||P_letraDoc||' - Reglas Proceso: '||PIDRP||' - Proceso: '||PIDProceso||' - Reglas de seleccion: : '||PIDRS||
		' - Mesa: '||PIDMesa||' - fecha elecciones: '||P_fecha||' - Cargo: '||P_cargo||' - Localizado: '||P_localizado||' - Alegaciones: '||
		P_alegaciones||' - Apto: '||P_apto;
	
    /* Comprobamos que el ID no sea nulo o bien que los otros atributos no sean nulos a la vez */
    IF (P_numDoc IS NULL) OR (P_letraDoc IS NULL) OR (PIDRP IS NULL) OR (PIDProceso IS NULL) OR (PIDRS IS NULL) OR (PIDMesa IS NULL) 
		OR ((P_fecha IS NULL) AND (P_cargo IS NULL) AND (P_localizado IS NULL) AND (P_apto IS NULL) AND (P_alegaciones IS NULL))
    THEN
        RAISE VACIO_ERROR;
    END IF;
    
    /* Comprobamos que exista el MiembroMesa a modificar  */
    BEGIN
        SELECT numDoc, letraDoc, ID_RP, ID_Proceso, ID_RS, ID_Mesa
        INTO numDoc_NULL, letraDoc_NULL, IDRP_NULL, IDProceso_NULL, IDRS_NULL, IDMesa_NULL
        FROM MiembroMesa
        WHERE numDoc = P_numDoc
		AND letraDoc = P_letraDoc
		AND ID_RP = PIDRP
		AND ID_Proceso = PIDProceso
		AND ID_RS = PIDRS
		AND ID_Mesa = PIDMesa
        AND ROWNUM = 1;
        EXCEPTION
            WHEN NO_DATA_FOUND THEN
            RAISE NO_DATA_FOUND;
     END;
    
    /* Si ha pasado las comprobaciones anteriores, actualizamos */
    UPDATE MiembroMesa SET fechaElecciones = P_fecha, cargo = P_cargo, localizado = P_localizado, alegaciones = P_alegaciones, apto = P_apto
	WHERE numDoc = P_numDoc AND letraDoc = P_letraDoc AND ID_RP = PIDRP AND ID_Proceso = PIDProceso AND ID_RS = PIDRS AND ID_Mesa = PIDMesa;
	
    RSP:= 'OK';
    COMMIT;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
EXCEPTION
    WHEN VACIO_ERROR THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL VALOR NO PUEDE SER NULO';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN NO_DATA_FOUND THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - la Regla de Seleccion NO SE ENCUENTRA - NO EXISTE';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN OTHERS THEN
    numERROR:= SQLCODE;
    tipoERROR:= SUBSTR(SQLERRM, 1, 90);
    RSP := TO_CHAR(numERROR)||' - '||tipoERROR;
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
END;
/

/************************************************************
* Procedimiento para AADIR un voto a la BD
* Entrada: ID_Voto, modalidad, partido, tipoVoto, ID_Mesa
* Salida: OK si se realiza con xito
*         ERROR+TIPO DE ERROR en caso contrario
* Pre-condicin: el Voto NO debe existir 
* Post-condicin: el Voto es dado de ALTA
************************************************************/

CREATE OR REPLACE PROCEDURE altaVoto(
	P_IDVoto IN Voto.ID_Voto%TYPE,
	P_modalidad IN Voto.modalidad%TYPE,
	P_partido IN Voto.partido%TYPE,
	P_tipo IN Voto.tipoVoto%TYPE,
	P_IDMesa IN Voto.ID_Mesa%TYPE,
	RSP OUT VARCHAR2) 
IS
    paramEntrada VARCHAR2(500);
	numERROR INTEGER;
	tipoERROR VARCHAR2(100);
    VACIO_ERROR EXCEPTION;

BEGIN
    paramEntrada:= P_IDVoto||'- Modalidad:'||P_modalidad||' - Partido: '||P_partido||' - Tipo: '||P_tipo||' - Mesa electoral: '||P_IDMesa;
	
    /* Comprobamos que ningn valor sea nulo excepto partido que puede estar en blanco*/
    IF (P_IDVoto IS NULL) OR (P_modalidad IS NULL) OR (P_tipo IS NULL) OR (P_IDMesa IS NULL)
    THEN
        RAISE VACIO_ERROR;
    END IF;
	  
	/*Realizamos la insercin del valor*/
    INSERT INTO Voto VALUES (P_IDVoto, P_modalidad, P_partido, P_tipo, P_IDMesa);
    RSP:= 'OK';
    COMMIT;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
EXCEPTION  
    WHEN DUP_VAL_ON_INDEX THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL VOTO EST DUPLICADo / YA EXISTE';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN VACIO_ERROR THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL VALOR NO PUEDE SER NULO';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN OTHERS THEN
    numERROR:= SQLCODE;
    tipoERROR:= SUBSTR(SQLERRM, 1, 90);
    RSP := TO_CHAR(numERROR)||' - '||tipoERROR;
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
END;
/


/************************************************************
* Procedimiento para dar de BAJA un Voto en la BD
* Entrada: ID_Voto 
* Salida: OK si se realiza con xito
*         ERROR+TIPO DE ERROR en caso contrario
* Pre-condicin: el voto DEBE existir 
* Post-condicin: el voto se ELIMINA
************************************************************/

CREATE OR REPLACE PROCEDURE bajaVoto(
	P_IDVoto IN Voto.ID_Voto%TYPE,
	RSP OUT VARCHAR2) 
IS
    paramEntrada VARCHAR2(500);
	numERROR INTEGER;
	tipoERROR VARCHAR2(100);
    VACIO_ERROR EXCEPTION;
    ID_NULL Voto.ID_Voto%TYPE;

BEGIN
    paramEntrada:= P_IDVoto;
    
    /* Comprobamos que el valor no sea nulo */
    IF (P_IDVoto IS NULL) 
    THEN
        RAISE VACIO_ERROR;
    END IF;
    /* Comprobamos que exista ese Partido Politico  */
    BEGIN
        SELECT ID_Voto
        INTO ID_NULL
        FROM Voto
        WHERE ID_Voto = P_IDVoto
        AND ROWNUM = 1;
        EXCEPTION
            WHEN NO_DATA_FOUND THEN
            RAISE NO_DATA_FOUND;
    END;
		
    /* Borramos si ha pasado las anteriores comprobaciones */
    DELETE FROM Voto WHERE ID_Voto = P_IDVoto;
    RSP:= 'OK';
    COMMIT;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
EXCEPTION
    WHEN VACIO_ERROR THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL VALOR NO PUEDE SER NULO';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN NO_DATA_FOUND THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL VOTO NO EXISTE';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN OTHERS THEN
    numERROR:= SQLCODE;
    tipoERROR:= SUBSTR(SQLERRM, 1, 90);
    RSP := TO_CHAR(numERROR)||' - '||tipoERROR;
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
END;
/

/************************************************************
* Procedimiento para MODIFICAR un voto en la BD
* Entrada: ID_Voto, modalidad, partido, tipoVoto, ID_Mesa
* Salida: OK si se realiza con xito
*         ERROR+TIPO DE ERROR en caso contrario
* Pre-condicin: el voto DEBE existir 
* Post-condicin: el voto es MODIFICADO
************************************************************/

CREATE OR REPLACE PROCEDURE modVoto(
	P_IDVoto IN Voto.ID_Voto%TYPE,
	P_modalidad IN Voto.modalidad%TYPE,
	P_partido IN Voto.partido%TYPE,
	P_tipo IN Voto.tipoVoto%TYPE,
	P_IDMesa IN Voto.ID_Mesa%TYPE,
	RSP OUT VARCHAR2) 
IS
    paramEntrada VARCHAR2(500);
	numERROR INTEGER;
	tipoERROR VARCHAR2(100);
    VACIO_ERROR EXCEPTION;
    ID_NULL Voto.ID_Voto%TYPE;

BEGIN
    paramEntrada:= P_IDVoto||'- Modalidad:'||P_modalidad||' - Partido: '||P_partido||' - Tipo: '||P_tipo||' - Mesa electoral: '||P_IDMesa;
	
    /* Comprobamos que el ID no sea nulo o bien que los otros atributos no sean nulos a la vez */
    IF (P_IDVoto IS NULL) OR ((P_modalidad IS NULL) AND (P_tipo IS NULL) AND (P_IDMesa IS NULL))
    THEN
        RAISE VACIO_ERROR;
    END IF;
    
    /* Comprobamos que exista el Voto a modificar  */
    BEGIN
        SELECT ID_Voto
        INTO ID_NULL
        FROM Voto
        WHERE ID_Voto = P_IDVoto
        AND ROWNUM = 1;
        EXCEPTION
            WHEN NO_DATA_FOUND THEN
            RAISE NO_DATA_FOUND;
    END;
    
    /* Si ha pasado las comprobaciones anteriores, actualizamos */
    UPDATE Voto SET modalidad= P_modalidad, partido= P_partido, tipoVoto=P_tipo, ID_Mesa=P_IDMesa  WHERE ID_Voto= P_IDVoto;
    RSP:= 'OK';
    COMMIT;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
EXCEPTION
    WHEN VACIO_ERROR THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL VALOR NO PUEDE SER NULO';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN NO_DATA_FOUND THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL VOTO NO SE ENCUENTRA - NO EXISTE';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN OTHERS THEN
    numERROR:= SQLCODE;
    tipoERROR:= SUBSTR(SQLERRM, 1, 90);
    RSP := TO_CHAR(numERROR)||' - '||tipoERROR;
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
END;
/


/************************************************************
* Procedimiento para AADIR un Recuento a la BD
* Entrada: ID_Recuento, Nulos, enBlanco, ID_Mesa
* Salida: OK si se realiza con xito
*         ERROR+TIPO DE ERROR en caso contrario
* Pre-condicin: el Recuento NO debe existir 
* Post-condicin: el Recuento es dado de ALTA
************************************************************/

CREATE OR REPLACE PROCEDURE altaRecuento(
	P_IDRecuento IN Recuento.ID_Recuento%TYPE,
	P_nulos IN Recuento.Nulos%TYPE,
	P_enBlanco IN Recuento.enBlanco%TYPE,
	P_IDMesa IN Recuento.ID_Mesa%TYPE,
	RSP OUT VARCHAR2) 
IS
    paramEntrada VARCHAR2(500);
	numERROR INTEGER;
	tipoERROR VARCHAR2(100);
    VACIO_ERROR EXCEPTION;

BEGIN
    paramEntrada:= P_IDRecuento||'- Nulos:'||P_Nulos||' - enBlanco: '||P_enBlanco||' - Mesa electoral: '||P_IDMesa;
	
    /* Comprobamos que ningn valor sea nulo*/
    IF (P_IDRecuento IS NULL) OR (P_Nulos IS NULL) OR (P_enBlanco IS NULL) OR (P_IDMesa IS NULL)
    THEN
        RAISE VACIO_ERROR;
    END IF;
	  
	/*Realizamos la insercin del valor*/
    INSERT INTO Recuento VALUES (P_IDRecuento, P_Nulos, P_enBlanco, P_IDMesa);
    RSP:= 'OK';
    COMMIT;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
EXCEPTION  
    WHEN DUP_VAL_ON_INDEX THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL RECUENTO EST DUPLICADo / YA EXISTE';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN VACIO_ERROR THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL VALOR NO PUEDE SER NULO';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN OTHERS THEN
    numERROR:= SQLCODE;
    tipoERROR:= SUBSTR(SQLERRM, 1, 90);
    RSP := TO_CHAR(numERROR)||' - '||tipoERROR;
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
END;
/


/************************************************************
* Procedimiento para dar de BAJA un Recuento en la BD
* Entrada: ID_Recuento 
* Salida: OK si se realiza con xito
*         ERROR+TIPO DE ERROR en caso contrario
* Pre-condicin: el Recuento DEBE existir 
* Post-condicin: el Recuento se ELIMINA
************************************************************/

CREATE OR REPLACE PROCEDURE bajaRecuento(
	P_IDRecuento IN Recuento.ID_Recuento%TYPE,
	RSP OUT VARCHAR2) 
IS
    paramEntrada VARCHAR2(500);
	numERROR INTEGER;
	tipoERROR VARCHAR2(100);
    VACIO_ERROR EXCEPTION;
    ID_NULL Recuento.ID_Recuento%TYPE;

BEGIN
    paramEntrada:= P_IDRecuento;
    
    /* Comprobamos que el valor no sea nulo */
    IF (P_IDRecuento IS NULL) 
    THEN
        RAISE VACIO_ERROR;
    END IF;
    /* Comprobamos que exista ese enBlanco Politico  */
    BEGIN
        SELECT ID_Recuento
        INTO ID_NULL
        FROM Recuento
        WHERE ID_Recuento = P_IDRecuento
        AND ROWNUM = 1;
        EXCEPTION
            WHEN NO_DATA_FOUND THEN
            RAISE NO_DATA_FOUND;
    END;
		
    /* Borramos si ha pasado las anteriores comprobaciones */
    DELETE FROM Recuento WHERE ID_Recuento = P_IDRecuento;
    RSP:= 'OK';
    COMMIT;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
EXCEPTION
    WHEN VACIO_ERROR THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL VALOR NO PUEDE SER NULO';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN NO_DATA_FOUND THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL Recuento NO EXISTE';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN OTHERS THEN
    numERROR:= SQLCODE;
    tipoERROR:= SUBSTR(SQLERRM, 1, 90);
    RSP := TO_CHAR(numERROR)||' - '||tipoERROR;
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
END;
/

/************************************************************
* Procedimiento para MODIFICAR un Recuento en la BD
* Entrada: ID_Recuento, Nulos, enBlanco, ID_Mesa
* Salida: OK si se realiza con xito
*         ERROR+TIPO DE ERROR en caso contrario
* Pre-condicin: el Recuento DEBE existir 
* Post-condicin: el Recuento es MODIFICADO
************************************************************/

CREATE OR REPLACE PROCEDURE modRecuento(
	P_IDRecuento IN Recuento.ID_Recuento%TYPE,
	P_nulos IN Recuento.Nulos%TYPE,
	P_enBlanco IN Recuento.enBlanco%TYPE,
	P_IDMesa IN Recuento.ID_Mesa%TYPE,
	RSP OUT VARCHAR2) 
IS
    paramEntrada VARCHAR2(500);
	numERROR INTEGER;
	tipoERROR VARCHAR2(100);
    VACIO_ERROR EXCEPTION;
    ID_NULL Recuento.ID_Recuento%TYPE;

BEGIN
    paramEntrada:= P_IDRecuento||'- Nulos:'||P_Nulos||' - enBlanco: '||P_enBlanco||' - Mesa electoral: '||P_IDMesa;
	
    /* Comprobamos que el ID no sea nulo o bien que los otros atributos no sean nulos a la vez */
    IF (P_IDRecuento IS NULL) OR ((P_Nulos IS NULL) AND (P_enBlanco IS NULL) AND (P_IDMesa IS NULL))
    THEN
        RAISE VACIO_ERROR;
    END IF;
    
    /* Comprobamos que exista el Recuento a modificar  */
    BEGIN
        SELECT ID_Recuento
        INTO ID_NULL
        FROM Recuento
        WHERE ID_Recuento = P_IDRecuento
        AND ROWNUM = 1;
        EXCEPTION
            WHEN NO_DATA_FOUND THEN
            RAISE NO_DATA_FOUND;
    END;
    
    /* Si ha pasado las comprobaciones anteriores, actualizamos */
    UPDATE Recuento SET Nulos= P_Nulos, enBlanco= P_enBlanco, ID_Mesa=P_IDMesa  WHERE ID_Recuento= P_IDRecuento;
    RSP:= 'OK';
    COMMIT;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
EXCEPTION
    WHEN VACIO_ERROR THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL VALOR NO PUEDE SER NULO';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN NO_DATA_FOUND THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL RECUENTO NO SE ENCUENTRA - NO EXISTE';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN OTHERS THEN
    numERROR:= SQLCODE;
    tipoERROR:= SUBSTR(SQLERRM, 1, 90);
    RSP := TO_CHAR(numERROR)||' - '||tipoERROR;
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
END;
/


/************************************************************
* Procedimiento para AADIR un Desglose a la BD
* Entrada: ID_Recuento, ID_Desglose, nombrePartido, totalVotos
* Salida: OK si se realiza con xito
*         ERROR+TIPO DE ERROR en caso contrario
* Pre-condicin: el Desglose NO debe existir 
* Post-condicin: el Desglose es dado de ALTA
************************************************************/

CREATE OR REPLACE PROCEDURE altaDesglose(
	P_IDRecuento IN Desglose.ID_Recuento%TYPE,
	P_IDDesglose IN Desglose.ID_Desglose%TYPE,
	P_nombrePartido IN Desglose.nombrePartido%TYPE,
	P_total IN Desglose.totalVotos%TYPE,
	RSP OUT VARCHAR2) 
IS
    paramEntrada VARCHAR2(500);
	numERROR INTEGER;
	tipoERROR VARCHAR2(100);
    VACIO_ERROR EXCEPTION;

BEGIN
    paramEntrada:= P_IDRecuento||P_IDDesglose||' - Partido: '||P_nombrePartido||' - Total Votos: '||P_total;
	
    /* Comprobamos que ningn valor sea nulo*/
    IF (P_IDRecuento IS NULL) OR (P_IDDesglose IS NULL) OR (P_nombrePartido IS NULL) OR (P_total IS NULL)
    THEN
        RAISE VACIO_ERROR;
    END IF;
	  
	/*Realizamos la insercin del valor*/
    INSERT INTO Desglose VALUES (P_IDRecuento, P_IDDesglose, P_nombrePartido, P_total);
    RSP:= 'OK';
    COMMIT;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
EXCEPTION  
    WHEN DUP_VAL_ON_INDEX THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL DESGLOSE EST DUPLICADo / YA EXISTE';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN VACIO_ERROR THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL VALOR NO PUEDE SER NULO';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN OTHERS THEN
    numERROR:= SQLCODE;
    tipoERROR:= SUBSTR(SQLERRM, 1, 90);
    RSP := TO_CHAR(numERROR)||' - '||tipoERROR;
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
END;
/


/************************************************************
* Procedimiento para dar de BAJA un Desglose en la BD
* Entrada: ID_Recuento, ID_Desglose 
* Salida: OK si se realiza con xito
*         ERROR+TIPO DE ERROR en caso contrario
* Pre-condicin: el Desglose DEBE existir 
* Post-condicin: el Desglose se ELIMINA
************************************************************/

CREATE OR REPLACE PROCEDURE bajaDesglose(
	P_IDRecuento IN Desglose.ID_Recuento%TYPE,
	P_IDDesglose IN Desglose.ID_Desglose%TYPE,
	RSP OUT VARCHAR2) 
IS
    paramEntrada VARCHAR2(500);
	numERROR INTEGER;
	tipoERROR VARCHAR2(100);
    VACIO_ERROR EXCEPTION;
	recuento_NULL Desglose.ID_Recuento%TYPE;
    desglose_NULL Desglose.ID_Desglose%TYPE;

BEGIN
    paramEntrada:= P_IDRecuento||P_IDDesglose;
    
    /* Comprobamos que el valor no sea nulo */
    IF (P_IDRecuento IS NULL) OR (P_IDDesglose IS NULL) 
    THEN
        RAISE VACIO_ERROR;
    END IF;
    /* Comprobamos que exista ese nombrePartido Politico  */
    BEGIN
        SELECT ID_Recuento, ID_Desglose
        INTO recuento_NULL, desglose_NULL
        FROM Desglose
        WHERE ID_Desglose = P_IDDesglose
		AND ID_Recuento = P_IDRecuento
        AND ROWNUM = 1;
        EXCEPTION
            WHEN NO_DATA_FOUND THEN
            RAISE NO_DATA_FOUND;
    END;
		
    /* Borramos si ha pasado las anteriores comprobaciones */
    DELETE FROM Desglose WHERE ID_Recuento = P_IDRecuento AND ID_Desglose = P_IDDesglose;
    RSP:= 'OK';
    COMMIT;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
EXCEPTION
    WHEN VACIO_ERROR THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL VALOR NO PUEDE SER NULO';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN NO_DATA_FOUND THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL DESGLOSE NO EXISTE';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN OTHERS THEN
    numERROR:= SQLCODE;
    tipoERROR:= SUBSTR(SQLERRM, 1, 90);
    RSP := TO_CHAR(numERROR)||' - '||tipoERROR;
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
END;
/

/************************************************************
* Procedimiento para MODIFICAR un Desglose en la BD
* Entrada: ID_Recuento, ID_Desglose, nombrePartido, totalVotos
* Salida: OK si se realiza con xito
*         ERROR+TIPO DE ERROR en caso contrario
* Pre-condicin: el Desglose DEBE existir 
* Post-condicin: el Desglose es MODIFICADO
************************************************************/

CREATE OR REPLACE PROCEDURE modDesglose(
	P_IDRecuento IN Desglose.ID_Recuento%TYPE,
	P_IDDesglose IN Desglose.ID_Desglose%TYPE,
	P_nombrePartido IN Desglose.nombrePartido%TYPE,
	P_total IN Desglose.totalVotos%TYPE,
	RSP OUT VARCHAR2) 
IS
    paramEntrada VARCHAR2(500);
	numERROR INTEGER;
	tipoERROR VARCHAR2(100);
    VACIO_ERROR EXCEPTION;
	recuento_NULL Desglose.ID_Recuento%TYPE;
    desglose_NULL Desglose.ID_Desglose%TYPE;

BEGIN
    paramEntrada:= P_IDRecuento||P_IDDesglose;
    
    /* Comprobamos que el valor no sea nulo */
    IF (P_IDRecuento IS NULL) OR (P_IDDesglose IS NULL) OR ((P_nombrePartido IS NULL) AND (P_total IS NULL))
    THEN
        RAISE VACIO_ERROR;
    END IF;
    /* Comprobamos que exista ese nombrePartido Politico  */
    BEGIN
        SELECT ID_Recuento, ID_Desglose
        INTO recuento_NULL, desglose_NULL
        FROM Desglose
        WHERE ID_Desglose = P_IDDesglose
		AND ID_Recuento = P_IDRecuento
        AND ROWNUM = 1;
        EXCEPTION
            WHEN NO_DATA_FOUND THEN
            RAISE NO_DATA_FOUND;
    END;
    
    /* Si ha pasado las comprobaciones anteriores, actualizamos */
    UPDATE Desglose SET nombrePartido= P_nombrePartido, totalVotos=P_total  WHERE ID_Recuento= P_IDRecuento AND ID_Desglose= P_IDDesglose;
    RSP:= 'OK';
    COMMIT;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
EXCEPTION
    WHEN VACIO_ERROR THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL VALOR NO PUEDE SER NULO';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN NO_DATA_FOUND THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL DESGLOSE NO SE ENCUENTRA - NO EXISTE';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN OTHERS THEN
    numERROR:= SQLCODE;
    tipoERROR:= SUBSTR(SQLERRM, 1, 90);
    RSP := TO_CHAR(numERROR)||' - '||tipoERROR;
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
END;
/

/************************************************************
* Procedimiento para AADIR un Resultado a la BD
* Entrada: ID_Demarcacion, ID_Recuento, ID_Metodo, fechaElecciones
* Salida: OK si se realiza con xito
*         ERROR+TIPO DE ERROR en caso contrario
* Pre-condicin: el Resultado NO debe existir 
* Post-condicin: el Resultado es dado de ALTA
************************************************************/

CREATE OR REPLACE PROCEDURE altaResultado(
	P_IDDemarcacion IN Resultado.ID_Demarcacion%TYPE,
	P_IDRecuento IN Resultado.ID_Recuento%TYPE,
	P_IDMetodo IN Resultado.ID_Metodo%TYPE,
	P_fecha IN Resultado.fechaElecciones%TYPE,
	RSP OUT VARCHAR2) 
IS
    paramEntrada VARCHAR2(500);
	numERROR INTEGER;
	tipoERROR VARCHAR2(100);
    VACIO_ERROR EXCEPTION;

BEGIN
    paramEntrada:= 'Demarcacion: '||P_IDDemarcacion||' - Recuento: '||P_IDRecuento||' - Mtodo: '||P_IDMetodo||' - Fecha Elecciones: '||P_fecha;
	
    /* Comprobamos que ningn valor sea nulo*/
    IF (P_IDDemarcacion IS NULL) OR (P_IDRecuento IS NULL) OR (P_IDMetodo IS NULL) OR (P_fecha IS NULL)
    THEN
        RAISE VACIO_ERROR;
    END IF;
	  
	/*Realizamos la insercin del valor*/
    INSERT INTO Resultado VALUES (P_IDDemarcacion, P_IDRecuento, P_IDMetodo, P_fecha);
    RSP:= 'OK';
    COMMIT;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
EXCEPTION  
    WHEN DUP_VAL_ON_INDEX THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL RESULTADO EST DUPLICADO / YA EXISTE';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN VACIO_ERROR THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL VALOR NO PUEDE SER NULO';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN OTHERS THEN
    numERROR:= SQLCODE;
    tipoERROR:= SUBSTR(SQLERRM, 1, 90);
    RSP := TO_CHAR(numERROR)||' - '||tipoERROR;
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
END;
/


/************************************************************
* Procedimiento para dar de BAJA un Resultado en la BD
* Entrada: ID_Demarcacion, ID_Recuento, ID_Metodo
* Salida: OK si se realiza con xito
*         ERROR+TIPO DE ERROR en caso contrario
* Pre-condicin: el Resultado DEBE existir 
* Post-condicin: el Resultado se ELIMINA
************************************************************/

CREATE OR REPLACE PROCEDURE bajaResultado(
	P_IDDemarcacion IN Resultado.ID_Demarcacion%TYPE,
	P_IDRecuento IN Resultado.ID_Recuento%TYPE,
	P_IDMetodo IN Resultado.ID_Metodo%TYPE,
	RSP OUT VARCHAR2) 
IS
    paramEntrada VARCHAR2(500);
	numERROR INTEGER;
	tipoERROR VARCHAR2(100);
    VACIO_ERROR EXCEPTION;
	demarcacion_NULL Resultado.ID_Demarcacion%TYPE;
    recuento_NULL Resultado.ID_Recuento%TYPE;
	metodo_NULL Resultado.ID_Metodo%TYPE;

BEGIN
    paramEntrada:= 'Demarcacion: '||P_IDDemarcacion||' - Recuento: '||P_IDRecuento||' - Mtodo: '||P_IDMetodo;
    
    /* Comprobamos que el valor no sea nulo */
    IF (P_IDDemarcacion IS NULL) OR (P_IDRecuento IS NULL) OR (P_IDMetodo IS NULL)
    THEN
        RAISE VACIO_ERROR;
    END IF;
    /* Comprobamos que exista ese ID_Metodo Politico  */
    BEGIN
        SELECT ID_Demarcacion, ID_Recuento, ID_Metodo
        INTO demarcacion_NULL, recuento_NULL, metodo_NULL
        FROM Resultado
        WHERE ID_Recuento = P_IDRecuento
		AND ID_Demarcacion = P_IDDemarcacion
		AND ID_Metodo = P_IDMetodo
        AND ROWNUM = 1;
        EXCEPTION
            WHEN NO_DATA_FOUND THEN
            RAISE NO_DATA_FOUND;
    END;
		
    /* Borramos si ha pasado las anteriores comprobaciones */
    DELETE FROM Resultado WHERE ID_Demarcacion = P_IDDemarcacion AND ID_Recuento = P_IDRecuento AND ID_Metodo = P_IDMetodo;
    RSP:= 'OK';
    COMMIT;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
EXCEPTION
    WHEN VACIO_ERROR THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL VALOR NO PUEDE SER NULO';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN NO_DATA_FOUND THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL RESULTADO NO EXISTE';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN OTHERS THEN
    numERROR:= SQLCODE;
    tipoERROR:= SUBSTR(SQLERRM, 1, 90);
    RSP := TO_CHAR(numERROR)||' - '||tipoERROR;
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
END;
/

/************************************************************
* Procedimiento para MODIFICAR un Resultado en la BD
* Entrada: ID_Demarcacion, ID_Recuento, ID_Metodo, fechaElecciones
* Salida: OK si se realiza con xito
*         ERROR+TIPO DE ERROR en caso contrario
* Pre-condicin: el Resultado DEBE existir 
* Post-condicin: el Resultado es MODIFICADO
************************************************************/

CREATE OR REPLACE PROCEDURE modResultado(
	P_IDDemarcacion IN Resultado.ID_Demarcacion%TYPE,
	P_IDRecuento IN Resultado.ID_Recuento%TYPE,
	P_IDMetodo IN Resultado.ID_Metodo%TYPE,
	P_fecha IN Resultado.fechaElecciones%TYPE,
	RSP OUT VARCHAR2) 
IS
    paramEntrada VARCHAR2(500);
	numERROR INTEGER;
	tipoERROR VARCHAR2(100);
    VACIO_ERROR EXCEPTION;
	demarcacion_NULL Resultado.ID_Demarcacion%TYPE;
    recuento_NULL Resultado.ID_Recuento%TYPE;
	metodo_NULL Resultado.ID_Metodo%TYPE;

BEGIN
    paramEntrada:= 'Demarcacion: '||P_IDDemarcacion||' - Recuento: '||P_IDRecuento||' - Mtodo: '||P_IDMetodo||' - Fecha Elecciones: '||P_fecha;
    
    /* Comprobamos que el valor no sea nulo */
    IF (P_IDDemarcacion IS NULL) OR (P_IDRecuento IS NULL) OR (P_IDMetodo IS NULL) OR (P_fecha IS NULL)
    THEN
        RAISE VACIO_ERROR;
    END IF;
    /* Comprobamos que exista ese ID_Metodo Politico  */
    BEGIN
        SELECT ID_Demarcacion, ID_Recuento, ID_Metodo
        INTO demarcacion_NULL, recuento_NULL, metodo_NULL
        FROM Resultado
        WHERE ID_Recuento = P_IDRecuento
		AND ID_Demarcacion = P_IDDemarcacion
		AND ID_Metodo = P_IDMetodo
        AND ROWNUM = 1;
        EXCEPTION
            WHEN NO_DATA_FOUND THEN
            RAISE NO_DATA_FOUND;
    END;
    
    /* Si ha pasado las comprobaciones anteriores, actualizamos */
    UPDATE Resultado SET fechaElecciones=P_fecha  WHERE ID_Demarcacion= P_IDDemarcacion AND ID_Recuento= P_IDRecuento AND ID_Metodo= P_IDMetodo;
    RSP:= 'OK';
    COMMIT;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
EXCEPTION
    WHEN VACIO_ERROR THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL VALOR NO PUEDE SER NULO';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN NO_DATA_FOUND THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL RESULTADO NO SE ENCUENTRA - NO EXISTE';
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
    
    WHEN OTHERS THEN
    numERROR:= SQLCODE;
    tipoERROR:= SUBSTR(SQLERRM, 1, 90);
    RSP := TO_CHAR(numERROR)||' - '||tipoERROR;
    ROLLBACK;
    INSERT INTO LogProcedimientos VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, RSP);
    COMMIT;
END;
/

